#include <math.h>
#include <qdir.h>
#include <qregexp.h>
#include "cfg.h"
#include "tools.h"
#include "trbconst.h"
#include "trberror.h"
#include "trbtime.h"
#include "messages.h"
#include "color.h"
#include "readmass.h"
#include "success.h"

static const char* ENTITY_CHAN[] = {"A","B","C","D"};
static int nEntityChan = 4;

static const char* ENTITY_SHFT[] = {"AS","BS","CS","DS"};
static int nEntityShft = 4;

static const char* ENTITY_CROS[] = {"AB","AC","AD","BC","BD","CD"};
static int nEntityCros = 6;

static const char* ENTITY_CROS_SHFT[] = {"ABS","ACS","ADS","BCS",
						"BDS","CDS"};
static int nEntityCrosShft = 6;

static const char* ENTITY_INTEGRAL[] = {"SEE","FSEE","HEFF","FHEFF",
				"ISOPL","M0","M2","TAU"};
static int nEntityIntegral = 8;

static const char* SHOW[] = {"G","B","F","N","GB","GF","BG","FG" };
static int nShow = 8;

static QString warningMess;

extern CFGTree* tree;
extern CFGTree* devicetree;

const char* getColor(QString& colorName) {
	const char* color;
	QString str;
	for(int i=0; i<Ncolors; i++) {
		str = JCOLORS[i];
		if(colorName.upper() == str.upper()) {
			color = JCOLORS[i];
			return color;
		}
	}
	warningMess = "Invalid Color " + colorName;
	showStrongMessage(warningMess);
	color = "white";
	return color;
}

const char* getShow(QString& showName) {
	const char* show;
	for(int i=0; i<nShow; i++) {
		if(showName.upper() == SHOW[i]) {
			show = SHOW[i];
			return show;
		}
	}
	warningMess = "Invalid Show parameter " + showName;
	showStrongMessage(warningMess);
	show = "N";
	return show;
}

const char* getEntity(const char* what, QString& entName) {
	const char* entity;
	if(strcmp(what,"INTEGRAL") == 0) {
		for(int i=0; i<nEntityIntegral; i++) {
			if(entName.upper() == ENTITY_INTEGRAL[i]) {
				entity = ENTITY_INTEGRAL[i];
				return entity;
			}
		}
		warningMess = "Invalid entity " + entName + " in " + what;
		showStrongMessage(warningMess);
		return NULL;
	}
	for(int i=0; i<nEntityChan; i++) {
		if(entName.upper() == ENTITY_CHAN[i]) {
			entity = ENTITY_CHAN[i];
			return entity;
		}
	}
	if(strcmp(what,"SCAN") == 0) {
		warningMess = "Invalid entity " + entName + " in " + what;
		showStrongMessage(warningMess);
		return NULL;
	}

	for(int i=0; i<nEntityShft; i++) {
		if(entName.upper() == ENTITY_SHFT[i]) {
			entity = ENTITY_SHFT[i];
			return entity;
		}
	}
	if(strcmp(what,"INDEX") == 0) {
		for(int i=0; i<nEntityCros; i++) {
			if(entName.upper() == ENTITY_CROS[i]) {
				entity = ENTITY_CROS[i];
				return entity;
			}
		}
		for(int i=0; i<nEntityCrosShft; i++) {
			if(entName.upper() == ENTITY_CROS_SHFT[i]) {
				entity = ENTITY_CROS_SHFT[i];
				return entity;
			}
		}
	}
	warningMess = "Invalid entity " + entName + " in " + what;
	showStrongMessage(warningMess);
	return NULL;
}

const char* getCrosIdx(const char* entity) {
	QString str = entity;

	int pos = str.find("S",0,false);
	if(pos != -1) str.remove(pos,1);
	for(int i=0; i<nEntityCros; i++) {
		if(str.upper() == ENTITY_CROS[i]) {
			const char* ret = ENTITY_CROS[i];
			return ret;
		}
	}
	warningMess = "Invalid cross entity " + QString(entity);
	showStrongMessage(warningMess);
	return NULL;
}

CFGTree *getTree(const char* cfgName) {
	CFGTree *p_tree = tree;
	if(strcmp(cfgName,cfgFile) == 0) p_tree = tree;
	else if(strcmp(cfgName,deviceCfgFile) == 0) p_tree = devicetree;
	else {
		warningMess = "Invalid cfgName \"";
		warningMess += cfgName;
		warningMess += "\"";
		showFatalError(warningMess);
//		exit(1);
	}
	return(p_tree);
}

CFGLeaf *getLeaf(const char* cfgName,
			const char* name1,
			const char* name2,
			const char* name3) {

	jdebug("getLeaf");
	bool isTreeChanged = false;
	CFGTree* p_tree = getTree(cfgName);
	CFGLeaf *leaf;
//RETRY:
	if(name3) leaf = p_tree->findBranch(name1)->findBranch(name2)
		->findLeaf(name3);
	else leaf = p_tree->findBranch(name1)->findLeaf(name2);

	QString* data = leaf->getData();
	if(data->upper() == "EMPTY") {

		return leaf;

		if(!isTreeChanged) {
//			if(QString(cfgName) == cfgFile) cfgName = deviceCfgFile;
//			else cfgName = cfgFile;
//			p_tree = getTree(cfgName);
//			isTreeChanged = true;
//			goto RETRY;
		} else {
				if(name3)
					warningMess.sprintf("Invalid path: %s\\%s\\%s\\%s!",
						cfgName,name1,name2,name3);
				else
					warningMess.sprintf("Invalid path: %s\\%s\\%s!",
						cfgName,name1,name2);

//				showFatalError(warningMess);
//				exit(1);
//			} else return NULL;
		}
	}
	return leaf;
}

CFGLeaf *getLeaf(const char* cfgName,
			const QString& pathName) {
	QString name1 = NULL;
	QString name2 = NULL;
	QString name3 = NULL;

	int pos0 = 0;
	int pos1 = pathName.find("\\",pos0);
	name1 = pathName.mid(pos0,pos1-pos0);
	pos0 = pos1+1;
	pos1 = pathName.find("\\",pos0);
	if(pos1 == -1) {
		name2 = pathName.right(pathName.length()-pos0);
	} else {
		name2 = pathName.mid(pos0,pos1-pos0);
		pos0 = pos1+1;
		name3 = pathName.right(pathName.length()-pos0);
	}
	CFGLeaf* leaf = getLeaf(cfgName,name1.data(),name2.data(),name3.data());
	return leaf;
}

bool isLeafChanged(const char* cfgName, const char* name1,
			const char* name2,
			const char* name3) {
	jdebug("isLeafChanged");
	CFGLeaf *leaf = getLeaf(cfgName, name1, name2, name3);
	return(leaf->isChanged());
}

void setLeafData(const char* cfgName, const char* section,
			const char* subsection, const char* key,
			QString& newData,
			FILE *out_f) {

	jdebug("setLeafData");
	CFGLeaf *leaf = getLeaf(cfgName, section, subsection, key);
	if(leaf && strcasecmp(leaf->getKey(),"Empty") != 0) {
		CFGTree* p_tree = getTree(cfgName);
		leaf->replaceData(newData);
		p_tree->setChanged(1);
		leaf->setChanged(true);
	}
	if(out_f) {
		QString pname;
		pname.sprintf("%s\\%s\\%s",
			section,subsection,key);
		fprintf(out_f,"%s %s=%s\n",
			getStamp("P ",trb_tm::currentUTDateTime()),
			pname.data(),
			newData.data());
		pname = pname.upper();
		rdm::add2map(pname.data(), newData.data());
	}
}

void setLeafData(const QString& pathName, const QString& newData,
			FILE* out_f,
			const char* cfgName) {

	CFGLeaf *leaf = getLeaf(cfgName, pathName);
	if(leaf && strcasecmp(leaf->getKey(),"Empty") != 0) {
		CFGTree* p_tree = getTree(cfgName);
		leaf->replaceData(newData);
		p_tree->setChanged(1);
		leaf->setChanged(true);
	}
	if(out_f) {
		fprintf(out_f,"%s %s=%s\n",
			getStamp("P ",trb_tm::currentUTDateTime()),
			pathName.data(),
			newData.data());
		rdm::add2map(pathName.data(), newData.data());
	}
	return;
}

QString* getLeafData(bool& changeble, const char* cfgName,
			const char* name1,
			const char* name2, const char* name3,
			FILE *out_f) {

	CFGLeaf *leaf = getLeaf(cfgName, name1, name2, name3);
	changeble = leaf->isChangeEnabled();
	QString* data = leaf->getData();
/*
	if(leaf->isChanged()) {
		if(name3)
			jdebug("------ Leaf %s is changed!!!",name3);
		else
			jdebug("------ Leaf %s is changed!!!",name2);

	}
*/
	if(out_f) {
		QString qpname = name1 + QString("\\") + name2;
		if(name3) qpname += QString("\\") + name3;
		QString out = rdm::getvalue(qpname.upper().data()).c_str();
//		out.replace(QRegExp("\n"),"");

		if(out != *data) {
			fprintf(out_f,"%s %s\\%s",
				getStamp("P ",trb_tm::currentUTDateTime()),
				name1,name2);
			if(name3) fprintf(out_f,"\\%s",name3);
			fprintf(out_f,"=%s\n",data->data());

			rdm::add2map(qpname.upper().data(),
						data->data());
		}
	}
	leaf->setChanged(false);
	return data;
}

int leafDataToStr(QString& str, bool isFirst,
			const char* cfgName, const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;
	str = p_str->data();
	return SUCCESS;
}

int leafDataToStr(QString** p_p_str, bool isFirst,
			const char* cfgName, const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;
	*p_p_str = p_str;
	return SUCCESS;
}

int leafDataToStr(const char** str, bool isFirst,
			const char* cfgName, const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;
	*str = p_str->data();
	return SUCCESS;
}

int strToInt(const QString* str, const QString& message) {
	bool ok;
	int pos;
	int value = 0;
	if((pos=str->find("x",0,false)) != -1) {
		value = str->right(str->length()-pos-1).toInt(&ok,16);
	} else value = str->toInt(&ok,10);

	if(!ok) {
//		QString warn = message;
//		warn.prepend("Invalid input: ");			
//		showStrongMessage(warn);
		return 0;
	}
	return value;
}

int strToInt(const QString* str, const QString& message,
				int min, int max) {
	bool ok;
	int pos;
	int value = 0;
	if((pos=str->find("x",0,false)) != -1) {
		value = str->right(str->length()-pos-1).toInt(&ok,16);
	} else value = str->toInt(&ok,10);

	if(ok && (value >= min) && (value <= max)) {
		return value;
	}

	QString warn = "Invalid input: ";
	warn += message;
	if(value < min) {
		QString warn_add;
		warn_add.sprintf("\n%d<%d!", value, min);
		warn += warn_add;
		showStrongMessage(warn);
	}
	if(value > max) {
		QString warn_add;
		warn_add.sprintf("\n%d>%d!", value, max);
		warn += warn_add;
		showStrongMessage(warn);
	}
//	showStrongMessage(warn);
	return 0;
}

int leafDataToChar(char& value, bool isFirst, const char* cfgName, const char* section, 
			const char* subsection, const char* key,
			FILE* out_f) {

	int value_int;
	int ret = leafDataToInt(value_int, isFirst, cfgName, section, subsection,
		key, MIN_CHAR, MAX_CHAR, out_f);
	if(ret == FAIL) return FAIL;
	value = char(value_int);

	return SUCCESS;
}

int leafDataToChar(uchar& value, bool isFirst, const char* cfgName, const char* section, 
			const char* subsection, const char* key,
			FILE* out_f) {

	int value_int;
	int ret = leafDataToInt(value_int, isFirst, cfgName, section, subsection,
		key, MIN_CHAR, MAX_CHAR, out_f);
	if(ret == FAIL) return FAIL;
	value = uchar(value_int);

	return SUCCESS;
}

int leafDataToInt(int& value, bool isFirst, const char* cfgName , const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* data = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	warningMess.sprintf("Invalid input:\n%s\\%s\\%s\\%s",
			cfgName,section,subsection, key);
	value = strToInt(data, warningMess);
	return SUCCESS;
}

int leafDataToInt(int& value, bool isFirst, const char* cfgName,const char* section,
			const char* subsection, const char* key,
			int min, int max, FILE *out_f) {

	bool changeble = true;
	QString* data = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	warningMess.sprintf("Invalid input:\n%s\\%s\\%s\\%s",
			cfgName,section,subsection, key);
	value = strToInt(data, warningMess, min, max);
	return SUCCESS;
}

double strToDouble(const QString* str, const QString& message) {

	bool ok;
	double value = str->toDouble(&ok);
	if(!ok) {
//		QString warn = message;
//		warn.prepend("Invalid input: ");			
//		showStrongMessage(warn);
		return 0;
	}
	return value;
}

double strToDouble(const QString* str, const QString& message,
				double min, double max) {

	bool ok;
	double value = str->toDouble(&ok);

	if(ok && (value >= min) && (value <= max)) {
		return value;
	}

	QString warn = "Invalid input: ";
	warn += message;
	if(value < min) {
		QString warn_add;
		warn_add.sprintf("\n%f<%f!", value, min);
		warn += warn_add;
		showStrongMessage(warn);
	}
	if(value > max) {
		QString warn_add;
		warn_add.sprintf("\n%f>%f!", value, max);
		warn += warn_add;
		showStrongMessage(warn);
	}
//	showStrongMessage(warn);
	return 0;
}

int leafDataToDouble(double& value, bool isFirst, const char* cfgName,const char* section,
			const char* key,
			FILE *out_f) {

	jdebug("leafDataToDouble");
	bool changeble = true;
	QString* data = getLeafData(changeble, cfgName, section,
				key, NULL, out_f);
	if(!changeble && !isFirst) return FAIL;
	
	warningMess.sprintf("\n%s\\%s\\%s",
		cfgName,section, key);
	value = strToDouble(data, warningMess);
	return SUCCESS;
}

int leafDataToDouble(double& value, bool isFirst, const char* cfgName,const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	jdebug("leafDataToDouble");
	bool changeble = true;
	QString* data = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	warningMess.sprintf("\n%s\\%s\\%s\\%s",
		cfgName,section,subsection, key);
	value = strToDouble(data, warningMess);
	return SUCCESS;
}

int leafDataToDouble(double& value, bool isFirst, const char* cfgName, const char* section,
		const char* subsection, const char* key,
		double min, double max, FILE *out_f) {

	jdebug("leafDataToDouble");
	bool changeble = true;
	QString* data = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	warningMess.sprintf("\n%s\\%s\\%s\\%s",
		cfgName,section,subsection, key);
	value = strToDouble(data, warningMess, min, max);
	return SUCCESS;
}

static const QString invForm = "Invalid format";
static const QString outRange = "Out of Range";

int leafDataToTimeSec(double& sec, bool isFirst, const char* cfgName,const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString *p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	QString str = *p_str;
	str = str.simplifyWhiteSpace();
	int startPos = 0;
	int spacePos = 0;
	double sign = 1.0;
	bool ok;
	if(str[0] == '+') { sign = 1.0; startPos = 1; }
	else if(str[0] == '-') { sign = -1.0; startPos = 1; }
	else startPos = 0;
	spacePos = str.find(' ',startPos);
	if(spacePos < 0) throw LoadError(invForm);
	int hour = str.mid(startPos,spacePos-startPos).toInt(&ok);
	if(!ok) throw LoadError(invForm);
	if(hour < 0) throw LoadError(invForm);

	startPos = spacePos + 1;
	spacePos = str.find(' ',startPos);
	if(spacePos < 0) throw LoadError(invForm);
	int min = str.mid(startPos,spacePos-startPos).toInt(&ok);
	if(!ok) throw LoadError(invForm);
	if(min < 0) throw LoadError(invForm);

	startPos = spacePos + 1;
	double sec_tmp = str.right(str.length()-startPos).toDouble(&ok);
	if(!ok) throw LoadError(invForm);
	if(sec_tmp < 0) throw LoadError(invForm);

	sec_tmp += ((double)hour*60.0 + (double)min)*60.0;
	if(sign < 0) sec_tmp = SEC_IN_DAY - sec_tmp;
	if((sec_tmp < 0) || (sec_tmp > SEC_IN_DAY)) throw LoadError(outRange);
	sec = sec_tmp;
	return SUCCESS;
}

int leafDataToDegSec(double& sec, bool isFirst, const char* cfgName,const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if(!changeble && !isFirst) return FAIL;

	QString str = *p_str;
	str = str.simplifyWhiteSpace();
	double sign = 1.0;
	int startPos = 0;
	int spacePos = 0;
	bool ok;
	if(str[0] == '+') { sign = 1.0; startPos = 1; }
	else if(str[0] == '-') { sign = -1.0; startPos = 1; }
	else startPos = 0;

	spacePos = str.find(' ',startPos);
	if(spacePos < 0) throw LoadError(invForm);
	int deg = str.mid(startPos,spacePos-startPos).toInt(&ok);
	if(!ok) throw LoadError(invForm);

	startPos = spacePos + 1;
	spacePos = str.find(' ',startPos);
	if(spacePos < 0) throw LoadError(invForm);
	int min = str.mid(startPos,spacePos-startPos).toInt(&ok);
	if(!ok) throw LoadError(invForm);
	if(min < 0) throw LoadError(invForm);

	startPos = spacePos + 1;
	double sec_tmp = str.right(str.length()-startPos).toDouble(&ok);
	if(!ok) throw LoadError(invForm);
	if(sec_tmp < 0) throw LoadError(invForm);

	sec_tmp += ((double)deg*60.0 + (double)min)*60.0;
	if(sec_tmp > SEC_IN_90) throw LoadError(outRange);
	sec = sign * sec_tmp;
	return SUCCESS;
}

int leafDataOnOffToInt(bool& onoff, bool isFirst, const char* cfgName, const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	int onoff_int = onoff;
	int ret = leafDataOnOffToInt(onoff_int, isFirst, cfgName, section,
			subsection, key, out_f);
	onoff = onoff_int;
	return ret;
}

int leafDataOnOffToInt(int& onoff, bool isFirst, const char* cfgName, const char* section,
			const char* subsection, const char* key,
			FILE *out_f) {

	bool changeble = true;
	QString* str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if((!changeble) && (!isFirst)) return FAIL;
	else if(str->upper() == "ON")  onoff = ON;
	else if(str->upper() == "OFF") onoff = OFF;
	else if(str->upper() == "OPEN") onoff = OPEN;
	else if(str->upper() == "CLOSE") onoff = CLOSE;
	else if(str->upper() == "YES") onoff = YES;
	else if(str->upper() == "NO") onoff = NO;
	else {
		warningMess.sprintf("Invalid input:\n%s\\%s\\%s\\%s",
			cfgName,section,subsection, key);
		showStrongMessage(warningMess);
		onoff = UNDEFINED;
		return FAIL;
	}
	return SUCCESS;
}

int leafDataToApertureData(char& stopCorrect, bool isFirst, const char* cfgName,const char* section,
			const char* subsection, const char* key,
			QString& name, double& foclen,
			FILE *out_f) {
	bool changeble = true;
	QString* p_str = getLeafData(changeble, cfgName, section, subsection,
				key, out_f);
	if((!changeble) && (!isFirst)) return FAIL;

	QString str = *p_str;

	str.replace(QRegExp(","),QString(" "));
	str.replace(QRegExp("\""),QString(" "));
	str = str.simplifyWhiteSpace();

	bool ok1 = false;
	bool ok2 = false;
	foclen = 0;
	char value_char = 0;

	int pos1 = str.find(" ");
	if(pos1 != -1) {
		name = str.left(pos1);
		if(name.upper() == "SPARE") {
			stopCorrect = 0;
			return FAIL;
		}

		int pos2 = str.find(" ",pos1+1);
		if(pos2 != -1) {
			foclen = str.mid(pos1,pos2-pos1).toDouble(&ok1);
			value_char = char(str.right(str.length()-pos2-1)
				.toInt(&ok2));
		}
	} else {
		if(str.upper() == "SPARE") {
			name = "Spare";
			stopCorrect = 0;
			return FAIL;
		}
 		ok1 = false;
	}
	if(!ok1 || !ok2) {
		warningMess.sprintf("Invalid input:\n%s\\%s\\%s\\%s:\n%s",
			cfgName,section,subsection, key, str.data());
		debug("%s",warningMess.data());
		showStrongMessage(warningMess);
		stopCorrect = 0;
		return FAIL;
	}
	stopCorrect = value_char;
	return SUCCESS;
}

int testFileName(QString& fileName, int dialog, bool isFatal) {
	QFile file(fileName);
	if(file.exists()) return 1;
	if(!dialog) return 0;
	warningMess.sprintf("File \"%s\" not found",fileName.data());
	if(isFatal) {
		showFatalError(warningMess);
//		exit(1);
	} else {
		showStrongMessage(warningMess);
	}
	return 0;
}

int testDirName(QString& dirName, int dialog, bool* isEmpty) {
	QDir dir(dirName);
	if(!dir.exists()) {
		if(!dialog) return 0;
		warningMess.sprintf("Directory \"%s\" not found",dirName.data());
		showFatalError(warningMess);
//		exit(1);
	}
	if(isEmpty) {
		if(dir.count() > 2) *isEmpty = false;
		else *isEmpty = true;
	}
	return 1;
}

void prepareDirName(QString& dirName, bool test) {
	if(dirName.find("/",dirName.length()-1) == -1) {
		dirName.append("/");
	}
	dirName.prepend(dataPath);
	if(test) testDirName(dirName,1);
}

int doubleTo255int(double val) {

	int int_val = int(rint(255.0 * val));
	if((int_val == 0) && (val > 0)) int_val = 1;
	if(int_val < 0) int_val = 0;
	if(int_val > 255) int_val = 255;
	return(int_val);
}

int calcNumberDoubleInString(QString* p_str, const char* path) {

	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();
	int number = str.contains(" ") + 1;

	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok;
		if(pos != -1) str.left(pos).toDouble(&ok);
		else str.toDouble(&ok);

		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			showStrongMessage(warningMess);
			return 0;
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
	}
	return number;
}

int stringToStrArray(QString* p_str, QArray<const char*>& arr,
		const char* what, const char* path, const char* validator) {
	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();

	int number = str.contains(" ") + 1;
	arr.resize(number);

	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok = false;
		QString str1;
		if(pos != -1) str1 = str.left(pos);
		else str1 = str;

		if(strcmp(validator,"color") == 0) {
			arr[i] = getColor(str1);
			ok = true;
		} else if(strcmp(validator,"show") == 0) {
			arr[i] = getShow(str1);
			ok = true;
		} else if(strcmp(validator,"entity") == 0) {
			const char* ent = getEntity(what,str1);
			if(ent) arr[i] = ent;
			else {
				i--;
				number--;
			}
			ok = true;
		}

		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			showStrongMessage(warningMess);
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
	}
	if(number != (int)arr.size()) arr.resize(number);
	return number;
}

int stringToDoubleArray(QString* p_str, QArray<double>& arr,
						const char* path) {
	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();
	int number = str.contains(" ") + 1;
	arr.resize(number);

	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok;
		if(pos != -1) arr[i] = str.left(pos).toDouble(&ok);
		else arr[i] = str.toDouble(&ok);

		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			showStrongMessage(warningMess);
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
	}
	return number;
}

int stringToDoubleArray(QString* p_str, QArray<double>& arr,
		const char* path, double min, double max) {

	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();
	int number = str.contains(" ") + 1;
	arr.resize(number);

	int _abort = 0;
	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok;
		if(pos != -1) arr[i] = str.left(pos).toDouble(&ok);
		else arr[i] = str.toDouble(&ok);
		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			_abort = 1;
		}
		if(arr[i] < min) {
			warningMess.sprintf("Invalid input:\n%s: %f<%f!",
			path, arr[i], min);
			_abort = 1;
		}
		if(arr[i] > max) {
			warningMess.sprintf("Invalid input:\n%s: %f>%f!",
			path, arr[i], max);
			_abort = 1;
		}
		if(_abort) {
			showStrongMessage(warningMess);
			_abort = 0;
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
	}
	return number;
}

int stringToIntArray(QString* p_str, QArray<int>& arr,
				const char* path) {

	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();
	int number = str.contains(" ") + 1;
	arr.resize(number);

	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok;
		int val_int;
		if(pos != -1) val_int = str.left(pos).toInt(&ok);
		else val_int = str.toInt(&ok);

		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			showStrongMessage(warningMess);
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
		arr[i] = val_int;
	}
	return number;
}

int stringDoubleTo255intArray(QString* p_str, QArray<int>& arr,
				const char* path) {

	if(!p_str) return(0);

	QString str = *p_str;
	str.replace(QRegExp(","),QString(" "));
	str = str.simplifyWhiteSpace();
	int number = str.contains(" ") + 1;
	arr.resize(number);

	for(int i=0; i<number; i++) {
		int pos = str.find(" ");
		bool ok;
		double val_double;
		if(pos != -1) val_double = str.left(pos).toDouble(&ok);
		else val_double = str.toDouble(&ok);

		if(!ok) {
			warningMess.sprintf("Invalid input:\n%s",path);
			showStrongMessage(warningMess);
		}
		pos = str.find(" ");
		if(pos != -1) str.remove(0,(uint)++pos);
		arr[i] = doubleTo255int(val_double);
	}
	return number;
}
