/*--------------------------------------------------------------------------*/
/** @file cfg.h
@brief  Tools for accessing to CFG tree in memory: search, validation, I/O

                  MASS project: TURBINA module file
                       header file for cfg.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef CFG_H
#define CFG_H

#include <stdio.h>
#include <qstring.h>
#include "names.h"
#include "tree.h"

const char* getShow(QString& showName);
const char* getColor(QString& colorName);
const char* getEntity(const char* what, QString& entName);
const char* getCrosIdx(const char* entity);

CFGTree* getTree(const char* cfgName);

CFGLeaf* getLeaf(const char* cfgName,
		const char* name1,
		const char* name2,
		const char* name3=NULL);

CFGLeaf* getLeaf(const char* cfgName,
		const QString& pathName);

bool	isLeafChanged(const char* cfgName,
		const char* name1,
		const char* name2,
		const char* name3=NULL);

void	setLeafData(const char* cfgName,
		const char* section,
		const char* subsection,
		const char* key,
		QString& newData,
		FILE *out_f=NULL);

void	setLeafData(const QString& pathName,
			const QString& newData,
			FILE *out_f=NULL,
			const char* cfgName=cfgFile);

QString* getLeafData(bool& changeble, 
		const char* cfgName,
		const char* name1,
		const char* name2,
		const char* name3=NULL,
		FILE *out_f=NULL);

int leafDataToStr(QString& str,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int leafDataToStr(QString** p_p_str,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int leafDataToStr(const char** str,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int	strToInt(const QString* str,
			const QString& message);

int	strToInt(const QString* str,
			const QString& message,
			int min,
			int max);

int leafDataToChar(
			char& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int leafDataToChar(
			uchar& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int	leafDataToInt(
			int& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int	leafDataToInt(
			int& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			int min,
			int max,
			FILE* out_f=NULL);

double	strToDouble(const QString* str,
			const QString& message);

double	strToDouble(const QString* str,
			const QString& message,
			double min,
			double max);

int leafDataToDouble(
			double& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			FILE* out_f=NULL);

int leafDataToDouble(
			double& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* key,
			FILE* out_f=NULL);

int leafDataToDouble(
			double& value,
			bool isFirst,
			const char* cfgName,
			const char* section,
			const char* subsection,
			const char* key,
			double min,
			double max,
			FILE* out_f=NULL);

int leafDataToTimeSec(
				double& sec,
				bool isFirst,
				const char* cfgName,
				const char* section,
				const char* subsection,
				const char* key,
				FILE *out_f=NULL);

int leafDataToDegSec(
				double& sec,
				bool isFirst,
				const char* cfgName,
				const char* section,
				const char* subsection,
				const char* key,
				FILE *out_f=NULL);

int	leafDataOnOffToInt(int& onoff,
				bool isFirst,
				const char* cfgName,
				const char* section,
				const char* subsection,
				const char* key,
				FILE *out_f=NULL);

int	leafDataOnOffToInt(bool& onoff,
				bool isFirst,
				const char* cfgName,
				const char* section,
				const char* subsection,
				const char* key,
				FILE *out_f=NULL);

int leafDataToApertureData(
				char& stopCorrect,
				bool isFirst,
				const char* cfgName,
				const char* section,
				const char* subsection,
				const char* key,
				QString& name,
				double& foclen,
				FILE *out_f=NULL);

int	testFileName(QString& fileName,
				int dialog, bool isFatal=true);

int	testDirName(QString& dirName,
				int dialog, bool* isEmpty=NULL);

void	prepareDirName(QString& dirName, bool test=true);
int	doubleTo255int(double val);

int	calcNumberDoubleInString(QString* p_str,
			const char* path);

int	stringToStrArray(QString* p_str,
			QArray<const char*>& arr,
			const char* what,
			const char* path,
			const char* validator);

int	stringToDoubleArray(QString* p_str,
			QArray<double>& arr,
			const char* path);

int	stringToDoubleArray(QString* p_str,
			QArray<double>& arr,
			const char* path,
			double min,
			double max);

int	stringToIntArray(QString* p_str,
			QArray<int>& arr,
			const char* path);

int	stringDoubleTo255intArray(QString* p_str,
			QArray<int>& arr,
			const char* path);

#endif
