#include <qapplication.h>
#include <qmessagebox.h>
#include <math.h>
#include "cfgdialog.h"
#include "tools.h"
#include "names.h"
#include "color.h"
#include "outcontrol.h"
#include "device.h"

extern CFGTree* tree;
extern DevManager devmanager;
extern OutControl outcontrol;
extern bool isMassRoot;

static const char* YESNO[] = {"Yes","No"};
static int Nyes = 2;

static const char* ONOFF[] = {"On","Off"};
static int Nonoff = 2; 

static const char* OPENCLOSE[] = {"Open","Close"};
static int Nopen = 2; 

static const char* HARA[] = {"HA","RA"};
static int Nhara = 2; 

static const char* METHOD[] = {"ApertureShift","TelescopeShift"};
static int Nmethod = 2; 

static const char* CN2METHOD[] = {"X","L"};
static int Ncn2method = 2; 

static const char* REFER[] = {"FirstMean","LastMean","SeasonMean"};
static int Nrefer = 3;

static const char* PRFTYPE[] = {"Bars","Smooth"};
static int Nprf = 2;

QValidator::State FloatValidator::validate(QString& text, int& pos) {
	jdebug("Text: %s len:%d pos: %d",text.data(),text.length(),pos);
	if(pos == 0) return Valid;
	if(!text.length()) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if((unsigned int)pos > text.length()) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}
/*
	if((!isdigit(text[pos-1])) && (text[pos-1] != '.')
			&& (text[pos-1] != '-') && (text[pos-1] != '+')) {
		jdebug("pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1]);
		return Invalid;
	}
*/
	if(text[pos-1] == '.') {
		if(text.contains('.') > 1) return Invalid;
	}
	if((text.find('-') != -1) && (text.find('-') != 0)) return Invalid;
	if((text.find('+') != -1) && (text.find('+') != 0)) return Invalid;

	if(text.contains('-') > 1) return Invalid;
	if(text.contains('+') > 1) return Invalid;

	if(text.length() < 1) {
		jdebug(" Less than five digits; not acceptable");
		return Valid;
	}
	return Valid;
//	return Acceptable;
}

QValidator::State IntValidator::validate(QString& text, int& pos) {
	jdebug("Text: %s len:%d pos: %d",text.data(),text.length(),pos);
	if(pos == 0) return Valid;
	if(!text.length()) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if((unsigned int)pos > text.length()) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}
/*
	if((!isdigit(text[pos-1])) && (text[pos-1] != '-')
			&& (text[pos-1] != '+')) {
		jdebug("pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1]);
		return Invalid;
	}
*/
	if((text.find('-') != -1) && (text.find('-') != 0)) return Invalid;
	if((text.find('+') != -1) && (text.find('+') != 0)) return Invalid;

	if(text.contains('-') > 1) return Invalid;
	if(text.contains('+') > 1) return Invalid;

	return Valid;
}
/*
QValidator::State TimeValidator::validate(QString& text, int& pos) {
	jdebug("Text: %s len:%d pos: %d",text.data(),text.length(),pos);
	if(pos == 0) return Valid;
	if(!text.length()) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if((unsigned int)pos > text.length()) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}

	if((!isdigit(text[pos-1])) && (text[pos-1] != '.') && (text[pos-1] != ' ')) {
		jdebug("pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1]);
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	jdebug("b1=%d len=%d",b1,text.length());	
	if(text.length() > b1) b2 = text.find(' ',b1+1);
	jdebug("b2=%d",b2);
	if(text[pos-1] == ' ') {
		if((b1 != 1) && (b1 != 2)) return Invalid;
		if(b2 != -1) {
			if(text.find(' ',b2+1) != -1) return Invalid;
		}
	}
//	if(atoi(&text[0]) >= 24) return Invalid;
	if((b1 != -1) && ((text.length()-1) > b1))
		if(atoi(&text[b1+1]) > 59) return Invalid;
	if((b2 != -1) && ((text.length()-1) > b2))
		if(atof(&text[b2+1]) >= 60) return Invalid;

	if((text.find('-') != -1) && (text.find('-') != 0)) return Invalid;
	if((text.find('+') != -1) && (text.find('+') != 0)) return Invalid;

	if(text.contains('-') > 1) return Invalid;
	if(text.contains('+') > 1) return Invalid;

	if((text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2))) return Invalid;
	if((text[pos-1] == '.') && (text.contains('.') > 1)) return Invalid;

	jdebug("Input Valid");
	return Valid;
}
*/
/*
QValidator::State DegreeValidator::validate(QString& text, int& pos) {
	jdebug("Text: %s len:%d pos: %d",text.data(),text.length(),pos);
	if(pos == 0) return Valid;
	if(!text.length()) {
		jdebug(" !text.length(), valid");
		return Valid;
	}
	if((unsigned int)pos > text.length()) {
		jdebug("  >= text.length(), valid");
		return Valid;
	}

	if((!isdigit(text[pos-1])) && (text[pos-1] != '.') && 
			(text[pos-1] != ' ') && (text[pos-1] != '-') &&
										(text[pos-1] != '+')) {
		jdebug("pos=%d Typed character: %c is neither digit nor dash",pos-1,text[pos-1]);
		return Invalid;
	}

	int b1 =  text.find(' ');
	int b2 = -1;
	jdebug("b1=%d len=%d",b1,text.length());	
	if(text.length() > b1) b2 = text.find(' ',b1+1);
	jdebug("b2=%d",b2);
	if(text[pos-1] == ' ') {
		if((b1 != 1) && (b1 != 2) && (b1 != 3)) return Invalid;
		if(b2 != -1) {
			if(text.find(' ',b2+1) != -1) return Invalid;
		}
	}
//	jdebug("%d",atoi(&text[0]));
//	if(abs(atoi(&text[0])) > 90) return Invalid;
	if((b1 != -1) && ((text.length()-1) > b1)){
		if(atoi(&text[b1+1]) > 59) return Invalid;
		if((abs(atoi(&text[0])) == 90) && (atoi(&text[b1+1]) > 0)) return Invalid;
	}
	if((b2 != -1) && ((text.length()-1) > b2)){
		if(atof(&text[b2+1]) >= 60) return Invalid;
		if((abs(atoi(&text[0])) == 90) && (atof(&text[b2+1]) > 0)) return Invalid;
	}
	if((text.find('-') != -1) && (text.find('-') != 0)) return Invalid;
	if((text.find('+') != -1) && (text.find('+') != 0)) return Invalid;

	if(text.contains('-') > 1) return Invalid;
	if(text.contains('+') > 1) return Invalid;

	if((text[pos-1] == '.') && ((b2 == -1) || ((pos-1) < b2))) return Invalid;
	if((text[pos-1] == '.') && (text.contains('.') > 1)) return Invalid;

	jdebug("Input Valid");
	return Valid;
}
*/

SubItem::SubItem(CFGBranch* branch) {
	_enable = true;
	_branch = branch;
}

SubItem::~SubItem() {
}

void SubItem::setEnabled(void) {
	_enable = true;
}

void SubItem::setDisabled(void) {
	_enable = false;
}

bool SubItem::searchChangeDisabledLeafs(void) {
	bool emitSignalChanged = false;
	_branch->toFirst();
	while(_branch->getCurrentLink()) {
		CFGLeaf* leaf =  _branch->getCurrentLink()->getLeaf();
		if(leaf && leaf->isChanged()
			&&!(leaf->isChangeEnabled())) {
			leaf->setChanged(false);
			QString text;
			text.sprintf("Field: %s Active after restarting",leaf->getKey());
			QMessageBox::warning(0,"cfg",text.data());
			jdebug("leaf: %s - change disabled!!!", leaf->getKey());
		}
		if(leaf && leaf->isChanged()) emitSignalChanged = true;
		_branch->toNext();
	}
	return emitSignalChanged;
}

void SubItem::slotExecBranchDialog(void) {
	BranchDialog* dialog = new BranchDialog(_branch, _enable);
	CHECK_PTR(dialog);
	connect(dialog,SIGNAL(signalShowStatus(const char*,int)),
				this,SIGNAL(signalShowStatus(const char*,int)));

	int ret = dialog->exec();

	if(ret == QDialog::Accepted) {
		jdebug("QDialog::Accepted");
		bool emitSignalChanged = searchChangeDisabledLeafs();
		tree->setChanged(1);
		emit signalShowStatus("Accepted",2000);
		if(emitSignalChanged) tree->emitSignalChanged();
	}
	delete dialog;
}

bool calcAccess	(bool _enable, bool for_root, bool count_disabled) {
	bool access = (_enable || !count_disabled) && 
		!(for_root && !isMassRoot); // enable only for mass_root
	return access;
}

JKQLineEdit::JKQLineEdit(QWidget* parent, CFGLeaf* leaf, bool enable) 
						: QLineEdit(parent) {
	_leaf = leaf;
	enable = calcAccess(enable,_leaf->isForRoot(),
				_leaf->isCountDisabled());
	setEnabled(enable);
}

JKQLineEdit::~JKQLineEdit() {
}

void JKQLineEdit::slotTextChanged(const QString& text) {
	jdebug("textChanged: %s", text.data());
//	_text = text;
//	_leaf->replaceData(text);
//	_leaf->setChanged(true);
}

void JKQLineEdit::enterEvent(QEvent* e) {
	emit signalShowStatus(_leaf->getHelp()->data(),-1);
	QLineEdit::enterEvent(e);
}

void JKQLineEdit::leaveEvent(QEvent* e) {
	emit signalShowStatus("", 0);
	QLineEdit::leaveEvent(e);
}

void JKQLineEdit::accept(void) {
	if(edited()) {
		_leaf->replaceData(text());
		_leaf->setChanged(true);
	}
}

JKQComboBox::JKQComboBox(bool rw, QWidget* parent,CFGLeaf* leaf,
				bool enable) : QComboBox(rw,parent) {
	_leaf = leaf;
	_edited = false;
	enable = calcAccess(enable,_leaf->isForRoot(),
				leaf->isCountDisabled());
	setEnabled(enable);
}

JKQComboBox::~JKQComboBox() {
}

void JKQComboBox::setCurrentText(const char* cur_text) {
	for(int i=0;i<count() ;i++) {
		if(!strcasecmp(text(i),cur_text)) {
			setCurrentItem(i);
		}
	}
}

void JKQComboBox::slotTextChanged(const QString& text) {
	jdebug("textChanged: %s", text.data());
	_edited = true;
//	_leaf->replaceData(text);
//	_leaf->setChanged(true);
}

void JKQComboBox::accept(void) {
	if(_edited) {
		_leaf->replaceData(currentText());
		_leaf->setChanged(true);
	}
}

void JKQComboBox::enterEvent(QEvent* e) {
	emit signalShowStatus(_leaf->getHelp()->data(),-1);
	QComboBox::enterEvent(e);
}

void JKQComboBox::leaveEvent(QEvent* e) {
	emit signalShowStatus("", 0);
	QComboBox::leaveEvent(e);
}

BranchDialog::BranchDialog(CFGBranch* branch, bool enable) :
		QDialog(0,"BranchDialog",true /*modal*/) {
	_branch = branch;
	int num_layout_cols=2;

	setCaption("Section Dialog");

	okbutton = new QPushButton("Apply",this);
	CHECK_PTR(okbutton);
	okbutton->setAutoDefault(true);
	connect(okbutton, SIGNAL(clicked()),
				this, SLOT(accept()));

	cancelbutton = new QPushButton("Cancel",this);
	CHECK_PTR(cancelbutton);
	cancelbutton->setAutoDefault(true);
	connect(cancelbutton, SIGNAL(clicked()),
				this, SLOT(reject()));

	int border = fontMetrics().height();
	int labelheight =  fontMetrics().height()*5/4 ;
	int row=0;
	_branch->toFirst();
	while(_branch->getCurrentLink()) {
		row++;
		_branch->toNext();
	}

	int w_height = (labelheight+border) * (row+3) + border; 
	if(w_height > qApp->desktop()->height()) {
		int maxWindowFontHeight = int(floor(qApp->desktop()->height() / (2.25*(row+3) + 1)));
		int newFontSize = maxWindowFontHeight - 1;	// empirical, size != height !!!
		const int minFontSize = 8;
		while(newFontSize > minFontSize) {
			QFont font = QFont("Helvetica",newFontSize);
//				if(font.exactMatch()) printf("is exactMatch!\n");				
			if(font.exactMatch()
				&& (QFontMetrics(font).height() <= maxWindowFontHeight)) break;
			newFontSize--;
		}
		setFont(QFont("Helvetica",newFontSize),true);
		border = fontMetrics().height();
		labelheight =  fontMetrics().height()*5/4 ;
	}

	layout = new QGridLayout(this,1,num_layout_cols,border);
	CHECK_PTR(layout);
	int layout_row = 1;

	int labelwidth = 0;
	int editwidth = 0;
	bool enableApply = false;
	int maxlabelwidth = okbutton->sizeHint().width();
	int maxeditwidth = cancelbutton->sizeHint().width();

	_branch->toFirst();
	while(_branch->getCurrentLink()) {
		if(_branch->getCurrentLink()->getId() == 'l') {
			Link* link = _branch->getCurrentLink();
			labelwidth = fontMetrics().width(link->getLeafKey());
			editwidth = fontMetrics().width(link->getLeafData());
			if(labelwidth > maxlabelwidth) maxlabelwidth = labelwidth;
			if(editwidth > maxeditwidth) maxeditwidth = editwidth;
			enableApply = enableApply ||
					(enable ||
				!(link->getLeaf()->isCountDisabled()));
		}
		_branch->toNext();
	}

	okbutton->setEnabled(enableApply);

	maxeditwidth += fontMetrics().width("ww"); //  ...
	if(maxeditwidth+maxlabelwidth < fontMetrics().width(_branch->getName())) {
		int dwidth = (fontMetrics().width(_branch->getName())-
							(maxeditwidth+maxlabelwidth))/2; 
		maxeditwidth += dwidth;
		maxlabelwidth += dwidth;
	}

	int halfDesk =  qApp->desktop()->width()/2;
	if(maxeditwidth > halfDesk) maxeditwidth = halfDesk;
	if(maxlabelwidth > halfDesk) maxlabelwidth = halfDesk;

//	int labelheight =  fontMetrics().height()*5/4 ;

	n_edit = 0;
	n_combo = 0;
	n_label = 0;
	edit.resize(10);
	combo.resize(10);
	label.resize(10);
	label[n_label] = new QLabel(_branch->getName(),this);
	CHECK_PTR(label[n_label]);
	label[n_label]->setFixedSize(label[n_label]->sizeHint());
	layout->addMultiCellWidget(label[n_label],0,0,0,1,AlignHCenter);	
	n_label++;

//	floatvalidator = new FloatValidator(this);
//	CHECK_PTR(floatvalidator);
//	timevalidator = new TimeValidator(this);
//	CHECK_PTR(timevalidator);
//	degreevalidator = new DegreeValidator(this);
//	CHECK_PTR(degreevalidator);

	_branch->toFirst();
	while(_branch->getCurrentLink()) {
		if(_branch->getCurrentLink()->getId() == 'l') {

			layout->expand(layout_row+1,num_layout_cols);

			if((int)label.size() < n_label+1) label.resize(label.size()+10);
			label[n_label] = new QLabel(_branch->getCurrentLink()->getLeafKey(),this);
			CHECK_PTR(label[n_label]);
			label[n_label]->setFixedSize(maxlabelwidth,labelheight);
			layout->addWidget(label[n_label],layout_row,0);
			n_label++;

			const char* validat = _branch->getCurrentLink()->getLeaf()->getValidatorType();
			if(!validat)	validat = "empty";


			if(!strcasecmp(validat,"yes")        ||
				!strcasecmp(validat,"onoff")  ||
				!strcasecmp(validat,"onoff")  ||
				!strcasecmp(validat,"open")   ||
				!strcasecmp(validat,"hara")   ||
				!strcasecmp(validat,"color")  ||
				!strcasecmp(validat,"method")  ||
				!strcasecmp(validat,"cn2method")  ||
				!strcasecmp(validat,"apert")  ||
				!strcasecmp(validat,"prftype")||
				!strcasecmp(validat,"lut")    ||
				!strcasecmp(validat,"refer")) {

				if((int)combo.size() < n_combo+1) combo.resize(combo.size()+10);
				combo[n_combo] = new JKQComboBox(false,this,
						_branch->getCurrentLink()->getLeaf(),enable);
				CHECK_PTR(combo[n_combo]);

				if(!strcasecmp(validat,"yes"))
					combo[n_combo]->insertStrList(YESNO,Nyes);
				else if(!strcasecmp(validat,"onoff"))
					combo[n_combo]->insertStrList(ONOFF,Nonoff);
				else if(!strcasecmp(validat,"open"))
					combo[n_combo]->insertStrList(OPENCLOSE,Nopen);
				else if(!strcasecmp(validat,"hara"))
					combo[n_combo]->insertStrList(HARA,Nhara);
				else if(!strcasecmp(validat,"method"))
					combo[n_combo]->insertStrList(METHOD,Nmethod);
				else if(!strcasecmp(validat,"cn2method"))
					combo[n_combo]->insertStrList(CN2METHOD,Ncn2method);
				else if(!strcasecmp(validat,"color"))
					combo[n_combo]->insertStrList(JCOLORS,Ncolors);
				else if(!strcasecmp(validat,"refer"))
					combo[n_combo]->insertStrList(REFER,Nrefer);
				else if(!strcasecmp(validat,"prftype"))
					combo[n_combo]->insertStrList(PRFTYPE,Nprf);
				else if(!strcasecmp(validat,"lut")) {
					int nl = outcontrol.getNlut();
					for(int i=0; i<nl; i++) {
						combo[n_combo]->
							insertItem(outcontrol.getLutName(i));
					}
				} else if(!strcasecmp(validat,"apert")) {
					int ns = devmanager.getNStops();
					for(int i=0; i<ns; i++) {
						combo[n_combo]->
							insertItem(devmanager.getStopName(i));
					}
				}

				combo[n_combo]->setFixedSize(maxeditwidth,labelheight);
				combo[n_combo]->setCurrentText(_branch->getCurrentLink()->getLeafData());
				connect(combo[n_combo],SIGNAL(activated(const QString&)),						
							combo[n_combo],SLOT(slotTextChanged(const QString&)));
				connect(combo[n_combo],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)));
				layout->addWidget(combo[n_combo],layout_row,1);
				n_combo++;

			} else {	
				if((int)edit.size() < n_edit+1) edit.resize(edit.size()+10);
				edit[n_edit] = new JKQLineEdit(this,
						_branch->getCurrentLink()->getLeaf(),enable);
				CHECK_PTR(edit[n_edit]);
				edit[n_edit]->setFixedSize(maxeditwidth,labelheight);
				edit[n_edit]->setText(_branch->getCurrentLink()->getLeafData());

				if(!strcasecmp(validat,"float")) {
//					edit[n_edit]->setValidator(floatvalidator);
				} else if(!strcasecmp(validat,"time")) {
//					edit[n_edit]->setValidator(timevalidator);
				} else if (!strcasecmp(validat,"degree")) {
//					edit[n_edit]->setValidator(degreevalidator);
				}
				connect(edit[n_edit],SIGNAL(textChanged(const QString&)),
						edit[n_edit],SLOT(slotTextChanged(const QString&)));
				connect(edit[n_edit],SIGNAL(signalShowStatus(const char*,int)),
							this,SIGNAL(signalShowStatus(const char*,int)));
				layout->addWidget(edit[n_edit],layout_row,1);
				n_edit++;
			}
			layout_row++;		

		}
		_branch->toNext();
	}

//	n_label--; n_edit--;n_combo--;
	layout->expand(layout_row+1,num_layout_cols);

	okbutton->setMinimumSize(okbutton->sizeHint());
	okbutton->setMaximumSize(maxlabelwidth,okbutton->sizeHint().height());
	layout->addWidget(okbutton,layout_row,0);	

	cancelbutton->setMinimumSize(cancelbutton->sizeHint());
	cancelbutton->setMaximumSize(maxeditwidth,cancelbutton->sizeHint().height());
	layout->addWidget(cancelbutton,layout_row,1);	

	w_height = (labelheight+border) * (layout_row+1) + border; 
	move((qApp->desktop()->width()-width())/2,
					(qApp->desktop()->height()-w_height)/2);
	emit signalShowStatus("Section dialog",4000);
}

BranchDialog::~BranchDialog() {
	jdebug("BranchDialog: Destructor \n");

	delete layout;
	delete cancelbutton;
	delete okbutton;
	for(int i=0;i<n_label;i++) {
		delete label[i];
	}
	for(int i=0;i<n_edit;i++) {
		delete edit[i];
	}
	for(int i=0;i<n_combo;i++) {
		delete combo[i];
	}

//	delete floatvalidator;
//	delete timevalidator;
//	delete degreevalidator;

	jdebug("BranchDialog: end of Destructor \n");
}

void BranchDialog::accept(void) {
	jdebug("BranchDialog::accept");
	for(int i=0;i<n_edit;i++) {
		edit[i]->accept();
	}
	for(int i=0;i<n_combo;i++) {
		combo[i]->accept();
	}
	QDialog::accept();
}

CFGDialog::CFGDialog(QPopupMenu* menu, int* _item_id,
					QArray<QString*> menu_help) {
	CFGBranch *branch, *s_branch;

	jdebug("CFG Dalog: start of Constructor"); 
//	work_tree = tree;
//	work_tree = new CFGTree(*tree);

	n_section_item = -1;
	n_s_section_item = -1;
	n_sub_item = -1;
	int item_id = *_item_id;

	section_item.resize(10);
	s_section_item.resize(10);
	sub_item.resize(10);

	tree->toFirst();
	while(tree->getCurrentLink()) {
		if(tree->getCurrentLink()->getId() == 'b') {
			branch = tree->getCurrentLink()->getBranch();
			if(branch->getNSubBranch()) { // ..,     =>   
				n_section_item++;
				if((int)section_item.size() < n_section_item+1) 
					section_item.resize(section_item.size()+10);

				section_item[n_section_item] = new QPopupMenu;
				CHECK_PTR(section_item[n_section_item]);	
				menu->insertItem(branch->getName(),section_item[n_section_item],item_id);
				if((int)menu_help.size() <= item_id) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString(branch->getHelp()->data());
				item_id++;

				branch->toFirst();
				while(branch->getCurrentLink()) {
					if(branch->getCurrentLink()->getId() == 'b') {
						s_branch = branch->getCurrentLink()->getBranch();
						if(s_branch->getNSubBranch()) {	//  (net!)   s_branch
						} else {	//     s_branch =>  
							n_sub_item++;
							if((int)sub_item.size() < n_sub_item+1) 
									sub_item.resize(sub_item.size()+10);
							sub_item[n_sub_item] = new SubItem(s_branch);
							CHECK_PTR(sub_item[n_sub_item]);
							connect(sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
								this,SIGNAL(signalShowStatus(const char*,int)));
							section_item[n_section_item]->insertItem(s_branch->getName(),
								sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id);
							if((int)menu_help.size() <= item_id) menu_help.resize(menu_help.size()+10); 
							menu_help[item_id] = new QString(s_branch->getHelp()->data());
							item_id++;
						}
					}
					branch->toNext();
				}
			} else {				//				 
				n_sub_item++;
				if((int)sub_item.size() < n_sub_item+1) 
					sub_item.resize(sub_item.size()+10);
				sub_item[n_sub_item] = new SubItem(branch);
				CHECK_PTR(sub_item[n_sub_item]);
				connect(sub_item[n_sub_item],SIGNAL(signalShowStatus(const char*,int)),
					this,SIGNAL(signalShowStatus(const char*,int)));
				menu->insertItem(branch->getName(),
					sub_item[n_sub_item],SLOT(slotExecBranchDialog()),0,item_id);
				if((int)menu_help.size() <= item_id) menu_help.resize(menu_help.size()+10); 
				menu_help[item_id] = new QString(branch->getHelp()->data());
				item_id++;
			}
		}
		tree->toNext();
	}
	menu->insertSeparator();
	menu->insertItem("Save CFG",this,SLOT(slotSaveFile()),0,item_id);
	if((int)menu_help.size() <= item_id) menu_help.resize(menu_help.size()+10); 
	menu_help[item_id] = new QString("Save CFG");
	item_id++;
	*_item_id = item_id;

	jdebug("CFG Dalog: end of Constructor"); 
}

CFGDialog::~CFGDialog() {
	jdebug("CFG Dialog: Destructor");
	for(int i=0;i<=n_section_item;i++) {
		delete section_item[i];
	}
	for(int i=0;i<=n_s_section_item;i++) {
		delete s_section_item[i];
	}
	for(int i=0;i<=n_sub_item;i++) {
		delete sub_item[i];
	}
	jdebug("end of CFG Dialog: Destructor");
}

//void CFGDialog::slotCfgChanged(void) {
//	_cfgChanged = 1;
//}
//
void CFGDialog::setEnabled(void) {
	int i;
	for(i=0;i<=n_sub_item;i++) {
		sub_item[i]->setEnabled();
	}
}

void CFGDialog::setDisabled(void) {
	int i;
	for(i=0;i<=n_sub_item;i++) {
		sub_item[i]->setDisabled();
	}
}

void CFGDialog::testCfgChang(void) {
	if(!tree->isChanged()) return;
	slotSaveFile();
}

void CFGDialog::slotSaveFile(void) {
	if(!tree->isChanged()) {
		emit signalShowStatus("CFG-file not changed so no update needed",2000);
		return;
	}

	QMessageBox mb("Warning","Save changes to CFG-file?",
		QMessageBox::Warning,QMessageBox::Yes,
		QMessageBox::No, 0);
	int ret = mb.exec();
	if(ret != QMessageBox::Yes) {
		emit signalShowStatus("CFG-file not saved",2000);
		return;
	}

	QString backCfg = cfgFile;
	backCfg.append(".back");
	rename(cfgFile,backCfg.data());
	ret = tree->saveToFile(cfgFile);
	
	if(ret == 0) emit signalShowStatus("CFG-file saved",2000);
	else emit signalShowStatus("CFG-file not saved",2000);
}
