#include <stdlib.h>	// To prevent problems with missing built-in abs():
#include <math.h>
#include "counter.h"
#include "tools.h"
#include "exchange.h"
#include "cfg.h"

Counter::Counter(void) {
	jdebug("counter::counter"); 
	_name = "Counters";
	resetVariables();
	jdebug("end of Counter's constructor");
}

Counter::~Counter() {
	jdebug("Destructor of Counter\n");
	delete[] data;
	jdebug("End of destructor of Counter\n");
}

void Counter::resetVariables(void) {
	jdebug("counter::resetVariables");
	ProCounter::resetVariables();
	_levelValue = UNDEFINED;
	jdebug("End of counter's resetVariables");
}

void Counter::create(int sizeOfData) {
	ProCounter::create(sizeOfData);
	_nData = 0;
	data = new count_t[_maxNData];
}

int Counter::loadOwnConst(void) {
	jdebug("counter::loadOwnConst");
	bool first = true;
	leafDataToChar(_setLevel,first,deviceCfgFile,_name,"Command","SetLevel");
	leafDataToChar(_getLevel,first,deviceCfgFile,_name,"Command","GetLevel");

	leafDataToDouble(_workLevel,first,deviceCfgFile,_name,_identName,"Discrimination");
	leafDataToDouble(_nonLinearity,first,deviceCfgFile,_name,_identName,"NonLinearity");
	leafDataToDouble(_nonPoisson,first,deviceCfgFile,_name,_identName,"NonPoisson");

	jdebug("End of counter's loadOwnConst");
	if(_setLevel == 0) return FAIL;
	return SUCCESS;
}

int Counter::loadConst(void) {
	jdebug("counter::loadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	if(ProCounter::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

void Counter::resizeData(int newSize) {
	if(newSize > _maxNData) {
		_maxNData = newSize;
		delete[] data;
		data = new count_t[ _maxNData ];
	}
}

bool Counter::expandData(void) {
	int nCountInBlock = _blockSize / BYTE_PER_COUNT;
	if(!nCountInBlock) return false;
	int finiteLabel = _expositionNumber / nCountInBlock;

	bool isCorrected = false;
	for(int label=0; label<finiteLabel; label++) {
		if(label >= _nBlockLabel) {
			insertBlock(label, INVALID_COUNT);
			isCorrected = true;
			continue;
		}
		if(blockLabel[label] > label) { // skipped
			insertBlock(label,INVALID_COUNT);
			isCorrected = true;
			continue;
		}
		if(blockLabel[label] < label) { // multiplieds
			if(removeBlock(label) == 0) break;
			label--;
			continue;
		}
	}
	return isCorrected;
}

int Counter::repairData(void) {
	double avg_d = 0;
	int nValid = 0 ;
	for(int i=0; i<_nData; i++) {
		if(data[i] != INVALID_COUNT) {
			avg_d += data[i];
			nValid++;
		}
	}
	if(nValid == 0) {
		// bad... nothing to divide for: return number of "corrected" = nData
		return _nData ;
	} 
	count_t avg = (count_t)rint(avg_d / nValid);

	// Replace INVALIDs with average:
	int nCorrected = 0;
	for(int i=0; i<_nData; i++) {
		if(data[i] == INVALID_COUNT) {
			data[i] = avg ;
			nCorrected++;
		}
	}
	return nCorrected ;
}

int Counter::insertBlock(int label, count_t value) {
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	int start = 0;
	int startLabel = 0;
	for(int i=0; i<_nBlockLabel; i++) {
		if(blockLabel[i] == label-1) {
			start = (i+1)*_nCountInBlock;
			startLabel = i+1;
		}
	}
	if((_nData+_nCountInBlock) > _maxNData) {
		jdebug("_nData+_nCountInBlock = %d > _maxNData = %d",_nData+_nCountInBlock,_maxNData);
		return(0);
	}
	_nData += _nCountInBlock;
	for(int j=(_nData-1); j>=(start+_nCountInBlock); j--) {
		data[j] = data[j-_nCountInBlock];
	}
	for(int j=start; j<(start+_nCountInBlock); j++) {
		data[j] = value;
	}

	_nBlockLabel++;
	for(int j=_nBlockLabel; j>startLabel; j--) {
		blockLabel[j] = blockLabel[j-1];
	}
	blockLabel[startLabel] = label;
	jdebug("%s: block %d inserted",_identName,label);
	return(1);
}

int Counter::removeBlock(int label) {
	int start;
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	for(int i=0; i<_nBlockLabel; i++) {
		if(blockLabel[i] == label) {
			start = i*_nCountInBlock;
			if(start+_nCountInBlock >= _nData) {
				jdebug("start+_nCountInBlock = %d > _nData = %d",start+_nCountInBlock, _nData);
				return(0); // not found
			}
			_nData -= _nCountInBlock;
			for(int j=start; j<_nData; j++) {
				data[j] = data[j+_nCountInBlock];
			}
			_nBlockLabel--;
			for(int j=i; j<_nBlockLabel; j++) {
				blockLabel[j] = blockLabel[j+1];
			}
			jdebug("%s: block %d removed",_identName,label);
			return(1);
		}
	}
	jdebug("block with label %d not found!", label);
	return(0); // not found
}

int Counter::putData(count_t dataVal) {
	if(_nData < _maxNData) {
		data[_nData] = dataVal;
		_nData++;
		return SUCCESS;
	}
	return FAIL;
}

int Counter::setOutNumber(int number) {
	if(number > _maxNData) return FAIL;
	_nData = number;
	return SUCCESS;
}

int Counter::searchTestErrors(count_t test_number) {
	int test_errors = 0;
	for(int i=0; i<_nData; i++) {
		test_number--;
		count_t derr = abs(data[i]-test_number);
		if(derr != 0) {
			test_errors += derr; 
			test_number = data[i];
		}
	}
	if(_nData > 0) test_errors += data[_nData-1]; //yesli ne vse otshioty prishli
	return test_errors;
}

int Counter::setLevel(double level, int dialog) {
	jdebug("counter::setLevel");

	int levelValue =
		int(rint(255+_const2-rint(level*(255.0+_const2-_const1))));

	if(levelValue < MIN_LEVEL_VALUE) levelValue = MIN_LEVEL_VALUE;
	if(levelValue > MAX_LEVEL_VALUE) levelValue = MAX_LEVEL_VALUE;

	return setValue(uchar(levelValue),_levelValue,_setLevel,_getLevel,
							"Error in setLevel!", dialog);
}

double Counter::getLevel(void) {
	return double(255.0+_const2-_levelValue)
					/ double(255.0+_const2-_const1);
}
