/*--------------------------------------------------------------------------*/
/** @file counter.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with photometric modules.

                  MASS project: TURBINA module file
                       header file for counter.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 2.0
*/
#ifndef COUNTER_H
#define COUNTER_H

#include "procounter.h"
#include "iocount.h"

class Counter : public ProCounter {
private:
	const char*	_channel;
	uchar _setLevel;
	uchar _getLevel;

	count_t		*data;
	int			_nData;

	int			_levelValue;

	double		_workLevel;
	double		_nonLinearity;
	double		_nonPoisson;

public:
	Counter(void);
	virtual ~Counter();

	virtual void resetVariables(void); //!!! virtual, t.k. vyzyvaetsia iz roditelia!!!
	void create(int sizeOfData);
	virtual int loadOwnConst(void);
	virtual int loadConst(void);

	void resetData(void) {_nData=0; _nBlockLabel=0;}
	void resizeData(int newSize);
	/**
	@return	The fact that data were expanded due to the loss */
	bool expandData(void);
	int repairData(void);
	int insertBlock(int label, count_t value);
	int removeBlock(int label);
	int putData(count_t dataVal);
	count_t* getData(void) {return(data);} 
	int getOutNumber(void) {return(_nData);}
	int setOutNumber(int number);
	int searchTestErrors(count_t test_number);

	void setChannel(const char* channel) {_channel=channel;}
	const char* getChannel(void) {return _channel;}

	int setLevel(double level, int dialog=1);
	double getLevel(void);	// stoit na samom dele
	double getWorkLevel(void) {	return _workLevel; }// dolzhen byt' (iz device.cfg)
	double getNonPoisson(void) { return _nonPoisson; }
	double getNonLinearity(void) { return _nonLinearity; }
};

#endif
