#include <math.h>
#include "module.h"
#include "tools.h"
#include "exchange.h"
#include "cfg.h" 

HardModule::HardModule(void) {
	jdebug("HardModule::HardModule");
	_ready = FALSE;
	jdebug("end of HardModule's constructor");
}

HardModule::~HardModule() {
}

int HardModule::loadOwnConst(void) {
	jdebug("HardModule: LoadOwnConst");
	bool first = true;
	leafDataToChar(_module,first,deviceCfgFile,_name,_identSection,"Module");
	leafDataToChar(_getCRC,first,deviceCfgFile,_name,"Command","GetCrc");
	leafDataToChar(_getStatus,first,deviceCfgFile,_name,"Command","GetStatus");
	leafDataToChar(_getIdent,first,deviceCfgFile,_name,"Command","GetIdent");
	leafDataToChar(_getConst,first,deviceCfgFile,_name,"Command","GetConst");
	leafDataToChar(_reset,first,deviceCfgFile,_name,"Command","Reset");

	const char* id;
	leafDataToStr(&id, first,deviceCfgFile,_name,_identSection,"Identification");
	sscanf(id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4);

	_ready = true;
	jdebug("end of HardModule's loadOwnConst");
	if(_module == 0) return FAIL;
	return SUCCESS;
}

// -------------------------  --------------------------

int HardModule::reset(int dialog) {
	jdebug("HardModule::reset");

	resetVariables();

	int retval;
	if((retval=exch::sendReset(_module,_reset)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getConst(int dialog) {
	jdebug("HardModule::getConst");
	uchar buf[NCONST];
	int retval;
	if((retval=exch::askData(_module,_getConst,buf,NCONST)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	_const1 = (int)buf[0];
	_const2 = (int)buf[1];
	_const3 = (int)buf[2];
	_const4 = (int)buf[3];
	return SUCCESS;
}

int HardModule::testIdent(int dialog) {
	uchar buf[NIDENT];
	int retval;
	if((retval=exch::askData(_module,_getIdent,buf,NIDENT)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	if((_ident1 != buf[0]) || (_ident2 != buf[1]) ||
			(_ident3 != buf[2]) || (_ident4 != buf[3])) {
		const char* errText = "Invalid Identification!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getValue(uchar& value, uchar getCommand,
				int dialog, bool saveToLog) {
	jdebug("HardModule::getValue(uchar)");
	int retval;
	if((retval=exch::askByte(_module, getCommand,value,saveToLog)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getValue(int& value, uchar getCommand,
				int dialog, bool saveToLog) {
	jdebug("HardModule::getValue(int)");
	int retval;
	if((retval=exch::askWord(_module, getCommand,value,saveToLog)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendCommand(uchar command, int dialog) {
	int retval;
	if((retval=exch::sendCommand(_module,command)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, uchar value, int dialog) {
	jdebug("HardModule::sendValue(uchar)");
	int retval;
	if((retval=exch::sendByte(_module,command,value)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, int value, int dialog) {
	jdebug("HardModule::sendValue(int)");
	int retval;
	if((retval=exch::sendWord(_module,command,value)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, uchar value1, uchar value2,
														 int dialog) {
	jdebug("HardModule::sendValue(value1,value2)");
	uchar buf[4];
	buf[0] = _module;
	buf[1] = command;
	buf[2] = value1;
	buf[3] = value2;
	int retval;
	if((retval=exch::sendData(buf,4)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

// ---------------------------   ----------

int HardModule::turn(int state,int& currentState,uchar setOn,uchar setOff, 
			int dialog,int waitTime) { // state = ON or OFF
	jdebug("HardModule::turn");
	if(currentState == state) return SUCCESS;

	uchar command = setOff;
	if(state == ON) command = setOn;
	currentState = UNDEFINED;
	if(sendCommand(command,dialog) != SUCCESS) return FAIL;
	currentState = state;
	if(waitTime && (state==ON)) wait_ms(waitTime);
	return SUCCESS;
}

// class X - int or uchar
template <class X> int HardModule::setValue_(X value, int& currentValue,
				uchar setCommand, uchar getCommand,
				const char* failMessage,
				int dialog,
				int waitTime) {
	jdebug("HardModule::setValue<X>");
	const char* errText;
	if(value == (X)currentValue) return SUCCESS;
	currentValue = UNDEFINED;

	if(sendValue(setCommand, value, dialog) != SUCCESS) return FAIL;

	X newValue;
	if(getValue(newValue, getCommand, dialog) != SUCCESS) return FAIL;
	if(newValue != value) {
		errText = failMessage;
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	currentValue = value;
	if(waitTime && (currentValue != 0)) wait_ms(waitTime);
	return SUCCESS;
}

int HardModule::setValue(uchar value, int& currentValue,
				uchar setCommand, uchar getCommand,
						const char* failMessage,
						int dialog,
						int waitTime) {
	jdebug("HardModule::setValue(uchar)");
	return setValue_(value, currentValue, setCommand, getCommand,
									failMessage, dialog, waitTime);
}

int HardModule::setValue(int value, int& currentValue,
				uchar setCommand, uchar getCommand,
						const char* failMessage,
						int dialog,
						int waitTime) {
	jdebug("HardModule::setValue(int)");
	return setValue_(value, currentValue, setCommand, getCommand,
									failMessage, dialog, waitTime);
}

const int CORRECT_CRC = 0;
const char* BadCRC = "EEPROM corrupted!";

int HardModule::testCRC(int dialog) {
	jdebug("HardModule::testCRC");
	uchar crc;
	if(getValue(crc,_getCRC,dialog) != SUCCESS) return FAIL;

	if(crc != CORRECT_CRC) {
		const char* errText = BadCRC;
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getStatus(uchar& status, int dialog, bool saveToLog) {
	jdebug("HardModule::getStatus");
	return getValue(status, _getStatus, dialog, saveToLog);
}

int HardModule::isMotion(int dialog) {
	int retval = exch::sendTest(_module,_testMotion);
	if(retval == SignalACY) return NO;
// Incahe pust' udet motiion...
	return YES;
/*
	if(retval == SignalACW) return YES;
// Yesli ne tot signal prishel, to...
	const char* errText = exch::getErrTxt(-1*retval);
	reportError(_identName,errText,dialog);
	return UNDEFINED;
*/
}

int HardModule::stopMotion(int dialog) {
	jdebug("HardModule::stopMotion");
	return sendCommand(_stopMotion, dialog);
}
