#include <string.h>
#include "motor.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

extern FILE * out_file;
const QString UndefinedAperture = "Undefined";

Motor::Motor(void) {
	jdebug("motor::motor");
	_name = "Apertures";
	_identName = "Apertures";
	_identSection = "Common";
}

Motor::~Motor() {
}

void Motor::resetVariables(void) {
	jdebug("motor::resetVariables");
	_currentAperture	= UNDEFINED;
	_currentDoor		= UNDEFINED;
}

int Motor::loadOwnConst(void) {
	jdebug("motor::LoadOwnConst");
	bool first = true;
	leafDataToChar(_loadStops,first,deviceCfgFile,_name,"Command","LoadStops");
	leafDataToChar(_getStops,first,deviceCfgFile,_name,"Command","GetStops");
	leafDataToChar(_getSensors,first,deviceCfgFile,_name,"Command","GetSensors");
	leafDataToChar(_getPosition,first,deviceCfgFile,_name,"Command","GetPosition");
	leafDataToChar(_moveTo,first,deviceCfgFile,_name,"Command","MoveTo");
	leafDataToChar(_shiftAt,first,deviceCfgFile,_name,"Command","ShiftAt");
	leafDataToChar(_synchAt,first,deviceCfgFile,_name,"Command","SynchAt");
	leafDataToChar(_testMotion,first,deviceCfgFile,_name,"Command","TestMotion");
	leafDataToChar(_stopMotion,first,deviceCfgFile,_name,"Command","StopMotion");
	leafDataToChar(_setAperture,first,deviceCfgFile,_name,"Command","SetAperture");
	leafDataToChar(_getAperture,first,deviceCfgFile,_name,"Command","GetAperture");
	leafDataToChar(_setStep,first,deviceCfgFile,_name,"Command","SetStep");
	leafDataToChar(_getStep,first,deviceCfgFile,_name,"Command","GetStep");
	leafDataToChar(_nullSet,first,deviceCfgFile,_name,"Command","NullSet");
	leafDataToChar(_openDoor,first,deviceCfgFile,_name,"Command","OpenDoor");
	leafDataToChar(_closeDoor,first,deviceCfgFile,_name,"Command","CloseDoor");
	leafDataToChar(_powerDown,first,deviceCfgFile,_name,"Command","PowerDown");

	_nStops = 0;

	_stop[_nStops] = 1;
	leafDataToApertureData(_stopCorrect[_nStops], first,cfgFile,"Devices",
		_name,"Stop1",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 2;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop2",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 3;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop3",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 4;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop4",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 5;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop5",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 6;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop6",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 7;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop7",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	_stop[_nStops] = 8;
	leafDataToApertureData(_stopCorrect[_nStops],first,cfgFile,"Devices",
		_name,"Stop8",_stopName[_nStops],_stopFoclen[_nStops],out_file);
	if(_stopName[_nStops].upper() != "SPARE") _nStops++;

	jdebug("_nStops=%d,Stop2=%s focus[Stop2]=%f",_nStops,_stopName[1].data(),_stopFoclen[1]);
	if(_loadStops==0) return FAIL;
	return SUCCESS;
}

int Motor::loadConst(void) {
	jdebug("motor::LoadConst");
	if(HardModule::loadOwnConst() == FAIL) return FAIL;
	return loadOwnConst();
}

int Motor::getSensors(uchar& status, int dialog) {
	jdebug("motor::getSensors");
	return getValue(status, _getSensors, dialog);
}

int Motor::getPosition(int& position, int dialog) {
	jdebug("motor::getPosition");
	return getValue(position, _getPosition, dialog);
}

int Motor::setStep(uchar step, int dialog) {
	jdebug("motor::setStep");
	if(step < MIN_STEP) step = MIN_STEP;
	if(step > MAX_STEP) step = MAX_STEP;
	int currentStep = UNDEFINED;
	return setValue(step, currentStep, _setStep, _getStep,
								"Error in setStep!", dialog);
}

int Motor::loadStops(int dialog) {
	jdebug("motor::loadStops");

	uchar buf[MAX_N_STOPS];

	jdebug("motor::nStops=%d",_nStops);
	for(int i=0; i<_nStops; i++) {
		if(sendValue(_loadStops,(uchar)_stop[i],(uchar)_stopCorrect[i],dialog)
											!= SUCCESS) return FAIL;
	}

	int retval = exch::askData(_module, _getStops, buf, MAX_N_STOPS);
	if(retval != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	for(int i=0; i<MAX_N_STOPS; i++) {
		int nApp = UNDEFINED;
		for(int j=0; j<_nStops; j++) {
			if(i == _stop[j]) {
				nApp = j;
				break;
			}
		}
		if(nApp == UNDEFINED) continue;
		if((uchar)_stopCorrect[nApp] != buf[i]) {
			const char* errText = "Error in loadStops!";
			reportError(_identName,errText,dialog);
			return FAIL;
		}
	}
	return SUCCESS;
}

int Motor::getApertureNumber(const QString& apName, int dialog) {
	for(int i=0; i<_nStops; i++) {
		if(apName.upper() == _stopName[i].upper()) {
			return i;
		}
	}
	QString errText = "Invalid aperture " + apName;
	reportError(_identName,errText.data(),dialog);
	return FAIL;
}

double Motor::getFoclen(const QString& apName, int dialog) {
	int aperture = getApertureNumber(apName, dialog);
	if(aperture >= 0) return getFoclen(aperture);
	return 0;
}

double Motor::getFoclen(int aperture, int dialog) {
	if((aperture < 0) || (aperture >= _nStops)) return 0;
	return _stopFoclen[aperture];
}

int Motor::waitMotor(int dialog) {
	for(int i=0; i<MAX_MOTOR_TEST_INTERVALS; i++) {
		int retval = isMotion(dialog);		
		switch(retval) {
			case UNDEFINED	: return FAIL;
			case YES		: break;
			case NO			: 
					// Posle uspeshoi ustanovki apertury echio zhdiom 50 ms:
								wait_ms(APERTURE_DELAY);
								return SUCCESS;
        }
        wait_ms(DEVICE_WAIT_INTERVAL);
    }
    stopMotion(dialog);
    const char* errText = "Infinite Motion!";
    reportError(_identName,errText,dialog);
    return FAIL;
}

int Motor::setStop_(uchar absStopNumber, int dialog) {
	if(sendValue(_setAperture, absStopNumber, dialog) != SUCCESS) return FAIL;
	return waitMotor(dialog);
}

int Motor::setAperture(const QString& apName, int dialog,
											bool hard) {
	int aperture = getApertureNumber(apName,dialog);
	if(aperture >= 0) return setAperture(aperture,dialog,hard);
	return FAIL;
}

int Motor::setAperture(int aperture, int dialog, bool hard) {
	jdebug("motor::setAperture: Stop%d",_stop[aperture]);

	if(!hard) if(aperture == _currentAperture) return SUCCESS;

	if((aperture < 0) || (aperture >= _nStops)) {
		QString errText;
		errText.sprintf("Invalid aperture number %d",aperture);
		reportError(_identName,errText.data(),dialog);
		return FAIL;
	}
	_currentAperture = UNDEFINED;
	if(setStop_(_stop[aperture], dialog) != SUCCESS) return FAIL;
	_currentAperture = aperture;
	return SUCCESS;
}

int Motor::setStop(uchar absStopNumber, int dialog) {
	jdebug("motor::setStop: Stop%d",absStopNumber);

//	if((absStopNumber < 0) || (absStopNumber >= MAX_N_STOPS)) {
	if(absStopNumber >= MAX_N_STOPS) {
		QString errText;
		errText.sprintf("Invalid stop number %d",absStopNumber);
		reportError(_identName,errText.data(),dialog);
		return FAIL;
	}
	_currentAperture = UNDEFINED;
	if(setStop_(absStopNumber, dialog) != SUCCESS) return FAIL;
	for(int j=0; j<_nStops; j++) {
		if(_stop[j] == absStopNumber) {
			_currentAperture = j;
			break;
		}
	}
	return SUCCESS;
}

int Motor::moveTo(int position, int dialog) {
	jdebug("motor::moveTo %d",position);

	while((position < 0) || (position > MAX_POSITION)) {
		if(position < 0) position = MAX_POSITION + 1 + position;
		if(position > MAX_POSITION) position = position - MAX_POSITION - 1;
	}

	if(sendValue(_moveTo, position, dialog) != SUCCESS) return FAIL;
	if(waitMotor(dialog) != SUCCESS) return FAIL;
	int newPosition;
	if(getPosition(newPosition, dialog) != SUCCESS) return FAIL;
	if(newPosition != position) {
		const char* errText = "Error in moveTo!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int Motor::synchAt(char shift, int dialog) {
	jdebug("motor::syncAt");
	return sendValue(_synchAt, (uchar)shift, dialog);
}

int Motor::shiftAt(char shift, int dialog) {
	jdebug("motor::shiftAt %d",shift);
	if(sendValue(_shiftAt, (uchar)shift, dialog) != SUCCESS) return FAIL;
	return waitMotor(dialog);
}

int Motor::turnDoor(const char* state, int dialog) {
	if(strcasecmp(state,"OPEN") == 0) return turnDoor(OPEN, dialog);
	return turnDoor(CLOSE, dialog);
}

int Motor::turnDoor(int state, int dialog) {
	jdebug("Motor::turnDoor %d", state);
	if((_currentDoor=testDoor(dialog)) == UNDEFINED) return FAIL;
	_currentAperture = UNDEFINED;
	if(turn(state, _currentDoor, _openDoor, _closeDoor, dialog)
								!= SUCCESS) return FAIL;
	if(waitMotor(dialog) != SUCCESS) return FAIL;
	_currentDoor = testDoor(dialog);
	if(_currentDoor != state) {
		const char* errText = "Error in turnDoor!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int Motor::testDoor(int dialog) {
	uchar status;
	if(getSensors(status, dialog) != SUCCESS) return UNDEFINED;
	if(status & DOOR) return CLOSE;
	else return OPEN;
}

int Motor::testInitStatus(int dialog) {
	jdebug("motor::testInitStatus");
	uchar status;
	if(getStatus(status, dialog) != SUCCESS) return FAIL;
	if(!(status & DOOR_CLOSE)) {
		_currentDoor = OPEN;
		const char* errText = "Input Door is not closed!";
		reportError(_identName,errText,dialog);
	} else _currentDoor = CLOSE;
	if(!(status & ZERO_POINT)) {
		const char* errText = "Null-Point in not defined!";
		reportError(_identName,errText,dialog);
	}
	return SUCCESS;
}

int Motor::nullSet(int dialog) {
	jdebug("motor::nullSet");
	if(_currentDoor != OPEN) {
		if(turnDoor(OPEN, dialog) != SUCCESS) return FAIL;
	}
	_currentAperture = UNDEFINED;
	_currentDoor = UNDEFINED;
	if(sendCommand(_nullSet, dialog) != SUCCESS) return FAIL;
	wait_ms(100);	// else SignalDSC... (?)
	return waitMotor(dialog);
}

int Motor::movePlus(int dialog) {
	jdebug("motor::movePlus");
	char shift = 1;
	if(_currentAperture == UNDEFINED) return FAIL;
	if(_stopCorrect_new[_currentAperture] >= MAX_SIGN_BYTE_VALUE) return FAIL;
	(_stopCorrect_new[_currentAperture])++;
	return shiftAt(shift,dialog);
}

int Motor::moveMinus(int dialog) {
	jdebug("motor::moveMinus");
	char shift = -1;
	if(_currentAperture == UNDEFINED) return FAIL;
	if(_stopCorrect_new[_currentAperture] <= MIN_SIGN_BYTE_VALUE) return FAIL;
	(_stopCorrect_new[_currentAperture])--;
	return shiftAt(shift,dialog);
}

int Motor::shiftAperture(int shift, int dialog) {
	jdebug("motor::shiftAperture %d", shift);
	int position;
	if(getPosition(position, dialog) != SUCCESS) return FAIL;
	jdebug("position=%d",position);
	_currentAperture = UNDEFINED;
	return moveTo(position+shift,dialog);
}

const QString& Motor::getStopName(int stopNumber) {
	if((stopNumber < 0) || (stopNumber >= _nStops)) {
		return UndefinedAperture;
	}
	return _stopName[stopNumber];
}

int Motor::getStopCorrect(void) {
	jdebug("Motor: currentApert = %d",_currentAperture); 
	return int(_stopCorrect[_currentAperture]);
}

int Motor::getStopCorrectNew(void) {
	jdebug("Motor: currentApert = %d",_currentAperture); 
	return int(_stopCorrect_new[_currentAperture]);
}

void Motor::startCorrection(void) {
	jdebug("motor::startCorrection");
	for(int i=0; i<_nStops; i++) {
		_stopCorrect_new[i] = _stopCorrect[i];
	}
}

void Motor::acceptCorrection(int dialog) {
	jdebug("Motor: Accept Correction");
	for(int i=0; i<_nStops; i++) {
		_stopCorrect[i] = _stopCorrect_new[i];
	}
	loadStops(dialog);

// Copy to tree:
	QString str,num,stopLeaf;
	for(int i=0; i<_nStops; i++) {
		str = _stopName[i];
		str.append("\t");
		num.setNum(_stopFoclen[i],'f',2);
		str.append(num);
		str.append(" ");
		num.setNum(_stopCorrect[i]);
		str.append(num);
		stopLeaf = "Stop";
		num.setNum(_stop[i]);
		stopLeaf.append(num);
		jdebug(" ________ str=%s leaf=%s",str.data(),stopLeaf.data());
		setLeafData(cfgFile, "Devices", _name, stopLeaf.data(), str);
	}
}
