/*--------------------------------------------------------------------------*/
/** @file mywidget.h
@brief  GUI-implementation of the program main window: menu structure, status 
bars etc. linking together all the components provided by widgets.h.

                  MASS project: TURBINA module file
                       header file for mywidget.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef MYWIDGET_H
#define MYWIDGET_H

#include <qmainwindow.h>
#include <qtoolbar.h>
#include <qbuttongroup.h>
#include <qpopupmenu.h>
#include <qstring.h>
#include <qlabel.h>
#include <qtoolbutton.h>
#include <qiconset.h>
#include <qprogressbar.h>
#include <qarray.h>

#include "widgets.h"
#include "cfgdialog.h"

class MyWidget : public QMainWindow {
    Q_OBJECT

public:
	MyWidget(QWidget *parent=0, const char *name=0);
	~MyWidget(void);
	void fitToFont(void);
private:

	void openProgressor(const QString&);
	void closeProgressor(void);
	void displayStarPosition(int code);
	void setLabelText(QLabel *label,
				const QString& text);
	int addItem(QMenuData* menuData,
			const char* helpText,
			bool withDeviceEnable=true,
			bool forRootEnable=false,
			bool countDisable=true,
			bool withSuperVisorDisable=true);

protected:
	void timerEvent(QTimerEvent* e);
	void keyPressEvent(QKeyEvent* k);
	void keyReleaseEvent(QKeyEvent* k);
	void closeEvent(QCloseEvent* e);
//	virtual void resizeEvent(QResizeEvent*);
//	virtual void moveEvent(QMoveEvent*);

private:
	int _fontSizeLarge;
	int _fontSizeSmall;
	int _winX;
	int _winY;
	int _winWidth;
	int _winHeight;
	int _showHA;
	int normalTimer;
	bool isTimerEventLocked;
	bool isMyQuitLocked;
	bool _updateHA;
//	bool _disableKeyEvent;

	DeviceState oldState;
	int oldStarPosition;
	bool isScenario;
	double scenarioTime; // ostalos' do finisha scenariya

	QToolBar *toolBarDevice,*toolBarObject,*toolBarProgram;
	QToolBar *toolBarButtons;
	DigitalClock* digitalclock;
#ifdef __MOTOR__
	QLabel *labelApert, *labelDoor;
#endif
#ifndef __COUNTER_BUTTON__
	QLabel *labelMirror;
#endif
	QLabel *labelTmpr, *labelDiaphragm, *labelStar, *labelZenit;
	QLabel *labelScenario, *labelMode, *labelScenarioTime, *labelModeTime;
	QLabel *labelTrack, *labelIllum, *labelHV;;

	QButtonGroup *playButtonGroup;
	QToolButton *svBigRedButton;
	QToolButton *playButton, *stopButton, *pauseButton;
	QToolButton *navigateButton;
	QToolButton *selectButton;

	QToolButton *raPlus, *raMinus, *decPlus, *decMinus; 

	QJKMainView* mainView;
	QProgressBar* progressor;
	QPopupMenu* file;
	QPopupMenu* measure;
	QPopupMenu* tests;
	QPopupMenu* help;
	QPopupMenu* config;
	QPopupMenu* tools;
	QPopupMenu* functions;
	QPopupMenu *scenario_set;
	QArray<int> scen_id;
	int illum_id, light_id, hv_id, safety_id, pb_id;
#ifdef __MOTOR__
	int door_id;
	QPopupMenu *aperture_set;
	QArray<int> apert_id;
#endif
#ifdef __COUNTER_BUTTON__
	int buttons_id;
#else
	int mirror_id;
#endif
	CFGDialog* cnfg;
// help:
	QArray<QString*> menu_help;
	int item_id;
	int stop_item;
	int play_item;
	int pause_item;
	int break_item;
	QArray<int> disabl_count;
	int n_dis;
	QArray<int> disabl_sv;
	int n_dis_sv;	
	QArray<int> device_enable;
	int n_dev;
	TalkDialog*		talkDialog;
	AdjustDialog*		adjustDialog;
	SelectStarDialog*	selectStarDialog;
	ShowStarDialog* 	showStarDialog;
	CommentDialog*		commentDialog;
	CenteringDialog*	centeringDialog;
	CorrectionDialog*	trackCalibrationDialog;
	IllumDialog*		illumDialog;
	LightDialog*		lightDialog;
	HVDialog*		hvDialog;
	BackCorrectionDialog*	backCorrectionDialog;
	CorrectionDialog* centerCorrectionDialog;
	HelpView*		helpView;
	bool		existTalkDialog;
	bool		existAdjustDialog;
	bool		existSelectStarDialog;
	bool		existShowStarDialog;
	bool		existCommentDialog;
	bool		existCenteringDialog;
	bool		existTrackCalibrationDialog;
	bool		existIllumDialog;
	bool		existLightDialog;
	bool		existHVDialog;
	bool		existBackCorrectionDialog;
	bool		existCenterCorrectionDialog;
	bool		existHelpView;

	void loadConst(bool first = true);
	void showStopScenario(void);
	void showPlayScenario(void);
	void enableSelectScenario(bool enable);

private:
	void disableDevice(bool disable);
	void disableTracker(bool disable);

private slots:
	void myQuit(void);
	void change(void);
	void slotEnableItems(void);
	void slotDisableItems(void);
	void slotEnableSV(void);
	void slotDisableSV(void);
	void slotHighlighted(int);
	void slotShowMessage(const char*);
	void slotShowMessage(const char*,int);
	void slotHelp(void);
	void slotPlayBack(void);
	void slotTalkDialog(void);
	void turnIllum(void);
	void turnLight(void);
	void switchHV(void);
	void turnSafety(void);
//#ifdef __COUNTER_BUTTON__
	void switchButtons(void);
//#endif
//#ifdef __MOTOR__
	void switchDoor(void);
	void setAperture(int);
//#endif
	void slotSetScenario(int);
	void slotAdjustDialog(void);
	void slotSelectStarDialog(void);
	void slotShowStarDialog(void);
	void slotCommentDialog(void);
//#ifdef __MOTOR__
	void slotCenteringDialog(void);
	void slotTrackCalibrationDialog(void);
	void slotCenterCorrectionDialog(void);
//#endif
	void slotIllumDialog(void);
	void slotLightDialog(void);
	void slotHVDialog(void);
	void slotBackCorrectionDialog(void);
	void setProgress(const QString&, double);
	void setMode(const QString&, double sec);
	void setScenario(const QString&, double);

	void playScenario(void);
	void stopScenario(void);
	void togglePauseMenu(void);
	void togglePause(void);
	void toggleSVbigRedButton(void);

	void slotTurnIllum(int);
	void slotTurnLight(int);
	void slotTurnHV(int);
	void slotTurnSafety(int);
//#ifdef __COUNTER_BUTTON__
	void slotTurnButtons(int);
//#else
	void slotTurnMirror(int);
//#endif
//#ifdef __MOTOR__
	void slotTurnDoor(int);
	void slotAperture(int);
//#endif
	void slotTemperature(double);

	int moveRAplus(void);
	int moveRAminus(void);
	int moveDecPlus(void);
	int moveDecMinus(void);
	void stopTrack(void);

	void normal(void);
	void generalized(void);
	void centerControl(void);
	void focusControl(void);
	void backgroundMeasurement(void);
	void fluxEstimation(void);
	void exchangeTest(void);
	void detectorTest(void);
	void statisticTest(void);
	void detectorCountingMeasurement(void);
	void detectorStatisticMeasurement(void);
};

#endif
