/*--------------------------------------------------------------------------*/
/** @file power.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with high voltage supply module.

                  MASS project: TURBINA module file
                       header file for power.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 2.0
*/
#ifndef POWER_H
#define POWER_H

#include "module.h"

#define HV_LOCK_WAIT		100		// 100 millisec
#define HV_WAIT_INTERVAL	3000	// 3 sec

// Status of power:

#define	HV_ON		0x01
#define SAFETY_ON	0x02
#define	OVR_LIGHT	0x04
#define HV_LOCK		0x08
#define	TEMPER_READY	0x40

#ifdef __COUNTER_BUTTON__
class Power : public HardModule {
#else
class Power : virtual public HardModule {
#endif

private:

	uchar _setVoltage;
	uchar _getVoltage;
	uchar _highOn;
	uchar _highOff;

	uchar _safetyOn;
	uchar _safetyOff;
	uchar _getTemper;

	int _safety;
	int _voltageValue;	// [0..255]

public:
	Power(void);
	virtual ~Power();

	virtual void resetVariables(void);
	virtual int loadOwnConst(void);
	virtual int loadConst(void);

	int setVoltage(double voltage,int dialog=1,
						int waitTime=HV_WAIT_INTERVAL);
	int getVoltage(void);
	int getTemper(double& temper, int dialog=1);
	int isOverLight(int dialog=1);
	int isHVLock(int dialog=1);
	int isHVOn(int dialog=1);
	int isSafetyOn(void) {return(_safety);}
	int testOverLight(int dialog=1);
	int turnHV(int state, int dialog=1);
	int turnHV(const char* state, int dialog=1);
	int turnSafety(int state, int dialog=1);
	int turnSafety(const char* state, int dialog=1);

};

#endif
