#include <math.h>
#include <stdlib.h>
#include <ctype.h>
#include "readmass.h"
#ifndef __DOS
#include <map>
#endif

// added 22.09.2003 to access standard C++ types without std::
using namespace std;

#ifdef READMASSTEST


int main(int argc, const char * argv[] ) {
	FILE * f1 = fopen("mass.in","rt");
	FILE * f2 = fopen("stm.in","rt");
#ifndef __DOS
	char type;
	while (type = rdm::readline(f1,'I',f2)) {
		cout << "T="<<type;
		int h,m,s;
		if (!rdm::isheader()) cout << " UT="<<rdm::getut()<<' ';
		switch(type) {
			case 'P':
				cout << "Par="<<rdm::getname()<<" Val="<<rdm::getvalue()<<'\n';
			break; case 'M':
				cout <<"Mode="<<rdm::getname();
				if (rdm::getvalue().length())
					cout <<" AirM="<<rdm::getvalue();
				cout <<'\n';
			break; case 'i': case 'j':
				cout << "Addr=" <<rdm::getaddr()<<'\n';
			break; case 'O':
				cout << "HR="<<rdm::getname()<<" is "<<rdm::getvalue()<<'\n';
			break; default:
				cout << rdm::getvalue() << '\n';
		}
	}
#else
	while (readline(f));
#endif

	int cnt = rdm::grabparams(f1);
	if (!cnt) {
		cerr << "No parameters found in mass-file\n";
	} else
		for (;;) {
			cout << cnt <<
				"'P'-records\nInput parameter name or [Enter] to exit:";
			string pname ;
			getline(cin,pname);
			if (!pname.length()) break;
			cout << pname << '=' << rdm::getvalue(pname) << '\n';
		}

	if (f1) fclose(f1);
	if (f2) fclose(f2);
	return 0;
}
#endif

#ifndef __DOS
namespace rdm {
	std::map<string,string> parmap;
#endif
	// line content storages: see documentation in readmass.h
	string value;
	string name;
	long addr = 0;
	char type = 0;
	bool ishead = false;
	long lastfpos = 0;	// mass-file position BEFORE readling line
	int uyear = -1;
	int umonth = -1;
	int uday = -1;
	int uth = -1;
	int utm = -1;
	int uts = -1;

	const int LONGEST = 500;	// longest mass-line length
	char buffer[LONGEST];		// reading buffer
	const int UTLEN = 8;	// UT representation length
	const int DATEUTLEN = 10+1+UTLEN;	// DATE+UT representation length
	const int LENBEG = 1+1+UTLEN;	// shortest mass-line length (prefix+UT)
#ifndef __DOS
}
#endif

char
#ifndef __DOS
rdm::
#endif
readline(FILE * mf) {
	if (!mf) return 0;
	lastfpos = ftell(mf) ;
	if (!fgets(buffer, LONGEST, mf) || ((int)strlen(buffer)<LENBEG)) return 0;

	// remove backspace character:
	char * cr = strrchr(buffer,'\n');
	if (cr) {
		// overwrite with a end-character:
		*cr = 0;
	}
	// get prefix:
	type = buffer[0];
	ishead = false;

	if (type=='#') {
		// comment or header line?
		ishead = !isspace(buffer[1]);

		if (ishead) {
			value = buffer;
			return type;
		}
	}
	char * rest = &(buffer[1]);
	while (*rest && isspace(*rest)) rest++; // prefix and space skipped

	int h=99,m=99,s=99;
	// version 1.6: date components:
	int yy=0,mm=0,dd=0;
	// first try to extract full date, if fails - the time only:
	if (sscanf(rest,"%d-%d-%d %d:%d:%d",&yy,&mm,&dd,&h,&m,&s)!=6) {
		if (sscanf(rest,"%d:%d:%d",&h,&m,&s)!=3) return 0;
		rest += UTLEN; // UT record skipped
	} else {
		// validate date:
		if (yy<2001 || yy>2100 || mm<1 || mm>12 || dd<1 || dd>31) return 0;
		uyear = yy; umonth = mm; uday = dd;
		rest += DATEUTLEN; // DATE+UT record skipped
	}
	// validate time: we allow m=60 and s=60 (trbtime.cpp bug)
	if (h<0 || h>=100 || m<0 || m>60 || s<0 || s>60) return 0;
	uth = h; utm = m; uts = s;

	while (*rest && isspace(*rest)) rest++; // spaces skipped

	if (!strlen(rest)) return 0;	// stop if empty string to the right of UT

	char * c;
	switch (type) {
		case 'P' :
			c = strchr(rest,'=');
			if (!c) return 0;
			*c = 0;	// set the terminate character
			value = c+1;	// all after first '='
			// turn the name into the upper case:
			for (char * ch=rest; (*ch)!=0; ch++) *ch = toupper(*ch);
			name = rest;	// all before '='
		break;
		case 'M' :
			// turn the name into the upper case:
			value = "";
			for (char * ch=rest; (*ch)!=0; ch++) *ch = toupper(*ch);
			// locate the possibly existing number at the end:
			// skip the spaces at the tail and cancel them:
			c=rest+strlen(rest)-1;
			while ( isspace(*c) && c!=rest) {*c=0; c--;}
			// skip non-spaces before this:
			if (c!=rest) while (!isspace(*c) && c!=rest) c-- ;
			// skip the spaces before the last word:
			if (c!=rest) while ( isspace(*c) && c!=rest) c-- ;
			if (c!=rest) { 	
				// still not the beginning: indeed there are several words.
				// Go back to the first space before the last word:
				c++ ;
				// try to convert the last word into the number:
				char * endptr;
				strtod(c,&endptr);
				if (endptr!=c) {
					// successful conversion: indeed number: isolate it in the "value":
					*c = 0;	// add a terminator for "name" on the 1st space after it
					// start value at the last word:
					do c++; while ( isspace(*c) && (c!=0));
					value = c;
				} 
			}
			name = rest;
		break;
		case 'O':
			// find and return the first single word (normally HR number):
			c = rest;
			while (*c && !isspace(*c)) c++; // non-spaces skipped
			if (c!=rest) {
				*c = 0;
				name = rest;
				c++;
				while (*c && isspace(*c)) c++; // spaces skipped
				value = c;
//printf("\tHR=%s, Pars=%s\n",name.c_str(),value.c_str());
			}
		break;
		case 'i': case 'j':
			// find and return the address, or -1 if not written
			c = strchr(rest,'@');
			if (!c)
				addr = NOADDR;
			else {
				*c = 0;	// terminate string for value here
				char * endptr;
				addr = strtol(c+1,&endptr,16);
				if (endptr==c+1) addr = NOADDR;	// error of recognition
//printf("\tAddr=%ld\n",addr);
			}
		// no break here: we need the rest of string as "value"
		default:
			value = rest;
//printf("\tRest=%s\n",value.c_str());
	}
	return type;
}

char
#ifndef __DOS
rdm::
#endif
gettype() { return type; }

long
#ifndef __DOS
rdm::
#endif
getstartpos() { return lastfpos; }

bool
#ifndef __DOS
rdm::
#endif
isheader() { return ishead; }

double
#ifndef __DOS
rdm::
#endif
getut(int * uuth, int * uutm, int * uuts) {
	if (uuth)
		*uuth = uth;
	if (uutm)
		*uutm = utm;
	if (uuts)
		*uuts = uts;
	if (uth>=0)
		return uth + utm/60. + uts / 3600.;
	else
		return -1;
}

double
#ifndef __DOS
rdm::
#endif
getut(int * uuyear, int * uumonth, int * uuday, int * uuth, int * uutm, int * uuts) {
	if (uuyear) *uuyear = uyear;
	if (uumonth) *uumonth = umonth;
	if (uuday) *uuday = uday;
	if (uuth) *uuth = uth;
	if (uutm) *uutm = utm;
	if (uuts) *uuts = uts;
	if (uth>=0)
		return uth + utm/60. + uts / 3600.;
	else
		return -1;
}

string
#ifndef __DOS
rdm::
#endif
getut() {
	const char UTFMT[] = "%02d:%02d:%02d";
	const char DATEUTFMT[] = "%04d-%02d-%02d %02d:%02d:%02d";
	const int LDATEUTFMT = 8+10+1;
	static char ut[LDATEUTFMT] = " ";
	if (uyear>0) {
		sprintf(ut,DATEUTFMT,uyear,umonth,uday,uth,utm,uts);
	} else {
		sprintf(ut,UTFMT,uth,utm,uts);
	}
	return string(ut);
}

string
#ifndef __DOS
rdm::
#endif
getname() { return name; }

string
#ifndef __DOS
rdm::
#endif
getvalue(const string pname) {
	#ifdef __DOS
	if (strlen(pname)) {
		return 0;
	#else
	if (pname.length()) {
		return parmap[pname];
	#endif
		// access the parameters map:
	} else {
		return value;
	}
}

long
#ifndef __DOS
rdm::
#endif
getaddr() { return addr; }

int 
#ifndef __DOS
rdm::
#endif
add2map(const string pname, const string pvalue) {
	if (pname.length())
		parmap[pname] = pvalue;
	return parmap.size() ;
}

#ifndef __DOS
int rdm::grabparams(FILE * mf) {
	int cnt = 0;
	char t = 0;
	parmap.clear();
	if (mf) fseek(mf,0,SEEK_SET);
	while ((t = readline(mf))!=0) {
		if (t=='P' && name.length() && value.length()) {
			// add to the parameter map:
			cnt++;
			add2map(name,value) ;
		}
	}
	return cnt;
}
#endif

char rdm::readline(FILE * mass, char sw, FILE * stm) {
	static string UTbeg, UTend;
	static bool switched = false;   // read stm if true
 
	// read current line:
	char res = rdm::readline(switched && (stm!=0) ? stm : mass);
 
	if( switched && stm ) {
		string UT;
		// skip predecessing lines (assumes sorted stm-file!)
		while( ((UT = rdm::getut())<UTbeg) && (res = rdm::readline(stm)) );
		// now: either EOF or UT>=UTbeg:
		if (!res || (UT>UTend)) {
			// no lines left or found: scope over or empty UTbeg-UTend selection
			switched = false;
			// get back the last-read line if read:
			if (res) fseek(stm,rdm::getstartpos(),SEEK_SET);
			// read back the primary file:
			res = rdm::readline(mass) ;
		} // else - Ok: the line's read.
	} else {
		if( res==sw ) {
			// switcher is encountered: check last field
			// check that there is UTend in the end:
			UTend = rdm::getvalue();
			// position of the beginning of UTend:
			int utEndPos = UTend.rfind(':') - 5;
			// locate "UTend":
			if (utEndPos > 0) {
				UTend = UTend.substr(utEndPos,8);
				// UTbeg assignment:
				UTbeg = rdm::getut();
//cout << UTbeg << "?" << UTend << "===" << utEndPos << '\n';
				// validate UTend relative to UTbeg:
				if ((UTend[2] == UTend[5]) &&
					// use character comparison valid here:
					(UTend>=UTbeg)) {
//cout << "UTbeg=" << UTbeg << " UTend=" << UTend << '\n';
					// Ok: limits are set.
					switched = true;
				}
			} //else:  old or invalid mass-file data: no switching
		}
	}
	return res;
}
