#include <pthread.h>
#include "scenariothr.h"
#include "tools.h"
#include "names.h"

extern  DevManager devmanager;

namespace scenariothr {
	bool count_cycle_condition = true;
	pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
	pthread_mutex_t countMutex = PTHREAD_MUTEX_INITIALIZER;
}

int scenariothr::scenario_cycle(void) {		// Scenario-thread cycle
//	atexit(quit_count_cycle);
	jdebug("count_cycle: try lockcount A...");
	pthread_mutex_lock(&countMutex);
	jdebug("count_cycle: lockcount A");
	while(getCycleCondition()) {
		jdebug("count_cycle: try lockcount B...");
		pthread_mutex_lock(&countMutex);
		jdebug("count_cycle: lockcount B");
		if(!getCycleCondition()) break;
		switch (trbscenario::getMode()) {
			case trbscenario::PLAYBACK_MODE           : devmanager.playback_mode(); break;
			case trbscenario::NORMAL_MODE             : devmanager.normal_mode(); break;
			case trbscenario::GENERALIZED_MODE        : devmanager.generalized_mode(); break;
			case trbscenario::EXCHANGE_TEST_MODE      : devmanager.exchange_test_mode(); break;
			case trbscenario::DETECTOR_TEST_MODE      : devmanager.detector_test_mode(); break;
			case trbscenario::STATISTIC_TEST_MODE     : devmanager.statistic_test_mode(); break;
			case trbscenario::DETECTOR_COUNTING_MODE  : devmanager.detector_counting_mode(); break;
			case trbscenario::DETECTOR_STATISTIC_MODE : devmanager.detector_statistic_mode(); break;
			case trbscenario::BACKGROUND_CORR_MODE    : devmanager.background_corr_mode(); break;
			case trbscenario::FLUX_ESTIMATION_MODE    : devmanager.flux_estimation_mode(); break;
			case trbscenario::CENTERING_MODE          : devmanager.centering_mode(); break;
			case trbscenario::CENTERING_CORR_MODE     : devmanager.centering_corr_mode(); break;
			case trbscenario::TRACK_CALIBRATION_MODE  : devmanager.track_calibration_mode(); break;
			case trbscenario::FOCUS_CONTROL_MODE      : devmanager.focus_control_mode(); break;
			case trbscenario::CENTER_CONTROL_MODE     : devmanager.center_control_mode(); break;
			case trbscenario::BACKGROUND_MEASUR_MODE  : devmanager.background_measur_mode(); break;
			case trbscenario::SCENARIO_MODE           : devmanager.scenario_mode(); break;
			case trbscenario::QUIT_PROGRAM            : devmanager.quitProgram(); break;
			case trbscenario::END_OF_PROGRAM          : jdebug("count_cycle: END OF PROGRAM"); break;
			default :
				jdebug("!!!!!!!! Invalid _count_mode = %d while=%d",trbscenario::getMode(), getCycleCondition());
				break;
		}
		jdebug("count_cycle:next cycle");
	}
	pthread_mutex_unlock(&countMutex);
	jdebug("count_cycle: unlocked C...");
	return 1;
}

void scenariothr::setCycleCondition(bool condition) {
	pthread_mutex_lock(&mutex);
	count_cycle_condition = condition;
	pthread_mutex_unlock(&mutex);
}

bool scenariothr::getCycleCondition(void) {
	pthread_mutex_lock(&mutex);
	bool condition = count_cycle_condition;
	pthread_mutex_unlock(&mutex);
	return condition;
}

void scenariothr::quitScenarioCycle(void) {
	jdebug("quitScenarioCycle()");
//	_count_mode = END_OF_PROGRAM;
	setCycleCondition(false);
	pthread_mutex_unlock(&countMutex);
	jdebug("end of quitScenarioCycle()");
}

void scenariothr::playScenario(void) {
//	jdebug("current scenarioScript: %s",scenarioScript[_currentScenarioNumber]->data());
	trbscenario::fixMode();
	trbscenario::setMode(trbscenario::SCENARIO_MODE);
	devmanager.cleanStopScenario();

	if(devmanager.startCounting(1) != SUCCESS) return;

	jdebug("playScenario: try unlock lockcount...");
	pthread_mutex_unlock(&countMutex);
	jdebug("playScenario: countMutex unlocked");
}

void scenariothr::startMode(trbscenario::mode which) {
	trbscenario::fixMode();
	trbscenario::setMode(which);

	bool testViewerMirror = trbscenario::testMirror(which);
	if(devmanager.startCounting(1,testViewerMirror) != SUCCESS) return;

	jdebug("scenariothr::startMode: try unlock lockcount...");
	pthread_mutex_unlock(&countMutex);
	jdebug("scenariothr::startMode: unlock lockcount");
}

void scenariothr::playBack(const char* fileName) {
	devmanager.setMassInputFileName(fileName);
	startMode(trbscenario::PLAYBACK_MODE);
}

void scenariothr::exchangeTest(void) {
	startMode(trbscenario::EXCHANGE_TEST_MODE);
}

void scenariothr::detectorTest(void) {
	startMode(trbscenario::DETECTOR_TEST_MODE);
}

void scenariothr::statisticTest(void) {
	startMode(trbscenario::STATISTIC_TEST_MODE);
}

void scenariothr::detectorStatisticMeasurement(void) {
	startMode(trbscenario::DETECTOR_STATISTIC_MODE);
}

void scenariothr::detectorCountingMeasurement(void) {
	startMode(trbscenario::DETECTOR_COUNTING_MODE);
}

void scenariothr::focusControl(void) {
	startMode(trbscenario::FOCUS_CONTROL_MODE);
}

void scenariothr::centerControl(void) {
	startMode(trbscenario::CENTER_CONTROL_MODE);
}

void scenariothr::normal(void) {
	startMode(trbscenario::NORMAL_MODE);
}

void scenariothr::generalized(void) {
	startMode(trbscenario::GENERALIZED_MODE);
}

void scenariothr::backgroundCorrection(matrix43* p_matrix) {
	devmanager.setCoef(p_matrix);
	startMode(trbscenario::BACKGROUND_CORR_MODE);
}

void scenariothr::backgroundMeasurement(void) {
	startMode(trbscenario::BACKGROUND_MEASUR_MODE);
}

void scenariothr::fluxEstimation(void) {
	startMode(trbscenario::FLUX_ESTIMATION_MODE);
}

void scenariothr::centerCorrection(PointDouble* p_point) {
	devmanager.setDelta(p_point);
	startMode(trbscenario::CENTERING_CORR_MODE);
}

void scenariothr::trackCalibration(PointDouble* p_point) {
	devmanager.setDelta(p_point);
	startMode(trbscenario::TRACK_CALIBRATION_MODE);
}

void scenariothr::centering(PointDouble* p_point) {
	devmanager.setDelta(p_point);
	startMode(trbscenario::CENTERING_MODE);
}

void scenariothr::setPause(bool isPause) {
	devmanager.setPause(isPause);
	jdebug("&&&&&&&&&&&&&&&&&& pauseScenario = 1");
	if(!isPause && !devmanager.isCounting()) playScenario();
}

void scenariothr::playInitialScenario(void) {
	if( devmanager.playInitialScenario() ) {
		devmanager.setCurrentScenario(0);
		playScenario();
	}
}
