/****************************************************************************/
/*                 MASS project: TURBINA module file                        */
/*                            scind.cpp                                     */
/*                    Description is given in scind.h                       */
/* Editor tabular size: 4                                                   */
/****************************************************************************/

/* Insert the definition SCINDTEST in compilation for the module self-tests:
gcc -c scind.c .... -DSCINDTEST=1 */
//#define SCINDTEST
//#define SCINDDEBUG
#include <math.h>
#include <stdio.h>
#include <map>
#include "scind.hxx"
#include "nrutil.h"
#include "nr.h"

using namespace std;
#ifdef SCINDTEST
/*---------------------------- TEST PART ---------------------------------*/
#define NTEST 10
/* Uncomment the definition SPEEDTEST to omit data re-simulation and printouts
to obtain the clean performance estimate: */
//#define SPEEDTEST

/*----- Comes normally from configuration file: cf_... vars and defs --------*/

/* Apertures: */
/* number of apertures in segmentor : */
const int CF_NCHAN = 4 ;
/* Physical size of segmentator annular apertures [cm] : */
double cf_aper_inn[] = { 0.000, 0.102, 0.188, 0.354};
double cf_aper_out[] = { 0.100, 0.185, 0.350, 0.650};
/* Magnification of the system: */
double cf_magnif = 20.0 ;

/* Exposure section: */
double cf_microexp = 1 ;	/* < microexposure: timespan per count in [ms] */
const int CF_BASENUMBER = 1000 ; 		/* number of microexposures per series */
const int CF_NMEAS = 10 ;			/* Number of base times in Accum. Time */

/* Detectors section: */
const double cf_deadtime[] = { 30E-6, 30E-6, 30E-6, 30E-6 };
//const double cf_deadtime[] = { 30E-12, 30E-12, 30E-12, 30E-12 };
/* < given for exposure time EXACTLY 1 [ms] !!! Here - about 30 ns */
//const double cf_nonpois[] = {  0.984,  0.993,  1.001,  0.989 };    
const double cf_nonpois[] = {  1.0,  1.0,  1.0,  1.0 };    
/* background counts measured in channels per [ms] */
//const double cf_bkgr[] = {0.030, 0.032, 0.040, 0.070}; 
const double cf_bkgr[] = {0., 0., 0., 0.};

const char CF_IDXFILE[] = "index.dat" ;
const char CF_MOMFILE[] = "rawdat.dat" ;
const char CF_AVGIDXFILE[] =  "aindex.dat" ;

/*------------- Other supplementary info: ------------------------*/
double sys_qe = 0.05 ;	/* System quantum efficiency */
double flux0 = 1E6 ; 	/* Photons per second per cm^2 from the mag=0 A0-star*/
#ifndef M_PI
#define M_PI (3.141592653589793228) 
#endif

int main( int ac, char **av ) {

/* Receiving buffers for channels counts: */
count_t channelA[CF_BASENUMBER] ;
count_t channelB[CF_BASENUMBER] ;
count_t channelC[CF_BASENUMBER] ;
count_t channelD[CF_BASENUMBER] ;
count_t * channel[] = { (count_t*)(&channelA), (count_t*)(&channelB), 
						(count_t*)(&channelC), (count_t*)(&channelD) } ;

double area, flux, magnitude=0, modul=0, utc = 0.0 ;

FILE * fdat = NULL ;	/* External counts file to reduce */
FILE * fidx = NULL ;	/* Index writing file */
FILE * fmom = NULL ;	/* Moments writing file */
FILE * favg = NULL ;	/* Average indices writing file */
#define DATALINE	(100)
char 	buf[DATALINE] ;	/* char. buffer to use */
char * s ; 	/* running pointer to this buffer */
const int STAMPLEN = (1+1+1+1+8+1) ;/* "# M HH.HHHHH", M=Mode symb; HH.HHHHH = UTC
hour */ 
char stamp[STAMPLEN] ;

int i, j, idum2=(-13),test, meas;
bool stattest = false;
long idum=(-13) ;
int ncorr[CF_NCHAN] = {0,0,0,0} ;

if (ac<2) {
	printf(" MASS project: SCIND module test\n");
	printf(" Use: %s <star magnitude> <modulation factor [0,1[> [STATTEST]\n",av[0]);
	printf(" or: \n");
	printf("       %s <datafile:  %d channel columns with Nx%d counts each>\n",
		av[0],CF_NCHAN, CF_BASENUMBER);
	return -1 ;
}

cout << "sizeof(count_t) = " << sizeof(count_t) << '\n';

if ((fdat=fopen(av[1],"rt"))==NULL) {
	magnitude = atof(av[1]) ;
	if (ac>2) modul = atof(av[2]) ;
	else modul = 0 ;
	if ((magnitude<-2) || (fabs(modul)>1)) {
		printf(" Error: magnitude must be >=-2; modul<=1.0\n");
		return -1 ;
	}
	stattest = (ac>3) ;
} else {
	/* File's open: read counts */
	printf("File %s reading mode\n",av[1]);
	stattest = false ;
}

if (stattest)
	printf("Statistical test of MASS activated\n");
	
//try {

//#define ISHEADER
#ifdef ISHEADER
	bool res = sc::isheader(fdat,"#I UT");
	cout << res << " on #I UT\n";
	res = sc::isheader(fdat,"#I UT");
	cout << res << " on #I UT\n";
	res = sc::isheader(fdat,"#J UT");
	cout << res << " on #J UT\n";
	res = sc::isheader(fmom,"#J UT");
	cout << res << " on #J UT\n";
	res = sc::isheader(fmom,"#J UT");
	cout << res << " on #J UT\n";
	res = sc::isheader(fdat,"#I");
	cout << res << " on #I\n";
	exit(0);
#endif

#ifdef IND_SEQNUM
/* 
	Test ind_seqnum():
*/
const int nvar = 7 ;
const char names[nvar][3] = {"A","D","AB","CD","CE","a","BA"} ;
for (int n=0; n<nvar; n++) {
	printf("ind_seqnum(%d,%s) => ",CF_NCHAN,names[n]);
	int i = sc::ind_seqnum(CF_NCHAN,names[n]); 
	if (nr::erget()) {
		printf("%s\n",nr::ermessage().c_str());
		printf("%s\n",nr::ercodemessage(nr::erget()));
		nr::erreset() ;
	} else {
		printf(" seq=%d\n",i);
	}
}
exit(0);
#endif

for (test=0; test<NTEST; test++) {
/* Cycle of all the system work scenario repetitions : */

	printf("Initialize the module.\n") ;
	sc::init(CF_BASENUMBER, CF_NMEAS*(1+stattest), CF_NCHAN); 
	
	printf("Initialize Channels.\n") ;
	for (i=0; i<CF_NCHAN; i++) {
		sc::chan_init(i, cf_bkgr[i], cf_deadtime[i], cf_nonpois[i], channel[i]);
//		printf("chan[%d].data => %x\n",i,(int)channel[i]);
	}

/*const char buff[] = "     37     151     425    1795     352    3799   17377 
105681     988    1688    1724    6875    7587   25787     166    2259   12455  
84055     574    1255    2056    5030    8523   24590      50     776    6198  
57075";
sc::str2mom(buff);*/
	printf("Opening the index output files %s and %s\n",
		CF_IDXFILE, CF_AVGIDXFILE);
	if ((fidx=fopen(CF_IDXFILE,"at"))==NULL) {
		fprintf(stderr,"File %s open error\n",CF_IDXFILE);
		return nr::ERFIO ;
	} else {
		/* write header: */
		sc::write(fidx,sc::DSI,"#i     UTC", "Address", true) ;
	}
	if ((favg=fopen(CF_AVGIDXFILE,"at"))==NULL) {
		fprintf(stderr,"File %s open error\n",CF_IDXFILE);
		return nr::ERFIO ;
	} else {
		sc::writeavg(favg,sc::DSI,0,"#I     UTC",0,true) ;
		sc::writeavg(favg,sc::AFLUX,0,"#F     UTC","Address N",true) ;
	}

	printf("Opening the moments output file %s\n",CF_MOMFILE);
	if ((fmom=fopen(CF_MOMFILE,"at"))==NULL) {
		fprintf(stderr,"File %s open error\n",CF_MOMFILE);
		return nr::ERFIO ;
	} else {
		/* write header: */
		sc::write(fmom,sc::MOM,"#m     UTC",0,true) ;
	}

	const double S2MS = 1000 ;

	for (meas=0; meas<CF_NMEAS*(1+stattest); meas++) {
		if (fdat==NULL) {
#ifdef SPEEDTEST
			if (meas==0) 
#endif			
			{
			printf("\nUT=%7.4fh: Simulate data in channels:\n", utc);
			for (i=0; i<CF_NCHAN; i++) {
				/* Uncomment following idum-initialization to make the
				scintillations themselves similar in all channels but leaving photon
				statistics independent (use of ran1() and ran2() ! ): */
//				idum = -13;
				double maxf = cf_magnif*cf_aper_out[i]/2 ; 
				double minf = cf_magnif*cf_aper_inn[i]/2 ;
				area = (maxf*maxf - minf*minf) * M_PI ;
				flux = flux0 * area * pow(10.0,-0.4*magnitude) * sys_qe ;
				printf ("%s: %5.2f cm2, flux=%10.1f ph/s =>", sc::apername(CF_NCHAN,i),
						area,flux) ;
				double mean = 0;
				maxf = 0 ;
				minf = 1e9 ;
				for (j=0; j<CF_BASENUMBER; j++) {
					double factor = 0 ;
					if (stattest) {
						if (!(j%4-1))
							factor = 1.0 + modul ;
						else {
							if (!(j%4-3))
								factor = 1.0 - modul ;
							else
								factor = 1.0 ;
						}
					} else {
						factor = 1.0 + modul*(2.0*nr::ran2(&idum) - 1.0);
					}
					if (factor<minf) minf=factor;
					else if (factor>maxf) maxf=factor;

					double fcount =	nr::poidev((flux*factor+cf_bkgr[i])
						*cf_microexp/S2MS, &idum2) ;
					channel[i][j] =	count_t(fcount) ;
					mean += fcount  ;
//					printf("%d ", channel[i][j]);
//					channel[i][j] = channel[0][j];
				}
				mean /= CF_BASENUMBER ;
				printf(" Mean count: %8.1f (modul:%4.1f%%-%5.1f%%)\n",mean,
				minf*100, maxf*100);
			}
			}
		} else {
			printf("Read data series from %s:\n",av[1]);
			for (j=0; j<CF_BASENUMBER; j++) {
				if (fgets(buf,DATALINE,fdat)==NULL) break ;
				s = buf ;
//				printf("line=%s\n",s);
				for (i=0; i<CF_NCHAN; i++) {
					channel[i][j] = atoi(strtok_r((i==0?s:NULL)," \t\n\r",&s));
//					printf("ch_%s[%d] = %d ",
//						sc::apername(CF_NCHAN,i),j,channel[i][j]) ;
				}
//				printf("\n");
			}
			if (j<CF_BASENUMBER) {
				/* Too little data read: */
				printf("File END\n");
				break ;
			}
		}
		sc::statflux(CF_BASENUMBER, ncorr, cf_microexp, meas+1, false) ;
		printf("Avg fluxes:");
		for (int ch=0; ch<CF_NCHAN; ch++) {
			printf(" %s:",sc::apername(CF_NCHAN,ch));
			double f = sc::getavgflux(false,ch) ;
			printf(sc::FLXFMT,sc::FLXDLM, sc::fluxprec(f), f);
			printf(sc::ERRFMT,"+-",sc::geterrflux(false,ch));
		}
		printf("\n");
#ifndef SPEEDTEST
#ifdef LOGIDX
		printf("\nCompute scintillation log-indices:\n");
#else
		printf("\nCompute scintillation indices:\n");
#endif
#endif
		sc::calcmom(CF_BASENUMBER, ncorr) ;

		sprintf(stamp,"m %8.5f",utc);
		sc::write(fmom,sc::MOM,stamp) ;
		sc::calcidx(cf_microexp, false) ;

		utc += cf_microexp * CF_BASENUMBER / 3600 / S2MS ;
#ifndef SPEEDTEST
		if (nr::erget()) {
			printf("%s\n",nr::ermessage().c_str());
			printf("%s\n",nr::ercodemessage(nr::erget()));
			nr::erreset() ;
			continue;
		}
		sprintf(stamp,"i %8.5f",utc);
		const char dummy_addr[] = "@01357ace" ;
		sc::write(fidx,sc::DSI,stamp,dummy_addr) ;
		if (stattest) {
			/* Adds another row in the storage matrices: */
			sc::stattest(CF_BASENUMBER, cf_microexp) ;
			sprintf(stamp,"e %8.5f",utc);
			sc::write(fidx,sc::DSI,stamp,dummy_addr) ;
			sprintf(stamp,"m %8.5f",utc);
			sc::write(fmom,sc::MOM,stamp) ;
			meas ++ ;
		}
		printf("%4s  %8s  %8s  %8s  %8s  %8s\n",
			"Chan","sc_ind","sc_desi","mean","cmean","sig/cov");
		for (j=0; j<sc::napcomb(CF_NCHAN); j++) {
			for (int k=sc::getcurmeas()-1-stattest; k<=sc::getcurmeas()-1; k++) {
				printf("%2s  %8.1E  ",sc::apername(CF_NCHAN,j),
					sc::getidx(k,sc::DSI,j));
				if (j<CF_NCHAN)
					printf("%8.1E  ", sc::getidx(k,sc::DESI,j)) ;
				else
					printf("%8s  ", "-") ;
				if (j<CF_NCHAN) 
					printf("%8.1E  %8.1E  %8.1E\n", 
						sc::getmean(0,j), sc::getmean(1,j),	sc::getsig(0,j)) ;
				else
					printf("%8s  %8s  %8.1E\n","-","-", sc::getsig(0,j));
			}
		}
#endif
	}
	/* Average indices: */
	printf("\nAverage scintillation indices\n");
	sc::average(stattest,sc::MAXLAG);

	printf("Ok. Here they are:\n");
	printf("Chan\tSI       e_SI      DESI      e_DESI\n");
	for (i=0; i<sc::napcomb(CF_NCHAN); i++) {
		printf("%2s\t%8.1E",sc::apername(CF_NCHAN,i),
				sc::getavgidx(sc::DSI,false,i));
		printf(sc::ERRFMT," ",sc::geterridx(sc::DSI,false,i));
		if (i<CF_NCHAN) {
			printf("%8.1E", sc::getavgidx(sc::DESI,false,i));
			printf(sc::ERRFMT, " ",sc::geterridx(sc::DESI,false,i));
		}
		printf("\n");
	}
	sprintf(stamp,"I %8.5f",utc);
	sc::writeavg(favg,sc::DSI,sc::MODENORM,stamp) ;
	if (stattest) {
		sprintf(stamp,"J %8.5f",utc);
		sc::writeavg(favg,sc::DSI,sc::MODEGEN,stamp) ;
	}
	sprintf(stamp,"F %8.5f",utc);
	sc::writeavg(favg,sc::AFLUX,sc::MODENORM,stamp,"@13579ace 1") ;

	if (fdat!=NULL) {
		fdat = freopen(av[1],"rt",fdat);
	}
	/* Leave indices */
	sc::again() ;
	printf("Release module arrays:\n") ;
	sc::done() ;
	fclose(fidx) ;
	fclose(fmom) ;
	fclose(favg) ;
	printf("===================== %d ===================\n",test);

} /* of test-cycle */

/*
catch(std::domain_error d) {
	cout << "Domain exception " << d.what() << "\n";
}
catch(std::invalid_argument a) {
	cout << "Inval.Arg. exception " << a.what() << "\n";
}
catch(std::out_of_range o) {
	cout << "Bad numeric parameter exception " << o.what() << "\n";
}
catch (sc::dataproblem d) {
	cout << "Data problem exception " << d.what() << "\n";
}
catch(std::runtime_error s) {
	cout << "Std run-time exception: " << s.what() << "\n" ;
}
*/
/** Alternative method to detect an error: */
if (nr::erget()) {
	cout << "Error: " << nr::ermessage() << "\n";
	nr::erreset() ;
}
if (fdat) fclose(fdat);
printf("Ok.\n");

return 0 ;
} /* end of main() */
#endif

namespace sc {

/** This structure contains the PMT count series for which the scintillation
index should be computed. To simplify the data handling, the descriptive
parameters of this series and the detector are included in this structure.
The length of the data buffer is not specified since it may vary. */
struct chan_t {
	count_t * data ;		/**< Array of counts */
	double nonpois ;		/**< non-Poisson factor (dispersion/mean) */
	/** background level (scaled to the used micro-exposure time) */
	double bkgr ; 			
	double deadtime ;		/**< deadtime (divided by micro-exposure) */
	chan_t() { data = 0; nonpois = deadtime = bkgr = 0 ; }
} ;

/*---------------------------------------------------------------------*/
/*----- Static data storages accessible by functions get...   ------*/
/*----- they are allocated by init() and released be done() -----*/
/*---- We use these global arrays to avoid their multiple -------------*/
/*---- allo/deallocation with all the times the same dimentions -------*/

double ** scgl_dsidx = NULL ;	/**< DSI matrix[Nmeas][Nidx+1]: last column is
for SC_D/SC_A ratio for filtering star excapes */
double ** scgl_desidx = NULL ;	/**< DESI matrix[Nmeas][Ndesi] */
double * scgl_adsi = NULL ; 	/**< average DSI vector[Nidx*2] */
double * scgl_e2dsi = NULL ;	/**< error2 of avg. DSI vector[Nidx*2] */
double * scgl_adesi = NULL ;	/**< average DESI vector[Nchan*2] */
double * scgl_e2desi = NULL ;	/**< error2 of avg DESI vector[Nchan*2]*/
double * scgl_aflux = NULL ;	/**< average flux vector[Nchannel] */
double * scgl_e2flux = NULL ;	/**< error2 of avg.flux vector[Nchannel] */
double * scgl_mean = NULL;  	/**< mean counts vector[Nchan] */
double ** scgl_cmean = NULL ;	/**< corrected means matrix[Nchan][Nmeas] */
double * scgl_sig = NULL ;  	/**< normalized count disp.and covar. vector[Nidx] */
double * scgl_r1 = NULL ;   	/**< normalized lag-1 covar.vector[Nidx] */
double * scgl_r2 = NULL ;		/**< normalized lag-2 covar.vector[Nchan] */
sc::chan_t * scgl_channel = NULL ;	/**< channel structure vector[Nchan] */

#ifdef LOGIDX
double * scgl_logmean = NULL ;	/**< Mean ln(count) vector[Nchan] */
double ** scgl_logi = NULL ;	/**< ln(count) matrix[Nchan][Ndata] */
#endif

signed char * scgl_isgen = NULL ;   	/**< Shifted-mode flag vector[Nmeas] */

/* Memory is allocated in init() and freed in done() according to: */
int scgl_maxnmeas = 0 ;/**< Maximal number of measurements to store Nmeas*/
int scgl_maxndata = 0 ;/**< Maximal number of counts in buffers */
int scgl_nchan = 0 ;	/**< Current channels number Nchan */
int scgl_nindex = 0 ;	/**< Respective current Nidx as napcomb(scgl_nchan) */
/* actual number "nmeas" of measurements made is got from scgl_curmeas; 
  actual number of counts put in buffer is given directly to calcidx() */
int scgl_curmeas = -1 ;/**< Current measurement number */
double scgl_curexp = 1 ;	/** current microexposure */

double scgl_bad = 99 ; /**> Address of this variable will be returned when
		there is nothing to return as a double-pointer */

} // of reopened namespace sc

/*-------------------------- getcurmeas ----------------------------------*/

int sc::getcurmeas() 
//throw (std::domain_error) 
{
	if (scgl_curmeas<0) {
//		throw std::domain_error(
		nr::nrerror("SCIND: getcurmeas() while uninitialized",nr::ERNUL) ;
	}
	return scgl_curmeas ;
}

/*-------------------------- getidx ---------------------------------------*/

double sc::getidx(int meas, scwhat what, int i) 
//throw (std::domain_error,std::out_of_range,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
//		throw std::domain_error(
		nr::nrerror("SCIND: getidx() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}

	if ((meas>scgl_maxnmeas) || (meas<0)) {
//		throw std::out_of_range(
		nr::nrerror("SCIND: getidx(): wrong measurement number",nr::ERPAR);
		meas = 0;	/* to return 1st measurement instead of NULL */
	}
	
	switch(what) {
		case DSI:	return scgl_dsidx[meas][i] ;
		case DESI:	return scgl_desidx[meas][i] ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: getidx(): wrong what-selection",nr::ERPAR) ;
			return scgl_bad; /* return something */
	}
}

/*-------------------------- getidxptr ---------------------------------------*/

double * sc::getidxptr(int meas, scwhat what) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: getidxptr() while uninitialized",nr::ERNUL) ;
		return &scgl_bad ;
	}

	if ((meas>scgl_maxnmeas) || (meas<0)) {
//		throw std::out_of_range(
		nr::nrerror("SCIND: getidxptr(): wrong measurement number",nr::ERPAR);
		meas = 0;	/* to return 1st measurement instead of NULL */
	}
	
	switch(what) {
		case DSI:	return scgl_dsidx[meas] ;
		case DESI:	return scgl_desidx[meas] ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: getidxptr(): wrong what-selection",nr::ERPAR) ; 
			return &scgl_bad; /* return something */
	}
}

/*-------------------------- getavgidx ---------------------------------------*/

double sc::getavgidx(scwhat what, bool isgen, int i) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: getavgidx() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}

	switch(what) {
		case DSI:	return (scgl_adsi  + int(isgen)*scgl_nindex)[i] ;
		case DESI:	return (scgl_adesi + int(isgen)*scgl_nchan)[i] ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: getavgidx(): wrong what-selection",nr::ERPAR) ; 
			return scgl_bad; /* return something */
	}
}

/*-------------------------- getavgidxptr ---------------------------------------*/

double * sc::getavgidxptr(scwhat what, bool isgen) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: getavgidxptr() while uninitialized",nr::ERNUL) ;
		return &scgl_bad ;
	}
	switch(what) {
		case DSI:	return (scgl_adsi  + int(isgen)*scgl_nindex) ;
		case DESI:	return (scgl_adesi + int(isgen)*scgl_nchan) ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: getavgidxptr(): wrong what-selection",nr::ERPAR) ; 
			return &scgl_bad; /* return something */
	}
}

/*-------------------------- geterridx ---------------------------------------*/

double sc::geterridx(scwhat what, bool isgen, int i) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: geterridx() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}
	double e2 = 0 ;
	double v = 0 ;
	switch(what) {
		case DSI:	e2 = (scgl_e2dsi  + int(isgen)*scgl_nindex)[i] ;
					v = (scgl_adsi  + int(isgen)*scgl_nindex)[i] ;
			break ;
		case DESI:	e2 = (scgl_e2desi + int(isgen)*scgl_nchan)[i] ;
					v = (scgl_adesi + int(isgen)*scgl_nchan)[i] ;
			break ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: geterridx(): wrong what-selection",nr::ERPAR) ; 
			return scgl_bad; /* return something */
	}
	return (e2>0 && v>0) ? sqrt(e2)/v : 0 ;
}

/*-------------------------- geter2idxptr ---------------------------------------*/

double * sc::geter2idxptr(scwhat what, bool isgen) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: geter2idxptr() while uninitialized",nr::ERNUL) ;
		return &scgl_bad ;
	}
	switch(what) {
		case DSI:	return (scgl_e2dsi  + int(isgen)*scgl_nindex) ;
		case DESI:	return (scgl_e2desi + int(isgen)*scgl_nchan) ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: geter2idxptr(): wrong what-selection",nr::ERPAR) ; 
			return &scgl_bad; /* return something */
	}
}

/*-------------------------- getmean --------------------------------------*/

double sc::getmean(bool iscor, int i, int meas)
//throw (std::domain_error) 
{

	if (iscor) {
		int m = (meas>=0 && meas<scgl_curmeas) ? meas : scgl_curmeas-1;
		if (m<0) {
			//throw std::domain_error(
			nr::nrerror("SCIND: getmean(true) on non-existant measurement",nr::ERNOD) ;
			return scgl_bad ;
		}
		return scgl_cmean[m][i]/scgl_curexp ;
	} else
		return scgl_mean[i]/scgl_curexp ;
}

/*-------------------------- getavgflux -----------------------------------*/

double sc::getavgflux(bool isgen, int i) 
//throw (std::domain_error) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: getavgflux() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}
	return (scgl_aflux + isgen*scgl_nchan)[i] / scgl_curexp ;
}

/*-------------------------- geterrflux -----------------------------------*/

double sc::geterrflux(bool isgen, int i) 
//throw (std::domain_error) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: geterrflux() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}
	double e2 = (scgl_e2flux + isgen*scgl_nchan)[i] 
		/ scgl_curexp / scgl_curexp  ;
	double v = (scgl_aflux + isgen*scgl_nchan)[i] / scgl_curexp ;
	return (e2>0 && v>0) ? sqrt(e2)/v : 0 ;
}

/*-------------------------- getsig --------------------------------------*/

double sc::getsig(int lag, int i) 
//throw (std::domain_error,std::invalid_argument) 
{
	if (scgl_curmeas<0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: getsig() while uninitialized",nr::ERNUL) ;
		return scgl_bad ;
	}

	switch (lag) {
		case 0: return (scgl_sig[i])/scgl_curexp/scgl_curexp ;
		case 1:	return (scgl_r1[i])/scgl_curexp/scgl_curexp ;
		case 2:	return (scgl_r2[i])/scgl_curexp/scgl_curexp ;
		default:   //throw std::invalid_argument(
			nr::nrerror("SCIND: getsig(): wrong lag value",nr::ERPAR) ;
			return scgl_bad; /* return something */
	}
}

/*------------------------- apername --------------------------------------*/
const char * sc::apername(int nchan, int index) 
//throw(std::out_of_range) 
{
	static char name[3] ;
	if ((index<0) || (nchan<0)) {
		//throw std::out_of_range(
		nr::nrerror("SCIND: apername(): wrong parameter",nr::ERPAR);
		return "" ;
	}
	
	if (index<nchan) {
		/* Normal index: */
		name[1] = '\0';
		name[0] = (char)((int)'A' + index);
		return name ;
	} else {
		/* Differential index: */
		int n = nchan;
		for (int i=0;i<nchan-1;i++)
			for (int j=i+1;j<nchan;j++)
				if ((n++)==index) {
					name[0] = (char)((int)'A' + i);
					name[1] = (char)((int)'A' + j);
					name[2] = '\0';
					return name ;
				}
	}
	/* If we drop here, then index > (number of apert. and their comb.) */
	//throw std::out_of_range(
	nr::nrerror("SCIND: apername(): wrong parameter",nr::ERPAR);
	return "";
}

/*------------------------- ind_seqnum --------------------------------------*/

int sc::ind_seqnum(int nchan, const char * name)
//throw(std::out_of_range) 
{
	if (nchan<0 || !name) {
		//throw std::out_of_range(
		nr::nrerror("SCIND: apername(): wrong parameter",nr::ERPAR);
		return -1 ;
	}

	if (!name[1]) {
		/* one-char name => normal index seq.number is required: */
		int i = (int)name[0] - (int)'A' ;
		if ((i>=0) && (i<nchan))
			return i ;
	} else {
		/* Differential index seq. number is required: */
		int n = nchan;
		const int len = 2 ;	/* (maximal) length of the index name: "A","BC" */
		char trial[len+1] = "00";
		for (int i=0;i<nchan-1;i++) {
			trial[0] = (char)((int)'A' + i) ;
			for (int j=i+1;j<nchan;j++) {
				trial[1] = (char)((int)'A' + j) ;
				if (!strncmp(trial,name,len))
					return n ;
				n++ ;
			}
		}
	}
	/* If we drop here, then no match for the name is found */
	//throw std::out_of_range(
	nr::nrerror("SCIND: ind_seqnum(): wrong index name",nr::ERPAR);
	return -1 ;
}

/*---------------------------- init ----------------------------------*/

void sc::init(int ndata, int nmeas, int nchan) 
//throw(std::out_of_range,std::bad_alloc) 
{
//	if (nr::erget()) return ;
	if ((nmeas<=0) || (nchan<=0) || (ndata<=0)) {
		//throw std::out_of_range(
		nr::nrerror("SCIND: init(): wrong parameter",nr::ERPAR);
		return ;
	}

	/* If channel number is changed - redo all: */
	if (nchan>scgl_nchan) {
		/* Free all memory if allocated: */
		if (scgl_nchan>0)
			done() ;
		
		/* Allocate vectors related to the number of channels only: */
		int nindex = napcomb(nchan) ;
		scgl_mean=nr::dvector(0,nchan-1) ;
#ifdef LOGIDX
		scgl_logmean=nr::dvector(0,nchan-1) ;
#endif
		scgl_sig=nr::dvector(0,nindex-1) ;
		scgl_adsi=nr::dvector(0,2*nindex-1) ;
		scgl_e2dsi=nr::dvector(0,2*nindex-1) ;
		scgl_adesi=nr::dvector(0,2*nchan-1) ;
		scgl_e2desi=nr::dvector(0,2*nchan-1) ;
		scgl_aflux=nr::dvector(0,2*nchan-1) ;
		scgl_e2flux=nr::dvector(0,2*nchan-1) ;
		scgl_r1=nr::dvector(0,nindex-1) ;
		scgl_r2=nr::dvector(0,nchan-1) ;
		/* Channel structures: */
		scgl_channel = new chan_t[nchan] ;
		scgl_nchan = nchan ;
		scgl_nindex = nindex ;
	} else {
		if (nchan!=scgl_nchan) {
			scgl_nchan = nchan ;
			scgl_nindex = napcomb(nchan) ;
		}
	}

	/* Data number in buffers changed: */
#ifdef LOGIDX
	/* logarithms of counts storage: */
	if (scgl_maxndata<ndata) {
		if (scgl_maxndata)
			nr::free_dmatrix(scgl_logi,0,scgl_nchan-1,0,scgl_maxndata-1) ;
		scgl_logi=nr::dmatrix(0,scgl_nchan-1,0,ndata-1) ;
		scgl_maxndata = ndata ;
	}
#endif		

	/* Maximal number of measurements changed: */
	if (scgl_maxnmeas<nmeas) {
		/* Generalized mode flags: */
		if (scgl_maxnmeas) {
			delete[] scgl_isgen ;
			nr::free_dmatrix(scgl_dsidx,0,scgl_maxnmeas-1,0,scgl_nindex) ;
			nr::free_dmatrix(scgl_desidx,0,scgl_maxnmeas-1,0,scgl_nchan-1) ;
			nr::free_dmatrix(scgl_cmean,0,scgl_maxnmeas-1,0,scgl_nchan-1) ;
		}
		
		scgl_isgen=new (signed char)[nmeas] ;
		/* Index storages: */
		// scgl_dsidx contains 1 element more for SC_A/SC_D:
		scgl_dsidx=nr::dmatrix(0,nmeas-1,0,scgl_nindex) ;
		scgl_desidx=nr::dmatrix(0,nmeas-1,0,scgl_nchan-1) ;
		scgl_cmean=nr::dmatrix(0,nmeas-1,0,scgl_nchan-1) ;
		scgl_maxnmeas  = nmeas ;
	} 

	/* Initialize measurements: */
	for (int i=0; i<scgl_maxnmeas; i++) 
		scgl_isgen[i] = MODENO ;	/* No measurement so far */
	scgl_curmeas = 0 ;
}

/*---------------------------- done ----------------------------------*/

inline void debug(const char* s) {
#ifdef SCINDDEBUG
	printf("SCIND:---------------%s----------------\n",s);
#endif
}


void sc::done() {
//	if (nr::erget()) return ;
	/* Check pointers to bear a signature "unallocated" - i.e. NULL, by
	our convention: */
	debug("sc::done()");
	if (scgl_nchan>0) {
		debug("delete average storages");
		nr::free_dvector(scgl_mean,0,scgl_nchan-1); scgl_mean = NULL ;
		nr::free_dvector(scgl_sig,0,scgl_nindex-1); scgl_sig = NULL ;
		nr::free_dvector(scgl_adsi,0,2*scgl_nindex-1) ; scgl_adsi = NULL ;
		nr::free_dvector(scgl_e2dsi,0,2*scgl_nindex-1) ; scgl_e2dsi = NULL ;
		nr::free_dvector(scgl_adesi,0,2*scgl_nchan-1) ; scgl_adesi = NULL ;
		nr::free_dvector(scgl_e2desi,0,2*scgl_nchan-1) ; scgl_e2desi = NULL ;
		nr::free_dvector(scgl_aflux,0,2*scgl_nchan-1) ; scgl_aflux = NULL ;
		nr::free_dvector(scgl_e2flux,0,2*scgl_nchan-1) ; scgl_e2flux = NULL ;
		nr::free_dvector(scgl_r1,0,scgl_nindex-1); scgl_r1 = NULL; 
		nr::free_dvector(scgl_r2,0,scgl_nchan-1); scgl_r2 = NULL; 
#ifdef LOGIDX
		nr::free_dvector(scgl_logmean,0,scgl_nchan-1); scgl_logmean = NULL; 
#endif
	}
#ifdef LOGIDX
	if (scgl_logi!=NULL) {
		nr::free_dmatrix(scgl_logi,0,scgl_nchan-1,0,scgl_maxndata-1);	scgl_logi = NULL; 
	}
#endif
		
	if (scgl_cmean!=NULL) { 
		debug("delete cmean[] storage");
		nr::free_dmatrix(scgl_cmean,0,scgl_maxnmeas-1,0,scgl_nchan-1); 
		scgl_cmean = NULL;}

	if (scgl_dsidx!=NULL) { 
		debug("delete index storages");
		// scgl_dsidx contains 1 element more for SC_A/SC_D:
		nr::free_dmatrix(scgl_dsidx,0,scgl_maxnmeas-1,0,scgl_nindex); 
		scgl_dsidx = NULL ; }
	if (scgl_desidx!=NULL) { nr::free_dmatrix(scgl_desidx,0,scgl_maxnmeas-1,0,scgl_nchan-1); 
		scgl_desidx = NULL ; }
	if (scgl_isgen!=NULL) { delete[] scgl_isgen; scgl_isgen = NULL ; }

	/* Channel structures: */
	if (scgl_channel != NULL) { delete[] scgl_channel ; scgl_channel = NULL ; }

	scgl_curmeas = -1 ;	/* Signals impossibility to store any index */
	scgl_nchan = scgl_nindex = 0 ;
	scgl_maxnmeas = 0 ;
	scgl_maxndata = 0 ;
	debug("done() Ok");
}

/*---------------------------- again ---------------------------------*/

void sc::again() 
//throw() 
{
//	if (nr::erget()) return ;
	if (scgl_curmeas>0) scgl_curmeas = 0 ;
}

/*---------------------------- chan_init -----------------------------*/

void sc::chan_init(int chan, double bkgr, double deadtime, 
	double nonpois, count_t * data) 
//throw (std::out_of_range, std::domain_error) 
{
//	if (nr::erget()) return ;
	if ((chan<0) || (chan>=scgl_nchan)) {
		//throw std::out_of_range(
		nr::nrerror("SCIND: chan_init(): invalid channel number",nr::ERPAR) ;
		return ;
	}
	if (!scgl_channel) {
		//throw std::domain_error(
		nr::nrerror("SCIND: chan_init() while uninitialized",nr::ERNUL) ;
		return ;
	}
	
	if (data) scgl_channel[chan].data = data;
	if (nonpois>=0) scgl_channel[chan].nonpois = nonpois;
	if (bkgr>=0) scgl_channel[chan].bkgr = bkgr;
	if (deadtime>=0) scgl_channel[chan].deadtime = deadtime;
}

/*--------------------------- write ---------------------------------*/

void sc::write(FILE * f, scwhat what, const char * stamp, 
	const char * suffix, bool header) 
//throw(std::domain_error, std::invalid_argument, sc::ioproblem) 
{
	if (nr::erget()) return ;

	if (scgl_nchan<=0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: write() while uninitialized",nr::ERNUL);
		return ;
	}

#ifdef NODUBHEAD
	// suppress the repetitive header writing:
	if (header && isheader(f,stamp)) return;
#endif

	if ((scgl_curmeas<=0) && !header && what!=sc::MOM) {
		//throw std::domain_error(
		nr::nrerror("SCIND: write() while no measurements done",nr::ERNOD);
		return ;
	} 

	if (stamp && (!fprintf(f,"%s",stamp))) {
		/* Failed write attempt: */
		//throw sc::ioproblem(
		nr::nrerror("SCIND: write(): cannot write file",nr::ERFIO) ;
		return ;
	}
	switch (what) {
	
		case DSI: case DESI:
			if (header) {
				/* File needs the header only. Create header: */
				/* Plain indices header: AUTOADJUSTMENT of the name to format*/
				// length of a number in current format: NO FLAGS ASSUMED
				char * endptr ;
				int fmtlen = strtol(rindex(IDXFMT,'%')+1, &endptr, 10) ;
				const char NORMSINAME[] = "SI_";	// plus channel ID
				string heading = string(IDXDLM)+NORMSINAME;
				int spaceadded = fmtlen-strlen(NORMSINAME)-1 ;
				string spaces = "";	for (int c=0; c<spaceadded;c++) spaces+=' ';
				
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%s%s%s",heading.c_str(),
					apername(scgl_nchan,i),spaces.c_str());


				const char DIFFSINAME[] = "DSI_";	// plus 2 channel IDs
				heading = string(IDXDLM)+DIFFSINAME;
				spaceadded = fmtlen-strlen(DIFFSINAME)-2 ;
				spaces = ""; for (int c=0; c<spaceadded;c++) spaces+=' ';

				for (int i=scgl_nchan;i<scgl_nindex;i++) 
					fprintf(f,"%s%s%s",heading.c_str(),
					apername(scgl_nchan,i),spaces.c_str());

				const char DESINAME[] = "DESI_";	// plus 1 channel ID
				heading = string(IDXDLM)+DESINAME;
				spaceadded = fmtlen-strlen(DESINAME)-1 ;
				spaces = ""; for (int c=0; c<spaceadded;c++) spaces+=' ';

				/* DESI header: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%s%s%s",heading.c_str(),
					apername(scgl_nchan,i),spaces.c_str());
			} else {
				/* Check the first attempt to write results in something: */
				int t = scgl_curmeas-1 ;	/* Row number to take in index storages */
				for (int i=0;i<scgl_nindex;i++) 
					fprintf(f,IDXFMT,IDXDLM,scgl_dsidx[t][i]);
				for (int i=0;i<scgl_nchan;i++) 
						fprintf(f,IDXFMT,IDXDLM,scgl_desidx[t][i]);
			}
			break ;
		case MOM:
			if (header) {
				/* File needs the header only. Create header: */
				/* Determine the length of the sample value written without
				delimiter: */
				char test[20];
				sprintf(test,MOMFMT+2,1.0);
				const int headlen = 7;	// length of longest column heading
				int nspace = strlen(test) - headlen;
				if (nspace<0) nspace=0;	// bad alignment results but write's ok
				for (int i=0; i<nspace; i++) test[i] = ' ';
				test[nspace] = 0;
				/* Nchan means: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sMEAN_%s %s",MOMDLM, apername(scgl_nchan,i), test);
		
				/* Lag-0 dispersions and covariances header: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sSIG_%s  %s",MOMDLM, apername(scgl_nchan,i), test);
				for (int i=scgl_nchan;i<scgl_nindex;i++) 
					fprintf(f,"%sCOV_%s %s",MOMDLM, apername(scgl_nchan,i), test);
		
				/* Lag-1 auto- and cross-covariances: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sCOV1_%s %s",MOMDLM, apername(scgl_nchan,i), test);
				for (int i=scgl_nchan;i<scgl_nindex;i++) 
					fprintf(f,"%sCOV1_%s%s",MOMDLM, apername(scgl_nchan,i), test);
				/* Lag-2 autocovariances: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sCOV2_%s %s",MOMDLM, apername(scgl_nchan,i), test);
			} else {
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,MOMFMT,MOMDLM,scgl_mean[i]);

				for (int i=0;i<scgl_nindex;i++) 
					fprintf(f,MOMFMT,MOMDLM,scgl_sig[i]);

				for (int i=0;i<scgl_nindex;i++) 
					fprintf(f,MOMFMT,MOMDLM,scgl_r1[i]);

				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,MOMFMT,MOMDLM,scgl_r2[i]);
		}
		break ;
		default:
			nr::nrerror("SCIND: write(): wrong what selection",nr::ERFIO) ;
			return ;	
	}
	if (suffix)
		fprintf(f,"%s%s\n",IDXDLM,suffix);
	else
		fprintf(f,"\n");
}

/*---------------------------- isheader ------------------------------*/
bool sc::isheader(FILE * f, const char * stamp) {
	static std::map <FILE *,string> heads;
	if (!stamp) {
		/* reset stamps collection for a given file: */
		heads[f] = "";
		return false;
	}
	const char DELIM = '|';
	string templ = string(stamp)+DELIM;
	int pos = heads[f].find(templ);
//cout << "pos(" << templ << ")="<<pos<<'\n';
	if (pos>=0) 
		return true;
	else {
		// add the stamp with a delimiter into collection:
		heads[f] += templ;
//cout <<	"heads[" << (int*)f << "]="<<heads[f]<< '\n';
		return false;
	}
}

/*---------------------------- str2mom -------------------------------*/

void sc::str2mom(const char * line) {
	/* running pointer: */
	const char * s = line;
	char * endptr = 0;
	for (int i=0; i<scgl_nchan; i++) {
		double d = strtod(s,&endptr);
		if (endptr==s) {
			nr::nrerror("SCIND: str2mom() short read 1",nr::ERFIO);
			return;
		}
		scgl_mean[i] = d;
		s = endptr;
	}
	for (int i=0; i<scgl_nindex; i++) {
		double d = strtod(s,&endptr);
		if (endptr==s) {
			nr::nrerror("SCIND: str2mom() short read 2",nr::ERFIO);
			return;
		}
		scgl_sig[i] = d;
		s = endptr;
	}
	for (int i=0; i<scgl_nindex; i++) {
		double d = strtod(s,&endptr);
		if (endptr==s) {
			nr::nrerror("SCIND: str2mom() short read 3",nr::ERFIO);
			return;
		}
		scgl_r1[i] = d;
		s = endptr;
	}
	
	for (int i=0; i<scgl_nchan; i++) {
		double d = strtod(s,&endptr);
		if (endptr==s) {
			nr::nrerror("SCIND: str2mom() short read 4",nr::ERFIO);
			return;
		}
		scgl_r2[i] = d;
		s = endptr;
	}
}


/*--------------------------- writeavg --------------------------------*/

void sc::writeavg(FILE * f, scwhat what, bool isgen, const char * stamp, 
	const char * suffix, bool header)
//throw(std::domain_error, std::invalid_argument, sc::ioproblem) 
{
	if (nr::erget()) return ;

	if (scgl_nchan<=0) {
		//throw std::domain_error(
		nr::nrerror("SCIND: writeavg() while uninitialized",nr::ERNUL);
		return ;
	}

	// suppress the repetitive header writing:
#ifdef NODUBHEAD
	if (header && isheader(f,stamp)) return;
#endif

	if ((scgl_curmeas<=0) && !header && (what==sc::DSI)) {
		//throw std::domain_error(
		nr::nrerror("SCIND: writeavg() while no measurements done",nr::ERNOD);
		return ;
	} 

	if (stamp && (!fprintf(f,"%s",stamp))) {
		/* Failed write attempt: */
		//throw sc::ioproblem(
		nr::nrerror("SCIND: writeavg(): cannot write file",
			nr::ERFIO) ;
		return ;
	}

	switch (what) {
		case DSI: case DESI:
			if (header) {
				/* File needs the header only. Create header: */
				/* # <header> */

				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sSI_%s   %se_SI_%s  ",IDXDLM, 
						apername(scgl_nchan,i),	IDXDLM, apername(scgl_nchan,i));
				for (int i=scgl_nchan;i<scgl_nindex;i++) 
					fprintf(f,"%sDSI_%s %se_DSI_%s",IDXDLM,
						apername(scgl_nchan,i),	IDXDLM, apername(scgl_nchan,i));
		
				/* DESI header: */
				for (int i=0;i<scgl_nchan;i++) 
					fprintf(f,"%sDESI_%s %se_DESI_%s",IDXDLM, 
						apername(scgl_nchan,i),	IDXDLM, apername(scgl_nchan,i));
			} else {		

				/* Write the average indices: */
				for (int i=0;i<scgl_nindex;i++) {
					fprintf(f,IDXFMT,IDXDLM,getavgidx(DSI,isgen,i));
					fprintf(f,ERRFMT,IDXDLM,geterridx(DSI,isgen,i));
				}
				for (int i=0;i<scgl_nchan;i++) {
					fprintf(f,IDXFMT,IDXDLM,getavgidx(DESI,isgen,i));
					fprintf(f,ERRFMT,IDXDLM,geterridx(DESI,isgen,i));
				}
			}
			break ;
		case AFLUX: case AFLUXP:
			if (header) {
				/* File needs the header only. Create header: */
				if (what==AFLUXP) 
					for (int i=0;i<scgl_nchan;i++) 
						fprintf(f,"%sFLUX_%s%se_FLUX_%s%sP    ",FLXDLM,
							apername(scgl_nchan,i), FLXDLM,
							apername(scgl_nchan,i), FLXDLM);
				else
					for (int i=0;i<scgl_nchan;i++) 
						fprintf(f,"%sFLUX_%s%se_FLUX_%s",FLXDLM, 
							apername(scgl_nchan,i),	FLXDLM, apername(scgl_nchan,i));
			} else {		

				/* Write the average fluxes: */
				for (int i=0;i<scgl_nchan;i++) {
					double flux = getavgflux(isgen,i) ;
					fprintf(f,FLXFMT,FLXDLM, fluxprec(flux), flux);
					fprintf(f,ERRFMT,FLXDLM,geterrflux(isgen,i));
					if (what==AFLUXP) 
						fprintf(f,PFMT,FLXDLM,	scgl_channel[i].nonpois);
				}
			}
			break ;
		default:
			nr::nrerror("SCIND: writeavg(): wrong what selection",nr::ERFIO) ;
			return ;	
	}
	if (suffix)
		fprintf(f,"%s%s\n",(what==DSI || what==DESI)?IDXDLM:FLXDLM,suffix);
	else
		fprintf(f,"\n");
}

#ifdef CNTMOD
/*--------------------- count data modifiers: ---------------------------*/
/*--------------------------- smooth --------------------------------*/
/**
	@brief	Smooth the data counts with sliding window
	@param	data	data array
	@param	ndata	its length
	@param	band	width of smoothing window
	@param	smdata	Resulting filtered data array
	
	Function smooths the data counts in sliding window assigning the averaged
	counts to near-center elements of an output array. On the left and right
	margins, the window is cut. If \em band is even, then window is displaced
	by 1/2 pixels to the lower index (e.g.: band==2 gives a window containing
	the current pixel and pixels to the "left").

	\note
	- smdata[] array must contain at least ndata elements;
	
	@see calcidx()
*/
void sc::smooth(count_t * data, int ndata, int band, count_t * smdata) 
//throw(std::invalid_argument) 
{
	if (nr::erget()) return ;
	count_t x = 0 ;
	
	/* Verify the pointers and parameters: */
	if ((data == NULL) || (smdata == NULL) || (band<=0)) {
		//throw std::invalid_argument(
		nr::nrerror("SCIND: smooth(): null or <0 parameter(s)",nr::ERPAR) ;
		return ;
	}

	int lband = band/2 ;	
	int hband = band - 1 - lband ;
	/* left margin: */
	for (int i=0; i<lband; i++) {
		k = 0; 
		x = 0;
		for (int j=0; j<=i+hband; j++) {k++; x += data[j] ;} 
		smdata[i] = x / k ;
	}
	/* middle of set: no marginal effects */
	for (int i=lband; i<ndata-hband; i++) {
		x = 0;
		for (int j=i-lband; j<=i+hband; j++) x += data[j] ;
		smdata[i] = x / band ;
	}
	/* right margin: */
	for (int i=ndata-hband; i<ndata; i++) {
		k = 0; 
		x = 0;
		for (int j=i-lband; j<ndata; j++) {k++; x += data[j] ;} 
		smdata[i] = x / k ;
	}
}

/*--------------------------- bin --------------------------------*/
/**
	@brief	Bin the data array 
	@param	data	data array
	@param	ndata	its length
	@param	bin	binning factor (>0)
	@param	nbin	Resulting number of bins - length of output bin_data array
	@param	bin_data	Resulting binned data array
	
	Procedure bins the data array producing one output value per each \em bin
	values of input arrays. Excess elements are ignored (e.g., 1000 input
	elements with 3-binning produce 333 output elements: 1 (last) element is
	thrown away). Number of elements is returned in nbin.
	
	\note
	- bin_data[] array must contain at least ndata elements.
	- providing bin=1 results in a plain copying of the input array into output
	
	@see calcidx()
*/
void sc::bin(count_t * data, int ndata, int bin, int * nbin, count_t * bin_data)
//throw(std::invalid_argument) 
{
	if (nr::erget()) return ;
	count_t x = 0 ;
	
	/* Verify the pointers and parameters: */
	if ((data == NULL) || (bin_data == NULL) || (bin<1)) {
		//throw std::invalid_argument(
		nr::nrerror("SCIND: bin(): null or <0 parameter(s)",nr::ERPAR) ;
		return ;
	}

	*nbin = 0 ;
	for (int i=0; i<ndata-bin+1; i+=bin) {
		x = 0;
		for (int j=i; j<i+bin; j++) x += data[j] ;
		bin_data[*nbin] = x ;
		(*nbin)++ ;
	}
}
#endif

/*--------------------------- calcmom --------------------------------*/

void sc::calcmom(int ndata, int * ncorr) 
//throw (std::domain_error, std::range_error, std::invalid_argument, std::out_of_range, sc::dataproblem) 
{
	const int LAG1 = 1 ;
	const int LAG2 = 2 ;
	int ch, ch2, i ;

	if (nr::erget()) return ;

	/* Prevent the call when no buffers are allocated: */
	if (scgl_curmeas<0 || !scgl_channel) {
		//throw std::domain_error(
		nr::nrerror("SCIND: calcmom() while uninitialized",nr::ERNUL) ;
		return ;
	}
	if (!ncorr) {
		//throw std::domain_error(
		nr::nrerror("SCIND: calcmom() with null ncorr[] array",nr::ERNUL) ;
		return ;
	}

	int nsets = scgl_nchan ;
	chan_t * sets = scgl_channel ;
	
	/* Check the total data number in channels: */
	for (int j=0; j<nsets; j++) 
		if (ndata-LAG2-ncorr[j]<=0) {
			//throw sc::dataproblem(
			nr::nrerror("SCIND: calcmom(): too few data supplied",nr::ERNOD) ;
			return ;
		}
	
	double dx0=0, dx1, dx2, dy0, dy1 ;
	/* Compute "raw" means */
	for (ch=0; ch<nsets; ch++) {
		dx1 = 0 ;
#ifdef LOGIDX
		dx2 = 0 ;
		for (i=0; i<ndata; i++) {
			if (sets[ch].data[i]>0) {
				dx0 = log((double)sets[ch].data[i]) ;
				scgl_logi[ch][i] = dx0 ;
				dx2 += dx0 ;
			} else {
				scgl_logi[ch][i] = 0.0 ;
			}
			dx1 += sets[ch].data[i] ;
		}
		scgl_logmean[ch] = dx2 / ndata ;
#else
		for (i=0; i<ndata; i++) dx1 += sets[ch].data[i] ;
#endif
		scgl_mean[ch] = dx1 / ndata ;
//		printf("<log[%d]>=%f\n",ch,scgl_logmean[ch]);
	}
	
	/* A cycle by channels: compute auto-correlations with lag up to 2
	(dispersions and auto-correlations with lags 1 and 2: */
	for (ch=0; ch<nsets; ch++) {
		/* Compute "tails" which go out of cycle limits for r0 and r1: */
#ifdef LOGIDX
		dx0 = scgl_logi[ch][0] - scgl_logmean[ch] ;
		dx1 = scgl_logi[ch][1] - scgl_logmean[ch] ;
#else
		dx0 = (double)sets[ch].data[0] - scgl_mean[ch] ;
		dx1 = (double)sets[ch].data[1] - scgl_mean[ch] ;
#endif
		scgl_sig[ch] = dx0*dx0 + dx1*dx1 ;
		scgl_r1[ch] = dx0*dx1 ;
		scgl_r2[ch] = 0 ;
		for (i=LAG2; i<ndata; i++) {
#ifdef LOGIDX
			dx2 = scgl_logi[ch][i] - scgl_logmean[ch] ;
#else
			dx2 = (double)sets[ch].data[i] - scgl_mean[ch] ;
#endif
			scgl_sig[ch] += dx2*dx2 ;
			scgl_r1[ch] += dx2*dx1 ;
			scgl_r2[ch] += dx2*dx0 ;
			dx0 = dx1 ;
			dx1 = dx2 ;
		}
		scgl_r1[ch] /= (ndata-LAG1-ncorr[ch]) ;
		scgl_r2[ch] /= (ndata-LAG2-ncorr[ch]) ;
		scgl_sig[ch] /= (ndata-ncorr[ch]) ;
#ifndef LOGIDX
		/* Normalize by the squared mean of counts: (ver2.1 and further)*/
		dx0 = (scgl_mean[ch] * scgl_mean[ch]) ;
		if (dx0>0) dx0 = 1.0 / dx0;
		// else - zero second moments result.
		scgl_sig[ch] *= dx0 ;
		scgl_r1[ch] *= dx0 ;
		scgl_r2[ch] *= dx0 ;
#endif
	} 

	/* Two embedded cycles by channels:  compute covariances */
	/* seq. number of channels combinations (channel1!=channel2) */
	int ncomb = nsets; 
	for (ch=0; ch<nsets-1; ch++) {
		for (ch2=ch+1; ch2<nsets; ch2++) {
#ifdef LOGIDX
			dx0 = scgl_logi[ch ][0] - scgl_logmean[ch ] ;
			dy0 = scgl_logi[ch2][0] - scgl_logmean[ch2] ;
#else
			dx0 = (double)sets[ch ].data[0] - scgl_mean[ch ] ;
			dy0 = (double)sets[ch2].data[0] - scgl_mean[ch2] ;
#endif
			scgl_sig[ncomb] = dx0*dy0 ;
			scgl_r1[ncomb] = 0 ;
			for (i=LAG1; i<ndata; i++) {
#ifdef LOGIDX			
				dx1 = scgl_logi[ch ][i] - scgl_logmean[ch ] ;
				dy1 = scgl_logi[ch2][i] - scgl_logmean[ch2] ;
#else
				dx1 = (double)sets[ch ].data[i] - scgl_mean[ch ] ;
				dy1 = (double)sets[ch2].data[i] - scgl_mean[ch2] ;
#endif
				scgl_sig[ncomb] += dx1*dy1 ;
				scgl_r1[ncomb] += dx1*dy0+dx0*dy1 ;
				dx0 = dx1 ;
				dy0 = dy1 ;
			}
			scgl_r1[ncomb] /= (2*(ndata-LAG1)) ;
			scgl_sig[ncomb] /= ndata ;
#ifndef LOGIDX
			/* Normalize by the squared mean of counts: (starting from ver2.1) */
			dx0 = (scgl_mean[ch] * scgl_mean[ch2]) ;
			if (dx0>0) 
				dx0 = 1.0 / dx0;
			else 
			// zero covariances result:
				dx0 = 0;
			scgl_sig[ncomb] *= dx0 ;
			scgl_r1[ncomb] *= dx0 ;
#endif
			ncomb ++;
		}
	}
}

/*--------------------------- calcidx --------------------------------*/

void sc::calcidx(double microexp, bool isgen) 
//throw (std::domain_error, std::range_error, std::invalid_argument, std::out_of_range, sc::dataproblem) 
{
	int ch, ch2, idx ;

	if (nr::erget()) return ;

	/* Prevent the overflow of index storages: */
	if (scgl_curmeas>=scgl_maxnmeas) {
		//throw std::range_error(
		nr::nrerror("SCIND: calcidx() skipped by index storages overflow",
			nr::EROFL) ;
		return ;
	}
	/* Prevent the call when no buffers are allocated: */
	if (scgl_curmeas<0 || !scgl_channel) {
		//throw std::domain_error(
		nr::nrerror("SCIND: calcidx() while uninitialized",nr::ERNUL) ;
		return ;
	}

	double * scind = scgl_dsidx[scgl_curmeas] ;
	double * desi  = scgl_desidx[scgl_curmeas] ;
	int nsets = scgl_nchan ;
	chan_t * sets = scgl_channel ;
	
	double dx0=0, dx2, dy0 ;
	/* Non-linearity correction: */
	int ncomb = nsets ;
	for (ch=0;ch<nsets;ch++) {
		for (ch2=ch;ch2<nsets; ch2++) {
			if (ch==ch2) {
				idx = ch ;
				/* Correct means: */ 
				scgl_cmean[scgl_curmeas][ch] = 
				scgl_mean[ch] * (1.0 + scgl_mean[ch] * sets[ch].deadtime /
					microexp) ;
				dx0 = 1.0 - 2.0 * scgl_mean[ch] * sets[ch].deadtime / microexp ;
				dy0 = dx0 ;
			} else {
				dx0 = 1.0 - 2.0 * scgl_mean[ch] * sets[ch].deadtime / microexp ;
				dy0 = 1.0 - 2.0 * scgl_mean[ch2] * sets[ch2].deadtime /microexp;
				idx = ncomb ;
				ncomb++ ;
			}
			dx2 = dx0*dy0 ;
			scgl_sig[idx] /= dx2 ;
			scgl_r1[idx] /= dx2 ;
			if (idx<scgl_nchan) scgl_r2[idx] /= dx2 ;
		}
	}

	/* Normal scintillation indices corrected to zero exposure: */
	for (ch=0; ch<nsets; ch++) {
// starting from ver2.1: below: not divide by (x-b)**2 but multiply by (x'/(x-b))**2 :
		dx0 = scgl_mean[ch] / 
			(scgl_cmean[scgl_curmeas][ch] - sets[ch].bkgr*microexp) ;
		scind[ch] = (
					1.5*scgl_sig[ch] - 0.5*scgl_r1[ch] - 1.5 * 
// ver2.1: nonpois must still be multiplied by cmean, not mean (normalizator of
// dispersion):
	sets[ch].nonpois * scgl_cmean[scgl_curmeas][ch]/scgl_mean[ch]/scgl_mean[ch]) ;
//if (!ch) printf("SCIND: non-norm-d idx[0] = %7.1e\n",scind[ch]);
		if (dx0>0)
// starting from ver2.1: normalize by multiplication:
			scind[ch] *= (dx0*dx0) ;
		else
			scind[ch] = 0 ;
	}

	/* Differential scintillation indices given the known already
		normal indices */
	ncomb = nsets ;
	for (ch=0;ch<nsets-1;ch++) {
		for (ch2=ch+1;ch2<nsets; ch2++) {
			dx0 = (scgl_cmean[scgl_curmeas][ch] - sets[ch].bkgr*microexp) ;
			dy0 = (scgl_cmean[scgl_curmeas][ch2] - sets[ch2].bkgr*microexp) ;
			if ((dx0>0) && (dy0>0))
				scind[ncomb] = scind[ch] + scind[ch2] - 2.0 *
					(1.5*scgl_sig[ncomb]-0.5*scgl_r1[ncomb]) *
// starting from ver2.1: normalized covariances sig, r1 are used: denormalize here:
					scgl_mean[ch] * scgl_mean[ch2]
					/ (dx0 * dy0) ;
			else
				scind[ncomb] = 0 ;
			ncomb ++;
		}
	}
	/* DESI */
	for (ch=0; ch<scgl_nchan; ch++) {
		dx0 = scgl_mean[ch] /
			(scgl_cmean[scgl_curmeas][ch] - sets[ch].bkgr*microexp) ;
		if (dx0>0)
			desi[ch] = (
			scind[ch]*3 + 
// starting from ver2.1: normalized covariances sig, r1 are used: denormalize here:
			(scgl_r2[ch] - scgl_r1[ch]*4) * dx0 * dx0
			)/ 4.5 ; 
		else
			desi[ch] = 0 ;
	}

	/* Save the current microexposure for scaling in get-functions */
	scgl_curexp = microexp ;

	// Validate the range: 
	for (int idx=0; idx<scgl_nchan; idx++) 
		if (scind[idx]>MAXSCIND || scind[idx]<MINSCIND) {
			nr::nrerror("SCIND: calcidx(): invalid scint. index: skipped meas.",
				nr::ERNOD);
			/* exit without incrementing the counter scgl_curmeas: */
			return;
		}
	// compute the last element : SC_D/SC_A for filtering the star escapes.
	// In general, we take smallest to largest apertures indices ratio:
	if (scind[0]>0)
		scind[scgl_nindex] = scind[scgl_nchan-1] / scind[0] ;
	else
		scind[scgl_nindex] = 0;
		
	if (!nr::erget()) {
		scgl_isgen[scgl_curmeas] = (isgen ? MODEGEN : MODENORM) ;
		scgl_curmeas++ ;
	}
}

/*--------------------------- avgmatrix --------------------------------*/

void sc::avgmatrix(double ** matrix, int ncol, int nrow, 
	signed char * rowselect, signed char select, 
	double * avg, double * err2, int lag) 
//throw(std::bad_alloc, std::invalid_argument, sc::dataproblem) 
{

	if (nr::erget()) return ;

	/* Verify the pointers and parameters: */
	if (!matrix || !avg || !err2) {
		//throw std::invalid_argument(
		nr::nrerror("SCIND: avgmatrix(): null input matrix "
		"or output vectors",nr::ERNUL);
		return ;
	}
		
	/* Allocate the storage for elementary differences */
	double *dif = nr::dvector(0, nrow-1) ;
	double *rho = nr::dvector(0, lag) ;

	/* Cycle by columns of the storage matrix: */
	for (int i=0; i<ncol; i++) {
		avg[i] = 0 ;
		/* Cycle by base-time measurements : Simple mean accumulation */
		int nmeas = 0 ;
		if  (ncol==1) {
			/* Treate the vector-like matrix transponed: */
			for (int t=0; t<nrow; t++) 
				if ((rowselect==NULL) || (rowselect[t]==select)) {
					dif[nmeas] = matrix[i][t] ;
					avg[i] += dif[nmeas] ;
					nmeas ++ ;
				}
		} else {
			for (int t=0; t<nrow; t++) 
				if ((rowselect==NULL) || (rowselect[t]==select)) {
					dif[nmeas] = matrix[t][i] ;
					avg[i] += dif[nmeas] ;
					nmeas ++ ;
				}
		}
		if (nmeas)
			avg[i] /= nmeas ;
		else {
			nr::free_dvector(dif,0,nrow-1) ;
			nr::free_dvector(rho,0,lag) ;
			//throw sc::dataproblem(
			nr::nrerror("SCIND: avgmatrix(): no data to be averaged",nr::ERNOD);
			return ;
		}
		if (nmeas==1) {
			err2[i] = 0 ;
			continue ;
		}
		/* Correct too high maximal lag: */
		if (lag>nmeas-1) lag = nmeas - 1 ;
	
		/* Produce the differences themselves: */
		for (int t=0; t<nmeas; t++) dif[t] -= avg[i] ;

		/* Compute the moments with lag up to "lag" */
		for (int k=0; k<=lag; k++) {
			rho[k] = 0 ;
			for (int t=0; t<nmeas-k; t++)
				rho[k] += dif[t] * dif[t+k] ;
		}
//#define PRNCORREL				
#ifdef PRNCORREL
if (lag>0) {
	printf("# corr.coef.for %d-th index (lag=1..%d):\n",i+1,lag);
	for (int k=1; k<=lag; k++) printf("%6.4f ",rho[k]/rho[0]);
	printf("\n");
}
#endif

		/* Error of mean: Korn and Korn, section 19.8 "Statistics and
		measurement of the random process": "means by finit time span" */
		err2[i] = rho[0] / (nmeas-1) ;
		for (int k=1; k<=lag; k++)
			err2[i] += 2*(1.0-(double)k/nmeas)*rho[k]/(nmeas-k) ;
		err2[i] /= nmeas ;
	}
	nr::free_dvector(dif,0,nrow-1) ;
	nr::free_dvector(rho,0,lag) ;
}

/*--------------------------- average --------------------------------*/

void sc::average(bool inclgen, int lag) 
//throw(std::bad_alloc, std::invalid_argument, sc::dataproblem) 
{
//	if (nr::erget()) return ;

	for (signed char isgen=0; isgen<=(inclgen==true); isgen++) {
	
		// Iterative data filtering by SC_<largest aper.>/SC_<smallest aper.>:
		int iter = 0;
		for (iter=0; iter<MAXFITER; iter++) {
			// compute mean ratio of indices in largest and smallest ap-s:
			int n = 0; // number of valid measurements
			double mean_da = 0;	// "mean SI_D / SI_A" (if nchan=4)
			for (int meas = 0; meas<getcurmeas(); meas++)
				if (scgl_isgen[meas]==isgen) {
					mean_da += scgl_dsidx[meas][scgl_nindex];	// stored there by calcidx()
					n++;
				}
			if (n<=1) break; 	// no valid measurements rest
			mean_da /= n;
			// compute the dispersion of indices ratio:
			double sig_da = 0;
			for (int meas = 0; meas<getcurmeas(); meas++)
				if (scgl_isgen[meas]==isgen) {
					double d = (scgl_dsidx[meas][scgl_nindex] - mean_da);
					sig_da += d*d;
				}
			sig_da = sqrt(sig_da/(n-1));
			// maximal allowed ratio of indices in largest and smallest	ap-s:
			double max_da = mean_da + KAPPA * sig_da;
#ifdef SCINDTEST
cout << "mean_da=" << mean_da << " sig_da="<<sig_da<<"	max_da="<<max_da<<"	n=" << n << "\n";
#endif
			// clip the high values:
			bool clipped = false;	// flag of clipped existence
			for (int meas = 0; meas<getcurmeas(); meas++) 
				if (scgl_isgen[meas]==isgen && 
					scgl_dsidx[meas][scgl_nindex]>max_da) {
					scgl_isgen[meas] = MODENO;	// omit measurement
					clipped = true;
				}
			// exit loop if no bad values found:
			if (!clipped) break;
		}
#ifdef SCINDTEST
for (int meas = 0; meas<getcurmeas(); meas++) 
	cout << "Ratio: " << scgl_dsidx[meas][scgl_nindex] << "\n";
cout << "iter="<<iter<<'\n';
#endif
		/* Average indices normal and differential aperture: */
		avgmatrix(scgl_dsidx, scgl_nindex, getcurmeas(),
			scgl_isgen, isgen, scgl_adsi+isgen*scgl_nindex,
				scgl_e2dsi+isgen*scgl_nindex, lag) ;

		/* Average DESI: */
		avgmatrix(scgl_desidx, scgl_nchan, getcurmeas(), scgl_isgen, isgen, 
			scgl_adesi+isgen*scgl_nchan, scgl_e2desi+isgen*scgl_nchan, lag) ;
				
		/* Average corrected mean counts: */
		avgmatrix(scgl_cmean,scgl_nchan,getcurmeas(), scgl_isgen, isgen,
			scgl_aflux+isgen*scgl_nchan,scgl_e2flux+isgen*scgl_nchan,lag) ;
	}
}

/*------------------------------ statflux ---------------------------------*/

void sc::statflux(int ndata, int * ncorr, double microexp, int k,
	bool isfilt, 
	double * nonpois, double * uavg, double * usig)
//throw (std::invalid_argument) 
{
//	if (nr::erget()) return ;

	/* Prevent the call when no buffers are allocated: */
	if (scgl_nchan<=0 || !scgl_channel || !scgl_aflux) {
		//throw std::domain_error(
		nr::nrerror("SCIND: statflux() while uninitialized",nr::ERNUL) ;
		return ;
	}

	if (k<1) {
		//throw std::invalid_argument(
		nr::nrerror("SCIND: statflux(): bad sequence number",
			nr::ERPAR) ;
		return ;
	}

	/* Compute the expected normal indices in all channels: */	
	for (int ch=0; ch<scgl_nchan; ch++) {

		double * avg = (uavg ? uavg : scgl_aflux) ;
		double * sig = (usig ? usig : scgl_e2flux) ;

		if ((ndata-ncorr[ch]-2)<0) {
			//throw std::invalid_argument(
			nr::nrerror("SCIND: statflux(): too few data",
				nr::ERNOD) ;
			return ;
		}

		/* Sum the non-linearity corrected counts in the current array: */
		double nonlin = scgl_channel[ch].deadtime / microexp ;
		count_t * data = scgl_channel[ch].data ;
		double avgdata = 0 ;
		for (int i=0; i<ndata; i++) {
			double d = data[i] ;
			avgdata += d * (1.0 + nonlin*d) ;
		}
		/* Average count: */
		avgdata /= ndata ;
	
		/* Dispersion of counts: [corrected count^2]*/
		double sigdata = 0 ;
		for (int i=0; i<ndata; i++) {
			double d = data[i] ;
			d = d * (1.0 + nonlin*d) - avgdata ;
			sigdata += d*d ;
		}
		sigdata /= (ndata-1-ncorr[ch]) ;
		
		int noldcount = (k-1)*ndata ;	// number of counts before this call
		
		/* Dispersion of counts accumulated BEFORE this call (see below): */
		double sigold = (k>1) ? (sig[ch]*noldcount*(microexp*microexp)) : 0 ;
		/* Average of counts accumulated before this call: */
		double avgold = (k>1) ? (avg[ch]*microexp) : 0 ;
		
		/* Updated results with the help of current series statistics: */
		double signew = ( (k-1) * sigold + sigdata ) / k ;
		double avgnew = ( (k-1) * avgold + avgdata ) / k ;
		/* New dispersion corrected for trend (if needed): */
		if (!isfilt) {
			double d = avgnew - avgold ;
			signew += (k-1) * d*d ;
		}
		
		if (nonpois) {
			/* compute the non-Poissonity in nonpois[] and store it in the
			local channel structure for writeavg(): */
			double p = avgnew>0 ? signew / avgnew : 0;
			scgl_channel[ch].nonpois = nonpois[ch] = p ;
		}

		/* Normalize the results by the microexposure and produce the squared
			error of mean from the dispersion: */
		int ncount = k*ndata ;	// total number of counts up to now
		avg[ch] = avgnew / microexp ;
		sig[ch] = (signew/ncount) / (microexp*microexp) ;
		/* The results are saved in average flux storages even if the avg[] and
		sig[] placeholders are explicit parameters. This allows to write the
		accumulated statistics with writeavg(). : */
		if (uavg) scgl_aflux[ch] = avg[ch] ;
		if (usig) scgl_e2flux[ch] = sig[ch] ;
	}
}

/*------------------------------ stattest ---------------------------------*/

void sc::stattest(int ndata, double microexp) {

	const int NTAKT	= 4	;	/* periodicity of the test signal */
	const int TAKTPLUS = 1 ;	/* each second takt the flux is increased */
	const int TAKTMINUS = 3	;	/* each fourth takt the flux is decreased */

//	if (nr::erget()) return ;

	/* Prevent the overflow of index storages: */
	if (scgl_curmeas>=scgl_maxnmeas) {
		//throw std::range_error(
		nr::nrerror("SCIND: stattest() skipped by index storages overflow",
			nr::EROFL) ;
		return ;
	}
	/* Prevent the call when no buffers are allocated: */
	if (scgl_curmeas<0 || !scgl_channel) {
		//throw std::domain_error(
		nr::nrerror("SCIND: stattest() while uninitialized",nr::ERNUL) ;
		return ;
	}

	double * scind = scgl_dsidx[scgl_curmeas] ;
	double * desi  = scgl_desidx[scgl_curmeas] ;
#ifdef STATTEST
	printf("------------ STATTEST ----------------\n");
#endif
	/* Compute the expected normal indices in all channels: */	
	for (int ch=0; ch<scgl_nchan; ch++) {
		double fluxmid = 0, fluxplus = 0, fluxminus = 0 ;
		int nmid = 0, nplus = 0, nminus = 0 ;
#ifdef STATTEST
		count_t cmin = 100000, cmax = 0 ;
#endif		
		count_t * data = scgl_channel[ch].data ;
		/* Sum the fluxes in different takts: */
		for (int i=0; i<ndata; i++) {
			int takt = i % NTAKT ;
#ifdef STATTEST
		if (cmin>data[i])
			cmin = data[i] ;
		if (cmax<data[i])
			cmax = data[i] ;
#endif		
			switch (takt) {
				case TAKTPLUS:
					fluxplus += (double) (data[i]) ;
					nplus++ ;
					break ;
				case TAKTMINUS:
					fluxminus += (double) (data[i]) ;
					nminus++ ;
					break ;
				default:
					fluxmid += (double) (data[i]) ;
					nmid++ ;
			}
		}

		if (!nplus*nminus*nmid) {
			//throw sc::dataproblem(
			nr::nrerror("SCIND: stattest(): no data for takt",nr::ERNOD);
			return ;
		}

		/* Average fluxes and correct them for non-linearity */
		double nonlin = scgl_channel[ch].deadtime/microexp ;
		fluxplus /= nplus; fluxplus *= (1.0 + nonlin*fluxplus) ;
		fluxminus /= nminus; fluxminus *= (1.0 + nonlin*fluxminus) ;
		fluxmid /= nmid; fluxmid *= (1.0 + nonlin*fluxmid) ;
		
		/* Normalize the corrected fluxes by the microexposure: */
		fluxplus /= microexp ; fluxminus /= microexp; fluxmid /= microexp ;

		double fluxavg = (fluxplus*nplus + fluxminus*nminus + fluxmid*nmid) /
				(nplus + nminus + nmid) ;

#ifdef STATTEST
		printf("CHAN %d: min=%d, max=%d ",ch,cmin,cmax) ;
		printf("flux: (+/-/mid/avg)=(%6.1f/%6.1f/%6.1f/%6.1f)\n",
			fluxplus,fluxminus,fluxmid,fluxavg) ;
#endif		
	
		/* Deviations from the average level: */
		double dplus = fluxplus - fluxavg ;
		double dminus = fluxminus - fluxavg ;
		double dmid = fluxmid - fluxavg ;
	
		/* Theoretic dispersion and lag=1,2 autocovariances: */
		double sig  = (2*dmid*dmid + dplus*dplus + dminus*dminus) / 4 ;
		double rho1 = dmid * (dplus + dminus) / 2 ;
		double rho2 = (dmid*dmid + dplus*dminus) / 2 ;

#ifdef STATTEST
		printf("  sig=%f, rho1=%f, rho2=%f\n", sig,rho1,rho2) ;
#endif	
		double fl2 = fluxavg * fluxavg ;
		
		/* Normal scintillation index and DESI */
		if (fl2>0) {
			scind[ch] = (1.5*sig - 0.5*rho1) / fl2 ;
			desi[ch] = (scind[ch]*3 + (rho2 - 4*rho1)/fl2)/ 4.5 ; 
		} else 
			scind[ch] = desi[ch] = 0 ;
#ifdef STATTEST
		printf("  scind=%6.4f, desi=%6.4f\n", scind[ch],desi[ch]) ;
#endif	
	} // of cycle by ch - channel number

	/* The expected differential aperture indices are zero: */	
	for (int idx=scgl_nchan; idx<scgl_nindex; idx++)
		scind[idx] = 0 ;

	/* We will save the expected index in the storage as "generalized": */
	const int MODEXPECT = MODEGEN ;
	if (!nr::erget()) {
		scgl_isgen[scgl_curmeas] = MODEXPECT ;
		scgl_curmeas++ ;
	}

}
