#include <math.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include "star.h"
#include "trberror.h"
#include "trbtime.h"
#include "names.h"
#include "atmos.hxx"
#include "weif.hxx"
#include "tools.h"
#include "messages.h"
#include "cfg.h"
//#include "mcua.h"	// SUCCESS/FAIL
#include "success.h" // SUCCESS/FAIL

//const int	UNDEFINED	= -1;
QString		Undefined	= "?";
QString		UndefinedName	= "Undefined";
const double	UndefinedVal	= -1;

extern FILE* out_file;
extern CFGTree* tree;

static QString str_message;

double toValidTimeInterval(double sec) {
	double isec = floor(sec);
	double res = (double)((int)isec % (int)SEC_IN_DAY) +
			sec - isec;
	if(res < 0) res += SEC_IN_DAY;
	return(res);
}

double toDeg(double sec) {
	return(sec / 3600.0);
}

void toStrHour(double sec, QString& str) {
	sec = toValidTimeInterval(sec);
	int _dd = (int)floor(sec / 3600.0);
	int _dm = (int)floor((sec / 3600.0 - _dd) * 60.0);
	int _ds = (int)sec % 60;
	str.sprintf(" %02d  %02d  %02d  ", _dd,_dm,_ds);
}
 
void toStrHour12(double sec, QString& str) {
	sec = toValidTimeInterval(sec);
	if(sec > SEC_IN_DAY / 2.0) sec -= SEC_IN_DAY; 
	char sign = ' ';
	if(sec < 0) sign = '-';
	sec = fabs(sec); 
	int _dd = (int)floor(sec / 3600.0);
	int _dm = (int)floor((sec / 3600.0 - _dd) * 60.0);
	int _ds = (int)sec % 60;
	str.sprintf(" %c%02d  %02d  %02d  ", sign,_dd,_dm,_ds);
}
 
void toStrDeg(double sec, QString& str) {
	char* sign = "+";
	if(sec < 0) sign = "-";
	sec = fabs(sec);
	int _dd = (int)floor(sec / 3600.0);
	int _dm = (int)floor((sec / 3600.0 - _dd) * 60.0);
	int _ds = (int)sec % 60;
	str.sprintf(" %s%02d  %02d  %02d  ", sign,_dd,_dm,_ds);
}


StarInfo::StarInfo(QString& _hr_,QString& _name_,double _ra_,double _dec_,
	QString& _magV_,double _magBVval_,QString& _sed_,QString& _spectrum_,QString& _duplicity_) {

	_hr = _hr_.copy();
	_name = _name_.copy();
	_ra = _ra_;
	_dec = _dec_;
	_decRad = _dec * RAD_IN_ASEC;
	_magV = _magV_.copy();
	_magVval = _magV.toDouble();
	_magBVval = _magBVval_;
	_sed = _sed_.copy();
	_spectrum = _spectrum_.copy();
	_duplicity = _duplicity_.copy();
	_zenit = -1.0;
	_airMass = -1.0;
	_ha = 0.0;

}

StarInfo::~StarInfo() {
}

void StarInfo::update(double st) {
	_ha  = toValidTimeInterval(st-_ra);
	double sinH = trb_tm::sinLatitude() * sin(_decRad) +
		trb_tm::cosLatitude() * cos(_decRad) * cos(_ha*RAD_IN_TSEC);
	if((sinH >= -1) && (sinH <= 1)) {
		_zenit = toDeg(SEC_IN_90 - asin(sinH) / RAD_IN_ASEC);
	} else _zenit = -1.0;

	double secz1 = 1.0 / sinH;
	double secz2 = secz1 - 1.0;
	_airMass = secz1 - secz2 * (0.0018167 +
			secz2 * (0.002875 + 0.0008083 * secz2));
}

QString& StarInfo::getRA(void) {
	toStrHour(_ra, _ra_str);
	return(_ra_str);
}

QString& StarInfo::getHA(void) {
	toStrHour12(_ha, _ha_str);
	return(_ha_str);
}

QString& StarInfo::getDec(void) {
	toStrDeg(_dec, _dec_str);
	return(_dec_str);
}

QString& StarInfo::getAirMass(void) {
	_airMass_str.setNum(_airMass,'f',2);
	return(_airMass_str);
}

QString& StarInfo::getZenit(void) {
	_zenit_str.setNum(_zenit,'f',2);
	return(_zenit_str);
}

Star::Star(void) {
	_nStars = 0;
	_currentStarNumber = UNDEFINED;
	starList.resize(100);
}

Star::~Star() {
	jdebug("Destructor of Star...");
	int i;

	for(i=0; i<_nStars; i++) {
		delete starList[i];
	}
	jdebug("End of destructor of Star");
}

QString Star::composeWfFileName(QString& sed) {
	QString fileName = sed.lower().prepend(_wfDirName)\
			.append(".").append(wfFileExt);
	return(fileName);
}

QString Star::composeSedFileName(QString& sed) {
	QString	fileName = sed.lower().prepend(_sedDirName)\
			.append(".").append(sedFileExt);
	return(fileName);
}

void Star::loadConst(bool isFirst) {
	jdebug("Star::loadConst   start...");
// To Mass-file:
	QString version;
	leafDataToStr(version, isFirst, cfgFile,"General",
		"Program version", "Version", out_file);
	if(version != VERSION) {
		version = VERSION;
		setLeafData(cfgFile,"General",
			"Program version", "Version",
					version, out_file);
	}
	QString str;
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Program version", "Date", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Program version", "Modified", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Configuration version", "Version", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Configuration version", "Date", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Configuration version", "Modified", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Site", "SiteName", out_file);

// Longitude i Latitude v rabote v trbtime.cpp, no tam yecho rano
// vyvodit' v out-file, t.k. on yechio ne otkryt
	double val;
	leafDataToTimeSec(val,isFirst, cfgFile,"General",
			"Site","Longitude",out_file);
	leafDataToDegSec(val,isFirst, cfgFile,"General",
			"Site","Latitude",out_file);

	leafDataToStr(str, isFirst, cfgFile,"General",
		"Site", "Altitude", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Site", "Location", out_file);
	leafDataToStr(str, isFirst, cfgFile,"General",
		"Site", "Level", out_file);
//	Optic:
	if(isFirst) {
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel A","Inner",out_file);
		_innerDiam[0] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel A","Outer",out_file);
		_outerDiam[0] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel B","Inner",out_file);
		_innerDiam[1] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel B","Outer",out_file);
		_outerDiam[1] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel C","Inner",out_file);
		_innerDiam[2] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel C","Outer",out_file);
		_outerDiam[2] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel D","Inner",out_file);
		_innerDiam[3] = MM2CM * val;
		leafDataToDouble(val, isFirst, deviceCfgFile,"Segmentator","Channel D","Outer",out_file);
		_outerDiam[3] = MM2CM * val;
		leafDataToDouble(_magnification, isFirst, deviceCfgFile,"Optics","Common","Magnification",out_file);

//  WF:
		leafDataToDouble(_minAltitude, isFirst, deviceCfgFile,"WeightFunction","MinAltitude",out_file);
		leafDataToDouble(_maxAltitude, isFirst, deviceCfgFile,"WeightFunction","MaxAltitude",out_file);
		leafDataToDouble(_gridStep, isFirst, deviceCfgFile,"WeightFunction","Step",out_file);
		leafDataToDouble(_minGridStep, isFirst, deviceCfgFile,"WeightFunction","MinStep",out_file);
	}

//	FILES & DIR-s:

	if(isFirst) {
		leafDataToStr(_responseFileName, isFirst, cfgFile,"General",
			"Inputs","Response",out_file);
		_responseFileName.prepend(dataPath);
		testFileName(_responseFileName,1);

		leafDataToStr(_wfDirName, isFirst, cfgFile,"General",
			"Inputs","WeightDir");
		if(_wfDirName.find("/",_wfDirName.length()-1) == -1) {
			_wfDirName.append("/");
		}
		_wfDirName.prepend(dataPath);
		testDirName(_wfDirName,1);

		leafDataToStr(_sedDirName, isFirst, cfgFile,"General",
				"Inputs","SpectraDir");
		if(_sedDirName.find("/",_sedDirName.length()-1) == -1) {
			_sedDirName.append("/");
		}
		_sedDirName.prepend(dataPath);
		{ bool isEmpty;
		const int dialog = 1;
		testDirName(_sedDirName, dialog, &isEmpty);
		if(isEmpty) {
			str_message = _sedDirName + " is empty!";
			showFatalError(str_message);
//			exit(1);
		} }

		leafDataToStr(_starFileName, isFirst, cfgFile,"General",
				"Inputs","StarData");
		_starFileName.prepend(dataPath);

		connect(tree, SIGNAL(changed()),
				this, SLOT(change()));
	}
	jdebug("Star::loadConst   finish");
}

void Star::change(void) {
	loadConst(false);
}

void Star::init(int dialog) {
	jdebug("star init...");
	try {
		loadStarData(1);
	} catch(LoadError err) {
		str_message = err.getMessage();
		if(dialog) showFatalError(str_message);
		else {
			printf("%s\n",str_message.data());
			exit(1);
		}
	}

//	if(dialog) {
//	atm::init(N_CHAN,_magnification, _innerDiam, _outerDiam,
//			_minAltitude,_maxAltitude,_gridStep,_minGridStep,_responseFileName.data());
/*
	fprintf(out_file,"atm::init: %d %f %f %f %f %f\n %f %f %f %f\n %f %f %f %f %s\n",
			N_CHAN,_magnification,
			_innerDiam[0],
			_innerDiam[1],
			_innerDiam[2],
			_innerDiam[3],
			_outerDiam[0],
			_outerDiam[1],
			_outerDiam[2],
			_outerDiam[3],
			_minAltitude,
			_maxAltitude,
			_gridStep,
			_minGridStep,
			_responseFileName.data());			
*/
/*
	if(nr::erget()) {
		if(!dialog) return;
		str_message = nr::ermessage().c_str();
		nr::erreset();
		switch(showWarning(str_message)) {
			case  QMessageBox::Abort : exit(1);
			case  QMessageBox::Ignore: break;
		}
	}
*/
//	}

	QString wfFileName;
	QString sedFileName;
	bool calculate = false;
	for(int i=0; i<_nStars; i++) {
		sedFileName = composeSedFileName(starList[i]->getSED());
		wfFileName = composeWfFileName(starList[i]->getSED());

//		atm::loadsed(sedFileName.data());
		bool calcthis = calculate;
	againlab:
		wf::checalc(sedFileName.data(),
			_responseFileName.data(),
			wfFileName.data(),
			N_CHAN,_magnification, _innerDiam, _outerDiam,
			_minAltitude,_maxAltitude,_gridStep,_minGridStep,
			calcthis, wf::progress);
		if(nr::erget()) {
			if(!dialog) return;
			str_message = nr::ermessage().c_str();
			nr::erreset();
			if(!calcthis) {
				switch(showWarning(str_message, 3,
						"Calculate", "Calculate All", "E&xit")) {
					case  QMessageBox::Retry   : calculate=true;
					case  QMessageBox::Ignore  : calcthis=true;
						goto againlab;
					case  QMessageBox::Abort   : exit(1);
				}
			}
		}
	}
	updateAll();
//	setCurrentStar(searchMinAirMass());
	setCurrentStar(UNDEFINED);
	jdebug("end of star init...");
}

int Star::setCurrentStar(const QString& hr) {
	for(int i=0; i<_nStars; i++) {
		QString listHR = starList[i]->getHR();
		listHR.replace(QRegExp(" "),"");
		if(hr == listHR) {
			return setCurrentStar(i);
		}
	}
	QString str_message = "Invalid HR \"" + hr + "\"";
	showStrongMessage(str_message);
	jdebug("end of setCurrentStar");
	return FAIL;
}

int Star::setCurrentStar(int number) {
	_currentStarNumber = number;
	if(number == UNDEFINED) return FAIL;
	QString sedFileName = composeSedFileName(starList[number]->getSED());
//	atm::loadsed(sedFileName.data());
	_wfFileName = composeWfFileName(starList[number]->getSED());
//	atm::checkwf(_wfFileName.data());
// Uzhe provereno po vsemu spisku, no malo li chto......
	if(nr::erget()) {
		str_message = nr::ermessage().c_str();
		nr::erreset();
		showStrongMessage(str_message);
		return FAIL;
//		exit(1);
	}
	updateCurrent();
	fprintf(out_file,"%s %s %s %s %s %s %s\n",
		getStamp("O ",trb_tm::currentUTDateTime()),
		getHR().data(),
		getName().data(),
		getRA().data(),
		getDec().data(),
		getSED().data(),
		getAirMass().data());
	return SUCCESS;
}

QString Star::invalidFormat(QString& _fileName, int _nRow) {
	str_message.sprintf("\"%s\" Row %d: Invalid format!",
			_fileName.data(), _nRow);
	return(str_message);
}

QString Star::notFound(QString& fileName) {
	str_message.sprintf("\"%s\" not found!",
			fileName.data());
	return(str_message);
}

QString Star::invalidDec(QString& _fileName, int _nRow) {
	str_message.sprintf("\"%s\" Row %d: Invalid Dec!",
			_fileName.data(), _nRow);
	return(str_message);
}

QString Star::invalidRA(QString& _fileName, int _nRow) {
	str_message.sprintf("\"%s\" Row %d: Invalid RA!",
			_fileName.data(), _nRow);
	return(str_message);
}

QString Star::emptyFile(QString& _fileName) {
	str_message.sprintf("\"%s\" is empty!",
			_fileName.data());
	return(str_message);
}

int Star::loadStarData(int dialog) {
	if(testFileName(_starFileName, dialog) != 1) return(0);

	QFile file(_starFileName);
	if(!file.open(IO_ReadOnly)) throw LoadError(notFound(_starFileName));
	QTextStream textStream(&file);
	_nStars = 0;
	int nRow = 0;
	while(!textStream.eof()) {
		str_message = textStream.readLine();
		nRow++;
		if(str_message.isEmpty()) continue;
		if(str_message[0] == '#') continue;

		str_message.replace(QRegExp(","),QString(" "));

		QString hr = str_message.left(4);
		QString name = str_message.mid(4,9) + ' ';

		bool ok = true;
		int h = str_message.mid(13,2).toInt(&ok);
		if(!ok) {
			throw LoadError(invalidFormat(_starFileName,nRow));
		}

		int m = str_message.mid(16,2).toInt(&ok);
		if(!ok) throw LoadError(invalidFormat(_starFileName,nRow));
		int s = str_message.mid(19,2).toInt(&ok);
		if(!ok) throw LoadError(invalidFormat(_starFileName,nRow));

		int sign = 1;
		if(str_message[22] == '-') sign = -1;

		int dd = str_message.mid(24,2).toInt(&ok);
		if(!ok) throw LoadError(invalidFormat(_starFileName,nRow));
		int dm = str_message.mid(27,2).toInt(&ok);
		if(!ok) throw LoadError(invalidFormat(_starFileName,nRow));
		int ds = str_message.mid(30,2).toInt(&ok);
		if(!ok) throw LoadError(invalidFormat(_starFileName,nRow));

		double ra = trb_tm::toSec(h,m,s);
		if((ra < 0) || (ra > SEC_IN_DAY)) throw LoadError(invalidRA(_starFileName,nRow));
		double dec = sign * trb_tm::toSec(dd,dm,ds);
		if(fabs(dec) > SEC_IN_90) throw LoadError(invalidDec(_starFileName,nRow));

		QString magV = str_message.mid(33,5);
		magV.toDouble(&ok);
		if(!ok) {
			throw LoadError(invalidFormat(_starFileName,nRow));
		}
		QString magBV_str = str_message.mid(39,5);
		double magBV = magBV_str.toDouble(&ok);
		if(!ok) {
			throw LoadError(invalidFormat(_starFileName,nRow));
		}
		QString sed = str_message.mid(44,5).stripWhiteSpace();
		QString spectrum = str_message.mid(48,7).stripWhiteSpace();

		QString duplicity = str_message.right(str_message.length()-65).stripWhiteSpace();

		if(_nStars >= (int)starList.size()) {
			starList.resize(starList.size()+10);
		}
		starList[_nStars] = new StarInfo(hr,name,ra,
				dec,magV,magBV,sed,spectrum,duplicity);
		_nStars++;
	}
	file.close();
	if(_nStars <= 0) throw LoadError(emptyFile(_starFileName));
	return(1);
}

int Star::searchMinAirMass(void) {
	int n = 0;
	double minAirMass = 10.0;
	for(int i=0; i<_nStars; i++) {
		double am = starList[i]->getAirMassVal();
		if((am >= 1.0) && (am < minAirMass)) {
			n = i;
			minAirMass = am;
		}
	}
	return(n);
}

void Star::updateAll() {
	double st = trb_tm::calcLocalST();
	for(int i=0; i<_nStars; i++) {
		starList[i]->update(st);
	}
}

void Star::updateCurrent() {
	if(_currentStarNumber == UNDEFINED) return;
	double st = trb_tm::calcLocalST();
	starList[_currentStarNumber]->update(st);
}

QString& Star::getSpectrum(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getSpectrum());
}

QString& Star::getSED(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getSED());
}

QString& Star::getName(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedName);
	return(starList[_currentStarNumber]->getName());
}

QString& Star::getHR(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getHR());
}

QString& Star::getMagV(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getMagV());
}

double Star::getMagVval(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedVal);
	return(starList[_currentStarNumber]->getMagVval());
}

double Star::getMagBVval(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedVal);
	return(starList[_currentStarNumber]->getMagBVval());
}

QString& Star::getDuplicity(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getDuplicity());
}

QString& Star::getRA(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getRA());
}

QString& Star::getHA(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getHA());
}

QString& Star::getDec(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getDec());
}

double Star::getDecVal(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedVal);
	return(starList[_currentStarNumber]->getDecVal());
}

QString& Star::getZenit(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getZenit());
}

double Star::getZenitVal(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedVal);
	return(starList[_currentStarNumber]->getZenitVal());
}

QString& Star::getAirMass(void) {
	if(_currentStarNumber == UNDEFINED) return(Undefined);
	return(starList[_currentStarNumber]->getAirMass());
}

double Star::getAirMassVal(void) {
	if(_currentStarNumber == UNDEFINED) return(UndefinedVal);
	return(starList[_currentStarNumber]->getAirMassVal());
}

bool Star::isStarSelected(void) {
	return(!(_currentStarNumber == UNDEFINED));
}
