/*--------------------------------------------------------------------------*/
/** @file trberror.h
@brief  The declarations of C++ exception classes which may be invoked from 
some points in star.h, device.h etc.

                  MASS project: TURBINA module file
                       header file for trberror.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 1.0
*/
#ifndef TRBERROR_H
#define TRBERROR_H

#include <qstring.h>

#define TRB_EXIT	-13
#define TRB_ABORT	-1
#define TRB_MESSAGE	-2

namespace trberr{
	bool dialog(void);
	void enableDialog(bool enable);
	bool quitProgram(void);
	void setCode(int code);
	void setMessage(const QString messsage);
	const char* getMessage(void);
	void reset(void);
	int getCode(void);
};

class ErrorBase {
	private:
		QString message;
	public:
		ErrorBase(QString msg) { message = msg.copy(); }
		QString getMessage(void) { return message; }
};

class LoadError		: public ErrorBase {
public:
	LoadError(QString msg) : ErrorBase(msg) {}
};

class CFGError		: public ErrorBase {
public:
	CFGError(QString msg) : ErrorBase(msg) {}
};

class DeviceError	: public ErrorBase {
public:
	DeviceError(QString msg) : ErrorBase(msg) {}
};

class SVError	: public ErrorBase {
public:
	SVError(QString msg) : ErrorBase(msg) {}
};

#endif
