#define QT_THREAD_SUPPORT
#include <qthread.h>
#include <math.h>
#include "trbexport.h"
#include "names.h"
#include "success.h"

// -------------- debug ----------------
#include <stdio.h>
// -------------------------------------
QMutex stateExportMutex;
QMutex svExportMutex;

ExportTable::ExportTable( void) {
}

ExportTable::~ExportTable() {
}

void ExportTable::setNum(int row, int col, double val,
			const char* format) {
	QString str;
	int prec = 0;
	double val_abs = fabs(val);
	if(strcmp(format,fluxFormat) == 0) {
		if(val_abs < 1.0) prec = 4;
		else if(val_abs < 10.0) prec = 3;
		else if(val_abs < 100.0) prec = 2;
		str = str.setNum(val,'f',prec);
	} else {
		str.sprintf(format,val);
	}
	if((row < MAX_ROW) && (col < MAX_COL)) {
		table[row][col] = str;
	}
}

void ExportTable::setNum(int row, int col, int val,
			const char* format) {
	QString str;
//	str.sprintf(format, val);
	if((row < MAX_ROW) && (col < MAX_COL)) {
		table[row][col] = str.setNum(val);
	}
}

void ExportTable::cleanRow(int row) {
	if(row >= MAX_ROW) return;
	for(int col=0; col<MAX_COL; col++) {
		setText(row,col,"");
	}
}

void ExportTable::setHHead(int row, const char* col0, const char* col1,
			const char* col2, const char* col3,
			const char* col4, const char* col5,
			const char* col6, const char* col7,
					const char* col8) {
	if(row >= MAX_ROW) return;
	cleanRow(row);
	setText(row,0,col0);
	if(col1) setText(row,1,col1);
	if(col2) setText(row,2,col2);
	if(col3) setText(row,3,col3);
	if(col4) setText(row,4,col4);
	if(col5) setText(row,5,col5);
	if(col6) setText(row,6,col6);
	if(col7) setText(row,7,col7);
	if(col8) setText(row,8,col8);
}

void ExportTable::setText(int row, int col, const char* text) {
	if((row < MAX_ROW) && (col < MAX_COL)) {
		table[row][col] = text;
	}
}

QString& ExportTable::getData(int row, int col) {
	return(table[row][col]);
}

namespace StateExport {
	bool _isApplicationReady = false;
	bool _isStartCounting = false;
	bool _isEndCounting = false;
	bool _isQuitProgram = false;
	bool _isWithDevice = false;
	bool _isWithoutDevice = false;
	bool _isSVconnected = false;
	bool _isSVdisconnected = false;
	bool _isChanged = false;
	DeviceState state;
	bool _isCorrectionReady = false;
	bool _isError = false;

	bool _isUserTrack = false;
	int _trackDirection;
	int _trackCoordinate;

	QString errMessage;
}

void StateExport::startUserTrack(int coordinate, int direction) {
	stateExportMutex.lock();
	_isUserTrack = true;
	_trackCoordinate = coordinate;
	_trackDirection = direction;
	stateExportMutex.unlock();
}

void StateExport::stopUserTrack(void) {
	stateExportMutex.lock();
	_isUserTrack = false;
	stateExportMutex.unlock();
}

bool StateExport::isUserTrack(void) {
	stateExportMutex.lock();
	bool ret = _isUserTrack;
	stateExportMutex.unlock();
	return ret;
}

int StateExport::trackDirection(void) {
	stateExportMutex.lock();
	int ret = _trackDirection;
	stateExportMutex.unlock();
	return ret;
}

int StateExport::trackCoordinate(void) {
	stateExportMutex.lock();
	int ret = _trackCoordinate;
	stateExportMutex.unlock();
	return ret;
}

#ifdef __MOTOR__
int StateExport::aperture(void) {
	stateExportMutex.lock();
	int apert = state.aperture;
	stateExportMutex.unlock();
	return apert;
}

int StateExport::door(void) {
	stateExportMutex.lock();
	int on = state.door;
	stateExportMutex.unlock();
	return on;
}
#endif

int StateExport::hv(void) {
	stateExportMutex.lock();
	int on = state.hv;
	stateExportMutex.unlock();
	return on;
}

#ifdef __COUNTER_BUTTON__
int StateExport::buttons(void) {
	stateExportMutex.lock();
	int on = state.buttons;
	stateExportMutex.unlock();
	return on;
}
#else
int StateExport::mirror(void) {
	stateExportMutex.lock();
	int on = state.mirror;
	stateExportMutex.unlock();
	return on;
}
#endif

int StateExport::illum(void) {
	stateExportMutex.lock();
	int on = state.illum;
	stateExportMutex.unlock();
	return on;
}

int StateExport::light(void) {
	stateExportMutex.lock();
	int on = state.light;
	stateExportMutex.unlock();
	return on;
}

int StateExport::safety(void) {
	stateExportMutex.lock();
	int on = state.safety;
	stateExportMutex.unlock();
	return on;
}

double StateExport::temperature(void) {
	stateExportMutex.lock();
	double t = state.temperature;
	stateExportMutex.unlock();
	return t;
}

int StateExport::brightness(void) {
	stateExportMutex.lock();
	int value = state.brightness;
	stateExportMutex.unlock();
	return value;
}

bool StateExport::error(void) {
	stateExportMutex.lock();
	bool err = _isError;
	stateExportMutex.unlock();
	return err;
}

bool StateExport::correction(void) {
	stateExportMutex.lock();
	bool ready = _isCorrectionReady;
	stateExportMutex.unlock();
	return ready;
}

void StateExport::exportError(QString& mess) {
	stateExportMutex.lock();
	_isError = true;
	errMessage = mess;
	stateExportMutex.unlock();
}

QString StateExport::message(void) {
	stateExportMutex.lock();
	QString mess = errMessage;
	stateExportMutex.unlock();
	return mess;
}

bool StateExport::isStartCounting(void) {
	stateExportMutex.lock();
	bool startC = _isStartCounting;
	stateExportMutex.unlock();
	return startC;
}

bool StateExport::isApplicationReady(void) {
	stateExportMutex.lock();
	bool ready = _isApplicationReady;
	stateExportMutex.unlock();
	return ready;
}

bool StateExport::isQuitProgram(void) {
	stateExportMutex.lock();
	bool quitPr = _isQuitProgram;
	stateExportMutex.unlock();
	return quitPr;
}

bool StateExport::isSVconnected(void) {
	stateExportMutex.lock();
	bool d = _isSVconnected;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isSVdisconnected(void) {
	stateExportMutex.lock();
	bool d = _isSVdisconnected;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isWithDevice(void) {
	stateExportMutex.lock();
	bool d = _isWithDevice;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isWithoutDevice(void) {
	stateExportMutex.lock();
	bool d = _isWithoutDevice;
	stateExportMutex.unlock();
	return d;
}

bool StateExport::isEndCounting(void) {
	stateExportMutex.lock();
	bool endC = _isEndCounting;
	stateExportMutex.unlock();
	return endC;
}

bool StateExport::isChanged(void) {
	stateExportMutex.lock();
	bool chng = _isChanged;
	stateExportMutex.unlock();
	return chng;
}

void StateExport::cleanError(void) {
	stateExportMutex.lock();
	_isError = false;
	stateExportMutex.unlock();
}

void StateExport::setCorrectionReady(bool ready) {
	stateExportMutex.lock();
	_isCorrectionReady = ready;
	stateExportMutex.unlock();
}

void StateExport::cleanCorrectionReady(void) {
	stateExportMutex.lock();
	_isCorrectionReady = false;
	stateExportMutex.unlock();
}

void StateExport::setHV(int on) {
	stateExportMutex.lock();
	state.hv = on;
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::setTemperature(double value) {     
	stateExportMutex.lock();
	state.temperature = value;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setBrightness(int value) {     
	stateExportMutex.lock();
	state.brightness = value;
	_isChanged = true; 
	stateExportMutex.unlock();
}

#ifdef __MOTOR__
void StateExport::setDoor(int open) {     
	stateExportMutex.lock();
	state.door = open;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setAperture(int aperture) {     
	stateExportMutex.lock();
	state.aperture = aperture;
	_isChanged = true; 
	stateExportMutex.unlock();
}
#endif

#ifdef __COUNTER_BUTTON__
void StateExport::setButtons(int on) {     
	stateExportMutex.lock();
	state.buttons = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}
#else
void StateExport::setMirror(int on) {     
	stateExportMutex.lock();
	state.mirror = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}
#endif

void StateExport::setIllum(int on) {     
	stateExportMutex.lock();
	state.illum = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setLight(int on) {     
	stateExportMutex.lock();
	state.light = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setSafety(int on) {     
	stateExportMutex.lock();
	state.safety = on;
	_isChanged = true; 
	stateExportMutex.unlock();
}

void StateExport::setStartCounting(void) {
	stateExportMutex.lock();
	_isEndCounting = false; 
	_isStartCounting = true;
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::cleanStartCounting(void) {
	stateExportMutex.lock();
	_isStartCounting = false;
	stateExportMutex.unlock();
}

void StateExport::setEndCounting(void) {
        stateExportMutex.lock();
        _isStartCounting = false;
        _isEndCounting = true;
		_isChanged = true;
        stateExportMutex.unlock();
}

void StateExport::cleanEndCounting(void) {
	stateExportMutex.lock();
	_isEndCounting = false;
	stateExportMutex.unlock();
}

void StateExport::setApplicationReady(void) {
	stateExportMutex.lock();
	_isApplicationReady = true;
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::cleanApplicationReady(void) {
	stateExportMutex.lock();
	_isApplicationReady = false;
	stateExportMutex.unlock();
}

void StateExport::setQuitProgram(void) {
	stateExportMutex.lock();
	_isQuitProgram = true;
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::setSVconnected(void) {
	stateExportMutex.lock();
	_isSVconnected = true; 
	_isChanged = true;
	InfoExport::withSV = true;
	stateExportMutex.unlock();
}

void StateExport::cleanSVconnected(void) {
	stateExportMutex.lock();
	_isSVconnected = false; 
	stateExportMutex.unlock();
}

void StateExport::setSVdisconnected(void) {
	stateExportMutex.lock();
	_isSVdisconnected = true; 
	_isChanged = true;
	InfoExport::withSV = false;
	stateExportMutex.unlock();
}

void StateExport::cleanSVdisconnected(void) {
	stateExportMutex.lock();
	_isSVdisconnected = false; 
	stateExportMutex.unlock();
}

void StateExport::setWithDevice(void) {
	stateExportMutex.lock();
	_isWithDevice = true; 
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::cleanWithDevice(void) {
	stateExportMutex.lock();
	_isWithDevice = false;
	stateExportMutex.unlock();
}

void StateExport::setWithoutDevice(void) {
	stateExportMutex.lock();
	_isWithoutDevice = true; 
	_isChanged = true;
	stateExportMutex.unlock();
}

void StateExport::cleanWithoutDevice(void) {
	stateExportMutex.lock();
	_isWithoutDevice = false;
	stateExportMutex.unlock();
}

void StateExport::cleanChanged(void) {
	stateExportMutex.lock();
	_isChanged = false;
	stateExportMutex.unlock();
}

const int SV_FIFO_SIZE = 30;

using namespace std;

namespace SVexport {
	string fifo[SV_FIFO_SIZE];
	int iset = 0;
	int iget = 0;
	int fifo_fullness = 0;
}

void SVexport::reset(void) {
	svExportMutex.lock();
	iset = 0;
	iget = 0;
	fifo_fullness = 0;
	svExportMutex.unlock();
}

int SVexport::set(string& instruction) {
	int ret = FAIL;
	svExportMutex.lock();
	if(fifo_fullness < SV_FIFO_SIZE) {
		fifo[iset] = instruction;
		iset++;
		if(iset == SV_FIFO_SIZE) iset = 0;
		fifo_fullness++;
		ret = SUCCESS;
	}
	svExportMutex.unlock();
	return ret;
}

int SVexport::get(string& instruction) {
	int ret = FAIL;
	svExportMutex.lock();
	if(fifo_fullness > 0) {
		instruction = fifo[iget];
		iget++;
		if(iget == SV_FIFO_SIZE) iget = 0;
		fifo_fullness--;
		ret = SUCCESS;
	}
	svExportMutex.unlock();
	return ret;
}
