#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <math.h>		// log10
#include <errno.h>
#include "trbsocket.h"
#include "cfg.h"
#include "trbexport.h"
#include "names.h"
#include "tools.h"		// jdebug()
#include "success.h"	// SUCCESS/FAIL

using namespace std;

namespace trbsocket {
	struct	sockaddr_in name;
	struct	sockaddr_in clientName;
	int		_port = 6001;
	FILE	*sock_file = NULL;
	size_t	size = sizeof(clientName);
	int		mountSocket = -1;
	int		clientSocket = -1;
	bool	socket_cycle_condition = true;
	bool	_connected = false;

	string::size_type	i0 = 0;
	pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
}

void trbsocket::loadConst(bool isFirst) {
	jdebug("trbsocket::loadConst()");
	leafDataToInt(_port, isFirst, cfgFile, "General",
			"Socket", "Port");
}

int trbsocket::init(void) {
	jdebug("trbsocket::init");

	sock_file = fopen("mass.log","a");
	name.sin_family = AF_INET;
//	name.sin_port = htons(MOUNT_PORT);
	name.sin_port = htons(_port);
	name.sin_addr.s_addr = htonl(INADDR_ANY);
	int sock = socket(PF_INET,SOCK_STREAM,0);
	if(sock < 0 ) {
		perror("server's socket");
		return FAIL;
	}
/*	
	int flags = fcntl(sock, F_GETFL, 0);
	fcntl(sock, F_SETFL, flags|O_NONBLOCK);
*/
	if(bind(sock, (struct sockaddr*)&name, sizeof(name)) < 0) {
		perror ("bind");
        return FAIL;
    }
	if(listen(sock, 1) < 0) {
		perror("listen");
		return FAIL;
	}
	mountSocket = sock;
//	atexit(closeSocket);
	jdebug("return from trbsocket::init");
	return SUCCESS;
}

void trbsocket::closeSocket(void) {
	jdebug("trbsocket::closeSocket");
	if(sock_file) fclose(sock_file);
 	if(mountSocket > 0) if(close(mountSocket) < 0) perror("close mountSocket");
	if(clientSocket > 0) if(close(clientSocket) < 0) perror("close clientSocket");
}

int trbsocket::newConnection(void) {
	jdebug("trbsocket::newConnection");

	int newSock = accept(mountSocket, (struct sockaddr *)&clientName, &size);
	if(newSock < 0) {
//		if(errno == EAGAIN) return SUCCESS;
		perror ("accept");
		return FAIL;
	}
/*
	int flags = fcntl(newSock, F_GETFL, 0);
	fcntl(newSock, F_SETFL, flags|O_NONBLOCK);
*/
	clientSocket = newSock;
	if(sock_file) fprintf(sock_file, "Connect from host %s, port %hd.\n",
			inet_ntoa(clientName.sin_addr), ntohs(clientName.sin_port));
	_connected = true;
	StateExport::setSVconnected();
	return SUCCESS;
}

int trbsocket::extractInstruction(int i) {
	i0 = string::size_type(i);
	return 0;
}

int trbsocket::extractInstruction(string from, string& instruction) {
	string::size_type i1 = from.find("\r\n",i0);
	if(i1 == string::npos) return 0;
	instruction = from.substr(i0,i1-i0);
	i0 = i1+2;	// pereskakivaem cherez "\r" i "\n"
	return i1;
}

const int max_length = 80;

int trbsocket::readCommand(void) {
	char str[max_length];
//	int len = recv(clientSocket,str,max_length,0);
	socklen_t fromlen;
	int len = recvfrom(clientSocket,str,max_length,0,
					(struct sockaddr *)&clientName, &fromlen);
	if(len <= 0) {
		if(len < 0) perror("server: recv");
		close(clientSocket);
		clientSocket = -1;
		_connected = false;
		StateExport::setSVdisconnected();
		return FAIL;
	}
	str[len] = '\0';
	string instruction;
	extractInstruction(0);
	while(extractInstruction(str,instruction)) {
		if(sock_file) fprintf(sock_file,"<-%s\n",instruction.c_str());
		SVexport::set(instruction);
	}
	return SUCCESS;
}

int trbsocket::sendToSV(int comid, const char* mess) {
	char num[max_comid_len];
	sprintf(num,"%d ",comid);
	string toSV = num;
	toSV += mess;
	if(sock_file) fprintf(sock_file,"->%s",toSV.c_str());
	if( send(clientSocket,toSV.c_str(),strlen(toSV.c_str()),0) <= 0 ) {
		perror("server: send");
		return FAIL;
	}
	return SUCCESS;
}

void trbsocket::setCycleCondition(bool condition) {
	pthread_mutex_lock(&mutex);
	socket_cycle_condition = condition;
	pthread_mutex_unlock(&mutex);
}

bool trbsocket::getCycleCondition(void) {
	pthread_mutex_lock(&mutex);
	bool condition = socket_cycle_condition;
	pthread_mutex_unlock(&mutex);
	return condition;
}

void trbsocket::quitSocketCycle(void) {
	jdebug("trbsocket::quitSocketCycle");
//	printf("trbsocket::quitSocketCycle\n");
	
	setCycleCondition(false);
}

int trbsocket::socket_cycle(void) {
	jdebug("trbsocket::socket_cycle");
	unsigned int nfds = 1;
	int timeout = 2;
	while(getCycleCondition()) {
		if(!_connected) {
			struct pollfd ufds_mount;
			ufds_mount.fd = mountSocket;
			ufds_mount.events = POLLIN;
			if(poll(&ufds_mount, nfds, timeout)) newConnection();
		} else {
			struct pollfd ufds_client;
			ufds_client.fd = clientSocket;
			ufds_client.events = POLLIN;
			if(poll(&ufds_client, nfds, timeout)) readCommand();
		}
	}
	jdebug("trbsocket::exit socket_cycle");
	closeSocket();
	return 1;
}
