#include <stdlib.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qmessagebox.h>
#include <qdict.h>
#include "tree.h"
#include "access.h"
#include "tools.h"

//	---------------		class Link:		-----------------

void Link::initLink(char id, void* data, CFGBranch* branch) {
	_data = data;
	_id = id;
	_next = NULL;
	branch->addLink(this);
}

void Link::initLink(char id, void* data, CFGTree* tree) {
	_data = data;
	_id = id;
	_next = NULL;
	tree->addLink(this);
}

Link::~Link(void) {
}

const char* Link::BranchName(void) {
	CFGBranch* branch;	
	
	if(_id == 'b') {
		branch = (CFGBranch*)_data;
		return branch->getName();
	}
	return NULL;
}

const char* Link::getLeafKey(void) {
	CFGLeaf* leaf;

	if(_id == 'l') {
		leaf = (CFGLeaf*)_data;
		return leaf->getKey();
	}
	return NULL;
}

char* Link::getLeafData(void) {
	CFGLeaf* leaf;

	if(_id == 'l') {
		leaf = (CFGLeaf*)_data;
		return (char*)(leaf->getData()->data());
	}
	return NULL;
}

//	---------------		class Leaf:		-----------------
void CFGLeaf::initLeaf(const char* key,
			bool for_root,
			bool enable_change,
			bool count_disable) {
	_data = "Empty";
	_changed = true;
	_key = key;
	validator_type = "text";
	_for_root = for_root;
	_enable_change = enable_change;
	_count_disable = count_disable;
}

void CFGLeaf::initLeaf(const char* key,
			QString& validator_type_,
			bool for_root,
			bool enable_change,
			bool count_disable) {
	_data = "Empty";
	_changed = true;
	_key = key;
	validator_type = validator_type_;
	_for_root = for_root;
	_enable_change = enable_change;
	_count_disable = count_disable;
}

CFGLeaf::~CFGLeaf() {
}

int CFGLeaf::replaceData(const QString& data) {
	_data = data;
	return 1;
}

int CFGLeaf::replaceData(const char* data) {
	_data = QString(data);
	return 1;
}

void CFGLeaf::setHelp(const char* help) {
	_help = QString(help);
}

//	---------------		class CFGBranch:		-----------------

void CFGBranch::initBranch(const char* name) {
	_name = name;
	first_link = NULL;
	last_link = NULL;
	current_link = NULL;
	n_leaf = 0;
	n_subbranch = 0;
}

CFGBranch::~CFGBranch(void) {
}

void CFGBranch::addLink(Link* link) {
	if(last_link) {
		last_link->setNext(link);
		last_link = link;
	} else {
		first_link = link;
		last_link = link;
	}
	if(link->getId() == 'l') n_leaf++;
	else if(link->getId() == 'b') n_subbranch++;
	else jdebug("addLink: ERROR!!!");
}

Link* CFGBranch::findBranchLink(const char* branch_name) {
	Link* link;
	link = first_link;

	if(_name == "Empty") return NULL;
	do {
		if(!link) break;
		if(link->getId()=='b') {
			if(!strcasecmp(link->BranchName(),branch_name))
				return link;
		}
		if(!link->getNext()) break;
		link = link->getNext();

	} while(1);
	return NULL;
}

CFGBranch* CFGBranch::findBranch(const char* branch_name) {
	if(_name == "Empty") {
		jdebug("findBranch|%s|: It's Empty Branch!",branch_name);
		return &branch_empty;
	}
	Link* link = findBranchLink(branch_name);
	if(link) return link->getBranch();
	else return &branch_empty;
}

void CFGBranch::setHelp(const char* help) {
	_help = help;
}


CFGLeaf* CFGBranch::findLeaf(const char* leaf_key) {
	Link* link;
	link = first_link;

	if(_name == "Empty") {
		jdebug("findLeaf |%s|: It's Empty Branch!",leaf_key);
		return &leaf_empty;
	}
	do {
		if(!link) break;
		if(link->getId() == 'l') {
			if(!strcasecmp(link->getLeafKey(),leaf_key)) {
				return link->getLeaf();
			}
		}
		if(!link->getNext()) break;
		link = link->getNext();
	} while(1);
	return &leaf_empty;
}

QString* CFGBranch::findLeaf_str(const char* leaf_key) {
	Link* link;
	link = first_link;
	do {
		if(!link) break;
		if(link->getId() == 'l') {
			if(!strcasecmp(link->getLeafKey(),leaf_key)) {
				return link->getLeaf()->getData();
			}
		}
		if(!link->getNext()) break;
		link = link->getNext();
	} while(1);
	return leaf_empty.getData();
}

char* CFGBranch::findLeaf_ch(const char* leaf_key) {
	Link* link;
	link = first_link;
	do {
		if(!link) break;
		if(link->getId() == 'l') {
			if(!strcasecmp(link->getLeafKey(),leaf_key)) {
				return(char*)(link->getLeaf()->getData()->data());
			}
		}
		if(!link->getNext()) break;
		link = link->getNext();
	} while(1);
	return (char*)(leaf_empty.getData()->data());
}

int CFGBranch::replaceLeaf(const char* key, const char* data) {
	CFGLeaf* leaf = findLeaf(key);
	if(leaf == NULL) return 0;
	leaf->replaceData(data);
	return 1;
}

int CFGBranch::replaceChanged(const char* key, bool isChanged) {
	CFGLeaf* leaf = findLeaf(key);
	if(leaf == NULL) return 0;
	leaf->setChanged(isChanged);
	return 1;
}

int CFGBranch::setLeafHelp(const char* key, const char* help) {
	CFGLeaf* leaf = findLeaf(key);
	if(leaf == NULL) return 0;
	leaf->setHelp(help);
	return 1;
}

void CFGBranch::printBranch(CFGBranch* branch) {
	if(!branch) return;
	branch->toFirst();
	while(branch->getCurrentLink()) {
		if(branch->getCurrentLink()->getId() == 'l') {
			printf("    Current Leaf: %c %s: %s\n",branch->getCurrentLink()->getId(),
								branch->getCurrentLink()->getLeafKey(),
								branch->getCurrentLink()->getLeafData());
		} else if(branch->getCurrentLink()->getId() == 'b') {
			CFGBranch* s_branch = branch->getCurrentLink()->getBranch();
			s_branch->toFirst();
			printf("    Current SubBranch: %s: \n",s_branch->getName());
			while(s_branch->getCurrentLink()) {
				if(s_branch->getCurrentLink()->getId() == 'l') {
					printf("		Current Leaf: %c %s: %s\n",s_branch->getCurrentLink()->getId(),
									s_branch->getCurrentLink()->getLeafKey(),
									s_branch->getCurrentLink()->getLeafData());
				}
				s_branch->toNext();
			}
		}
		branch->toNext();
	}
}

// 	---------------		class CFGTree:		-----------------

CFGTree::CFGTree(void) {
	last_link = NULL;
	first_link = NULL;
	current_link = NULL;
	_changed = 0;
}

CFGTree::~CFGTree(void) {
	jdebug("destructor of tree:\n");
	resetTree();
}

void CFGTree::addLink(Link* link) {
	if(last_link) {
		last_link->setNext(link);
		last_link = link;
	} else {
		first_link = link;
		last_link = link;
	}
}

Link* CFGTree::findBranchLink(const char* branch_name) {
	if(branch_name == "Empty") return NULL;
	Link* link;
	link = first_link;
	do {
		if(!link) break;
		if(link->getId()=='b') {
			if(!strcasecmp(link->BranchName(),branch_name))
				return link;
		}
		if(!link->getNext()) break;
		link = link->getNext();

	} while(1);
	return NULL;
}

CFGBranch* CFGTree::findBranch(const char* branch_name) {
	Link* link = findBranchLink(branch_name);
	if(link) return link->getBranch();
	else  return &branch_empty;
}

typedef struct StatusField {
	QString validator_type;
	int		for_root;
	int		enable_change;
	int		count_disable;
};

void readStatusFile(const char* fnstatus, QDict<StatusField>& statDict) {
	QFile st(fnstatus);
	if(!st.open(IO_ReadOnly)) {
		QString text = fnstatus;
		text += " not found!";
		QMessageBox::critical(0,"cfg",text.data()); exit(1);
	}

	QTextStream stream(&st);

	while(!stream.eof()) {
		QString line = stream.readLine();
		line = line.simplifyWhiteSpace();
		int pos1 = line.find("\"",0);
		if(pos1 == -1) continue;
		int pos2 = line.find("\"",pos1+1);
		if(pos2 == -1) continue;
		QString key = line.mid(pos1+1,pos2-pos1-1);

		if( (pos1=line.find("\"",pos2+1)) == -1 ) continue;
		if( (pos2=line.find("\"",pos1+1)) == -1 ) continue;
		QString validator = line.mid(pos1+1,pos2-pos1-1);

		const int nf = 3;
		int attrib[nf] = {-1,-1,-1};
		for(int i=0; i<nf; i++) {
			if( (pos1=line.find(" ",pos2)) == -1 ) continue;
			QString stat;

			if( (pos2=line.find(" ",pos1+1)) == -1 ) {
				stat = line.right(line.length()-pos1-1);
			} else {
				stat = line.mid(pos1+1,pos2-pos1-1);
			}
			bool ok;
			int attr = stat.toInt(&ok);
			if(!ok) continue;
			attrib[i] = attr;
		}

		StatusField* statval = new StatusField();
		statval->validator_type = validator;
		statval->for_root = attrib[0];
		statval->enable_change = attrib[1];
		statval->count_disable = attrib[2];

		if(statDict.size() <= statDict.count()+1) statDict.resize(statDict.size()+100);
		statDict.insert(key,statval);
	}
 	st.close();
}

void CFGTree::printTree(CFGTree* tree) {
	if(!tree) return;
	tree->toFirst();
	while(tree->getCurrentLink()) {
		if(tree->getCurrentLink()->getId() == 'b') {
			printf("Current Section: %s\n",tree->CurrentName());
			CFGBranch* branch = tree->getCurrentLink()->getBranch();
			branch->printBranch(branch);
		}	
		tree->toNext();
	}
	return;
}

int CFGTree::readCfg(const char* fncfg, const char* fnstatus) {

// -------------------- read status-file -------

	QDict<StatusField> statDict(100,false);
	if(fnstatus) readStatusFile(fnstatus,statDict);

// ----------------------- read cfg-file ------------

	CFGTree* tree = this;

	QString text;
	QFile cfgfile(fncfg);
	if(!cfgfile.open(IO_ReadOnly)) {
		text = fncfg;
		text += " not found!";
		QMessageBox::critical(0,"cfg",text.data()); exit(1);
	}

	QTextStream stream(&cfgfile);
	CFGBranch *branch=NULL, *s_branch=NULL;
	CFGBranch* local_branch=NULL;
	QString path;

	while(!stream.eof()) {
		QString line = stream.readLine();
		QString help;
		int pos_help;
		if((pos_help=line.find(";")) != -1) {
			help = line.mid(pos_help+1,line.length())
				.simplifyWhiteSpace();
			line.truncate(pos_help);
		} else help = "";

		line = line.simplifyWhiteSpace();
		if(line[0] == '#') continue;
		int i_spb;//,i_eol;
		if((i_spb=line.find(" ",0)) > -1) {
			int i_eol = line.length();
			QString d_key = line.left(i_spb);
			QString item = line.right(i_eol-i_spb-1);
			if(!strcmp(d_key.upper().data(),"SECTION")) {
				item.remove(0,1); //  
				item.truncate(item.find('\"',1)); //      
		
				branch = tree->findBranch(item.data());
				if(strcasecmp(branch->getName(),"Empty") == 0) { 
					branch = new CFGBranch(item.data(),tree);
				}
				branch->setHelp(help.data());
				local_branch = branch;
				path = branch->getName();
				path += "\\";

			} else if(!strcmp(d_key.upper().data(),"SUBSECTION")) { 
				item.remove(0,1); //  
				item.truncate(item.find('\"',1));  //      

				s_branch = branch->findBranch(item.data());
				if(strcasecmp(s_branch->getName(),"Empty") == 0) {
					s_branch = new CFGBranch(item.data(),branch);
				}
				s_branch->setHelp(help.data());
				local_branch = s_branch;
				path = branch->getName();
				path += "\\";
				path += s_branch->getName();
				path += "\\";

			} else {

				CFGLeaf* leaf = local_branch->findLeaf(d_key.data());
				if(strcasecmp(leaf->getKey(),"Empty") == 0) {
					StatusField *status = statDict[path+d_key]; 
					if(status) {
						if(status->count_disable != -1) {
							leaf = new CFGLeaf(d_key.data(),
									status->validator_type,
									local_branch,
									status->for_root,
									status->enable_change,
									status->count_disable);
						} else if(status->enable_change != -1) {
							leaf = new CFGLeaf(d_key.data(),
									status->validator_type,
									local_branch,
									status->for_root,
									status->enable_change);
						} else if(status->for_root != -1) {
							leaf = new CFGLeaf(d_key.data(),
									status->validator_type,
									local_branch,
									status->for_root);
						} else {
							leaf = new CFGLeaf(d_key.data(),
									status->validator_type,
									local_branch);
						}
					} else {
						leaf = new CFGLeaf(d_key.data(),local_branch);
					}
				}
				leaf->replaceData(item);
				leaf->setHelp(help.data());
			}
		} else {
//			i_eol = line.length();
//			d_key = line.left(i_spb);
//			if(!strcmp(d_key.upper().data(),"ENDSUBSECTION")) local_branch = branch;
		}
	}
	cfgfile.close();

// ---------------------------------

	QDictIterator<StatusField> it(statDict);

	while(it.current()) {
/*
		printf("|%s| -> |%s|.|%d|.|%d|.|%d|\n",
			it.currentKey().data(),
			it.current()->validator_type.data(),
			it.current()->for_root,
			it.current()->enable_change,
			it.current()->count_disable
		);
*/
		delete it.current();
		++it;
	}

// ---------------------------------
//	printTree(tree);
	return 0;
}

void CFGTree::resetTree(void) {
	CFGTree* tree = this;

//	printf("reset tree\n");
	tree->toFirst();
	while(tree->getCurrentLink()) {
		if(tree->getCurrentLink()->getId() == 'b') {
			CFGBranch* branch = tree->getCurrentLink()->getBranch();
			branch->toFirst();
			while(branch->getCurrentLink()) {
				if(branch->getCurrentLink()->getId() == 'l') {
					CFGLeaf* leaf = branch->getCurrentLink()->getLeaf();
					delete leaf;
				} else if(branch->getCurrentLink()->getId() == 'b') {
					CFGBranch* s_branch = branch->getCurrentLink()->getBranch();
					s_branch->toFirst();
					while(s_branch->getCurrentLink()) {
						if(s_branch->getCurrentLink()->getId() == 'l') {
							CFGLeaf* leaf = s_branch->getCurrentLink()->getLeaf();
							delete leaf;
						}
						Link* tmp_link = s_branch->getCurrentLink();
						s_branch->toNext();
						delete tmp_link;
					}
					delete s_branch;
				}
				Link* tmp_link = branch->getCurrentLink();
				branch->toNext();
				delete tmp_link;
			}
			delete branch;
		}	
		Link* tmp_link = tree->getCurrentLink();
		tree->toNext();
		delete tmp_link;
	}
}

int CFGTree::saveToFile(const char* fn) {
	CFGBranch *branch,*s_branch,*s_s_branch;

	QFile f(fn);
	if(!f.open(IO_WriteOnly)) {
		QString text = fn;
		text += " I/O error!";
		QMessageBox::critical(0,"cfg",text.data());
		return(1);
	}

	setAccess(fn,true);

	toFirst();
	while(getCurrentLink()) {
		if(getCurrentLink()->getId() == 'b') {
			branch = getCurrentLink()->getBranch();
			QString s = "Section \"";
			s +=(CurrentName());
			s +=("\"\t;"); 
			s +=(branch->getHelp()->data());
			s +=("\n");
			f.writeBlock(s.data(),s.length());
			branch->toFirst();
			while(branch->getCurrentLink()) {
				if(branch->getCurrentLink()->getId() == 'l') {

					s = "\t";
					s +=(branch->getCurrentLink()->getLeafKey());
					s +=("\t\t\t");
					s +=(branch->getCurrentLink()->getLeafData());
					s +=("\t;"); 
					s +=(branch->getCurrentLink()->getLeaf()->getHelp()->data());
					s +=("\n"); 
					f.writeBlock(s.data(),s.length());

				} else if(branch->getCurrentLink()->getId() == 'b') {
					s_branch = branch->getCurrentLink()->getBranch();
					s_branch->toFirst();

					s = "\tSubSection \"";
					s +=(s_branch->getName());
					s +=("\"\t;"); 
					s +=(s_branch->getHelp()->data());
					s +=("\n");
					f.writeBlock(s.data(),s.length());

					while(s_branch->getCurrentLink()) {
						if(s_branch->getCurrentLink()->getId() == 'l') {
							s = "\t\t";	
							s +=(s_branch->getCurrentLink()->getLeafKey());
							s +=("\t\t\t");
							s +=(s_branch->getCurrentLink()->getLeafData());
							s +=("\t;"); 
							s +=(s_branch->getCurrentLink()->getLeaf()->getHelp()->data());
							s +=("\n"); 
							f.writeBlock(s.data(),s.length());
						} else if(s_branch->getCurrentLink()->getId() == 'b') {
							s_s_branch = s_branch->getCurrentLink()->getBranch();
							s_s_branch->toFirst();

							s = "\t\tSubSubSection \"";
							s +=(s_s_branch->getName());
							s +=("\"\t;"); 
							s +=(s_s_branch->getHelp()->data());
							s +=("\n");
							f.writeBlock(s.data(),s.length());

							while(s_s_branch->getCurrentLink()) {
								if(s_s_branch->getCurrentLink()->getId() == 'l') {
									s = "\t\t\t";	
									s +=(s_s_branch->getCurrentLink()->getLeafKey());
									s +=("\t\t\t");
									s +=(s_s_branch->getCurrentLink()->getLeafData());
									s +=("\t;"); 
									s +=(s_s_branch->getCurrentLink()->getLeaf()->getHelp()->data());
									s +=("\n"); 
									f.writeBlock(s.data(),s.length());
								}
								s_s_branch->toNext();
							}	
							s = "\t\tEndSubSubSection\n";
							f.writeBlock(s.data(),s.length());
						}
						s_branch->toNext();
					}
					s = "\tEndSubSection\n";
					f.writeBlock(s.data(),s.length());
				}
				branch->toNext();
			}
			s = "EndSection\n";
			f.writeBlock(s.data(),s.length());
		}	
		toNext();
	}
	f.close();
	setSaved();
	return(0);
}


void CFGTree::emitSignalChanged(void) {
	emit changed();
}
