/*		MASS Software module parser.cpp: unroll the scenario record into
		a sequence of operations presented as a character array
		
		The code is a remake of the "calculator" from Bierne Straustrup, "Jazyk
		Programmirovanija C++", 3rd edition, Moscow, Binom, 2001, p. 147.
*/
#include "unroll.h"
#include <stdio.h>
#include <iostream>
#include "nrutil.h"

// added 22.09.2003 to access standard C++ type string
using namespace std;

#ifdef UNROLLTEST

int main() {
	const char DICT[] = "ABCDE" ;
	string rec ;
	begi:
	cout << "Enter the scenario record to unroll:\n" ;
	getline(cin,rec) ;
	string res = unroll(rec.c_str(), DICT) ;
	cout << "Result:\n" << res << "\n" ;
	if (nr::erget()) 
		cout << "error=" << nr::ermessage() << '\n' ;
	goto begi;
}

#endif

enum Token_value {
	NAME, NUMBER, END, PLUS='+', MUL='*', LP='(', RP=')', PRINT=';'
} ;

Token_value curr_tok = PRINT ;
long number_value ;
string string_value ;
const char * curc = 0 ;
const char * list = 0 ;

inline long string2long(const string& s) {
	const int base = 10 ;	/* numbers are decimal */
	return strtol(s.c_str(), 0, base) ;
}

string long2string(const long l) {
	char snum[10] ;
	sprintf(snum, "%ld",l) ;
	return string(snum) ;
}

Token_value get_token() {
	char ch = *curc ;
	const int base = 10 ;
//cout << "get_token |"<< curc << "|\n" ;
	switch (ch) {
		case 0: 
			return curr_tok = END ;
		case ';' :
		case '*' :
		case '+' :
		case '(' :
		case ')' :
			curc++ ;
			return curr_tok = Token_value(ch) ;
		case '0': case '1': case '2': case '3': case '4': 
		case '5': case '6': case '7': case '8': case '9': 
			{
			char * endptr = 0 ;
			number_value = strtoul(curc, &endptr, base) ;
			if (endptr) {
				curc = endptr;
				return curr_tok = NUMBER ;
			}
			}
		default:
			if (isspace(ch)) {
				curc++ ;
				return get_token() ;
			}
			if (isalpha(ch) && strchr(list,ch)) {
				char sval[2] = " ";
				sval[0] = ch ;
				string_value = string(sval) ;
				curc++ ;
				return curr_tok = NAME ;
			}
			nr::nrerror("UNROLL: wrong lexem in scenario",nr::ERNOD) ;
			return curr_tok = END ;
	}
}

string expr(bool get) ;

string prim(bool get) {
//cout << "prim():\n" ;
	if (get) get_token() ;
	switch (curr_tok) {
		case NUMBER:
//cout << "prim: NUM\n";
			{
			long numb = number_value ;
			get_token() ;
			return long2string(numb) ;
			}
		case NAME:
			{
//cout << "prim: NAM\n";
			string res = string_value ;
			get_token() ;
			return res ;
			}
		case LP:
//cout << "prim: LP\n";
			{
			string e = expr(true) ;
			if (curr_tok != RP) {
				nr::nrerror("UNROLL: ) expected",nr::ERNOD);
				return "" ;
			}
			get_token() ;
			return e ;
			}
		default:
			nr::nrerror("UNROLL: primary expression expected",nr::ERNOD) ;
			return "" ;
	}
}

string term(bool get) {
//cout << "term():\n" ;
	string left = prim(get) ;
//cout << "term() from prim():"<<left<<" curr_tok="<<curr_tok<<'\n';
	for (;;)
		switch(curr_tok) {
			case MUL:
				{
				string right = prim(true) ;
				/* Multiplication of strings: either two integer numbers or
					one number and one string: */
				long mul1 = string2long(left) ;
				long mul2 = string2long(right) ;
//cout << "left=" << left << " right=" << right << '\n' ;
				if (!mul1 && !mul2) {
					nr::nrerror("UNROLL: modes cannot be multiplied",nr::ERNOD);
					return "" ;
				}
				long prod = mul1*mul2 ;
				if (prod) 
					/* both operands are numbers: return their string repres-n*/
					return long2string(prod) ;

				/* one of operands is a number, other is a modes string */
				if (!mul1) {
					/* left is a string, right is a number: */
					right = left ;
				} else {
					/* right is a string, left is a number: */
					left = right ;
					mul2 = mul1 ;
				}
				/* multiply the string operand a certain number of times: */
				for (int i=1; i<mul2; i++) left += right ;
				break ;
				}
			default:
				return left ;
		}
}

string expr(bool get) {
//cout << "expr():\n" ;
	string left = term(get) ;
	for (;;) {
//cout << "expr left=" << left << " tok=" << char(curr_tok) << '\n';
		switch (curr_tok) {
			case PLUS:
				{
				string right = term(true) ;
				bool isall = strchr(list,left.c_str()[0]) ;
				bool isalr = strchr(list,right.c_str()[0]) ;
//				cout << isall << ' ' << isalr << "===" << (isalr^isall) <<'\n';
				if (!(isall&&isalr)) {
					nr::nrerror("UNROLL: cannot add numbers",nr::ERNOD) ;
					return "";
				}
				left += right ;
				break ;
				}
			case NAME:
				nr::nrerror("UNROLL: cannot accept doubled mode char",nr::ERNOD) ;
				return "";
			default:
				return left ;
		}
	}
}

string result ;

const string& unroll(const char * rec, const char * dict) {
	curc = rec ;
	list = dict ;
	get_token() ;
	result = expr(false) ;
	return result ;
}

